@@ -1,15 +1,16 @@
 // 
-//   Copyright (C) 2005, 2006 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
+// 
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
@@ -24,10 +25,14 @@
 
 
 // TODO: Instead of re-creating AGG fill styles again and again, they should
-// be cached somewhere. 
+// be cached somewhere.
 
 
-using namespace gnash;
+// Enable this DEFINE to limit the alpha value of all colors to 50% at most.
+// This works only with solid and gradient fills (not bitmaps) and is used
+// for debugging hidden characters.
+//#define DEBUG_LIMIT_COLOR_ALPHA 
+
 
 namespace gnash {
 
@@ -39,11 +44,17 @@ class agg_style_base
 {
 public:
   // for solid styles:
-  bool m_is_solid;
+  bool m_is_solid;   
   agg::rgba8 m_color; // defined here for easy access
   
   // for non-solid styles:
   virtual void generate_span(agg::rgba8* span, int x, int y, unsigned len)=0;
+
+  // Everytime a class has a virtual method it should
+  // also have a virtual destructor. This will ensure
+  // that the destructor for the *derived* class is invoked
+  // when deleting a pointer to base class !!
+  virtual ~agg_style_base() {}
 };
 
 
@@ -56,6 +67,10 @@ public:
   agg_style_solid(const agg::rgba8 color) {
     m_is_solid = true;
     m_color = color;
+    
+#ifdef DEBUG_LIMIT_COLOR_ALPHA
+    m_color.a = m_color.a>127 ? 127 : m_color.a;
+#endif    
   }
 
   void generate_span(agg::rgba8* /*span*/, int /*x*/, int /*y*/, unsigned /*len*/)
@@ -75,7 +90,7 @@ public:
   image_accessor_clip_transp(const PixelFormat& pixf)  
   {
     agg::image_accessor_clip<PixelFormat>::image_accessor_clip(pixf, 
-      agg::rgba8_pre(255, 0, 0, 0).premultiply());
+      agg::rgba8_pre(255, 0, 0, 0));
   }
 };
 
@@ -89,7 +104,6 @@ bitmap format is most probably only used for rectangular bitmaps anyway.
 #define image_accessor_clip_transp agg::image_accessor_clone
 
 
-
 /// AGG bitmap fill style. There are quite a few combinations possible and so
 /// the class types are defined outside. The bitmap can be tiled or clipped.
 /// It can have any transformation matrix and color transform. Any pixel format
@@ -99,10 +113,13 @@ template <class PixelFormat, class span_allocator_type, class img_source_type,
 class agg_style_bitmap : public agg_style_base
 {
 public:
+
+  bool m_force_premultiply;
     
   agg_style_bitmap(int width, int height, int rowlen, uint8_t* data, 
     gnash::matrix mat, gnash::cxform cx) :
     
+    m_force_premultiply(false),
     m_rbuf(data, width, height, rowlen),  
     m_pixf(m_rbuf),
     m_img_src(m_pixf),
@@ -121,7 +138,7 @@ public:
       mat.m_[0][0], mat.m_[1][0], 
       mat.m_[0][1], mat.m_[1][1], 
       mat.m_[0][2], mat.m_[1][2]);
-      
+            
     m_cx = cx;
       
   }
@@ -132,13 +149,20 @@ public:
   void generate_span(agg::rgba8* span, int x, int y, unsigned len)
   {
     m_sg.generate(span, x, y, len);
-
     // Apply color transform
     // TODO: Check if this can be optimized
-    if (!m_cx.is_identity())      
-    for (unsigned int i=0; i<len; i++) {
-      m_cx.transform(span->r, span->g, span->b, span->a);
-      ++span;
+    if (!m_cx.is_identity()) {      
+      for (unsigned int i=0; i<len; i++) {
+        m_cx.transform(span->r, span->g, span->b, span->a);
+        span->premultiply();
+        ++span;
+      }
+    } else 
+    if (m_force_premultiply) {
+      for (unsigned int i=0; i<len; i++) { 
+        span->premultiply();
+        span++;
+      }    
     }
   }
     
@@ -196,23 +220,27 @@ public:
       mat.m_[0][1], mat.m_[1][1], 
       mat.m_[0][2], mat.m_[1][2]);
       
-      
-    // Built gradient lookup table
+    m_cx = cx;
+            
+    // Build gradient lookup table
     m_gradient_lut.remove_all(); 
     
     for (int i=0; i<fs.get_color_stop_count(); i++) {
     
       const gradient_record gr = fs.get_color_stop(i); 
+      rgba trans_color = m_cx.transform(gr.m_color);
       
-      m_gradient_lut.add_color(gr.m_ratio/255.0, agg::rgba8(gr.m_color.m_r, 
-        gr.m_color.m_g, gr.m_color.m_b, gr.m_color.m_a));
+#ifdef DEBUG_LIMIT_COLOR_ALPHA
+      trans_color.m_a = trans_color.m_a>127 ? 127 : trans_color.m_a;
+#endif    
+      
+      m_gradient_lut.add_color(gr.m_ratio/255.0, agg::rgba8_pre(trans_color.m_r, 
+        trans_color.m_g, trans_color.m_b, trans_color.m_a));
         
     } // for
     
     m_gradient_lut.build_lut();
     
-    m_cx = cx;
-
   } // agg_style_gradient constructor
 
 
@@ -224,14 +252,6 @@ public:
   void generate_span(color_type* span, int x, int y, unsigned len) 
   {
     m_sg.generate(span, x, y, len);
-
-    // Apply color transform
-    // TODO: Check if this can be optimized
-    if (!m_cx.is_identity())      
-    for (unsigned int i=0; i<len; i++) {
-      m_cx.transform(span->r, span->g, span->b, span->a);
-      ++span;
-    }
   }
   
 private:
@@ -285,7 +305,7 @@ public:
 
     /// Called by AGG to ask if a certain style is a solid color
     bool is_solid(unsigned style) const {
-     
+      assert(style < m_styles.size());
       return m_styles[style]->m_is_solid; 
     }
     
@@ -298,12 +318,12 @@ public:
     /// Adds a new bitmap fill style
     void add_bitmap(agg_bitmap_info_base* bi, gnash::matrix mat, gnash::cxform cx, 
       bool repeat, bool smooth) {
-      
+
       if (bi==NULL) {
-	// See server/styles.h comments about when NULL return is possible.
-	// Don't warn here, we already warn at parse-time
+        // See server/styles.h comments about when NULL return is possible.
+        // Don't warn here, we already warn at parse-time
         //log_msg("WARNING: add_bitmap called with bi=NULL");
-        add_color(agg::rgba8(0,0,0,0));
+        add_color(agg::rgba8_pre(0,0,0,0));
         return;
       }
 
@@ -369,10 +389,10 @@ public:
     
 
     void add_bitmap_repeat_nn_rgb24(agg_bitmap_info_base* bi, gnash::matrix mat, gnash::cxform cx) {
-    
+
       // tiled, nearest neighbor method (faster)   
 
-      typedef agg::pixfmt_rgb24 PixelFormat;
+      typedef agg::pixfmt_rgb24_pre PixelFormat;
       typedef agg::span_allocator<PixelFormat> span_allocator_type;
       typedef agg::wrap_mode_repeat wrap_type;
       typedef agg::image_accessor_wrap<PixelFormat, wrap_type, wrap_type> img_source_type; 
@@ -395,7 +415,7 @@ public:
 
       // clipped, nearest neighbor method (faster)   
 
-      typedef agg::pixfmt_rgb24 PixelFormat;
+      typedef agg::pixfmt_rgb24_pre PixelFormat;
       typedef agg::span_allocator<PixelFormat> span_allocator_type;
       typedef image_accessor_clip_transp<PixelFormat> img_source_type; 
       typedef agg::span_interpolator_linear_subdiv<agg::trans_affine> interpolator_type;
@@ -416,7 +436,7 @@ public:
 
       // tiled, bilinear method (better quality)   
 
-      typedef agg::pixfmt_rgb24 PixelFormat;
+      typedef agg::pixfmt_rgb24_pre PixelFormat;
       typedef agg::span_allocator<PixelFormat> span_allocator_type;
       typedef agg::wrap_mode_repeat wrap_type;
       typedef agg::image_accessor_wrap<PixelFormat, wrap_type, wrap_type> img_source_type; 
@@ -437,7 +457,7 @@ public:
 
       // clipped, bilinear method (better quality)   
 
-      typedef agg::pixfmt_rgb24 PixelFormat;
+      typedef agg::pixfmt_rgb24_pre PixelFormat;
       typedef agg::span_allocator<PixelFormat> span_allocator_type;
       typedef image_accessor_clip_transp<PixelFormat> img_source_type; 
       typedef agg::span_interpolator_linear_subdiv<agg::trans_affine> interpolator_type;
@@ -460,7 +480,7 @@ public:
     
       // tiled, nearest neighbor method (faster)   
 
-      typedef agg::pixfmt_rgba32 PixelFormat;
+      typedef agg::pixfmt_rgba32_pre PixelFormat;
       typedef agg::span_allocator<PixelFormat> span_allocator_type;
       typedef agg::wrap_mode_repeat wrap_type;
       typedef agg::image_accessor_wrap<PixelFormat, wrap_type, wrap_type> img_source_type; 
@@ -471,7 +491,9 @@ public:
         interpolator_type, sg_type> st_type;
       
       st_type* st = new st_type(bi->get_width(), bi->get_height(),
-          bi->get_rowlen(), bi->get_data(), mat, cx);       
+          bi->get_rowlen(), bi->get_data(), mat, cx);
+          
+      st->m_force_premultiply = true; // important for alpha channel       
         
       m_styles.push_back(st);
     }
@@ -483,7 +505,7 @@ public:
 
       // clipped, nearest neighbor method (faster)   
 
-      typedef agg::pixfmt_rgba32 PixelFormat;
+      typedef agg::pixfmt_rgba32_pre PixelFormat;
       typedef agg::span_allocator<PixelFormat> span_allocator_type;
       typedef image_accessor_clip_transp<PixelFormat> img_source_type; 
       typedef agg::span_interpolator_linear_subdiv<agg::trans_affine> interpolator_type;
@@ -493,7 +515,9 @@ public:
         interpolator_type, sg_type> st_type;
       
       st_type* st = new st_type(bi->get_width(), bi->get_height(),
-          bi->get_rowlen(), bi->get_data(), mat, cx);       
+          bi->get_rowlen(), bi->get_data(), mat, cx);
+          
+      st->m_force_premultiply = true; // important for alpha channel       
         
       m_styles.push_back(st);
     }
@@ -504,7 +528,7 @@ public:
 
       // tiled, bilinear method (better quality)   
 
-      typedef agg::pixfmt_rgba32 PixelFormat;
+      typedef agg::pixfmt_rgba32_pre PixelFormat;
       typedef agg::span_allocator<PixelFormat> span_allocator_type;
       typedef agg::wrap_mode_repeat wrap_type;
       typedef agg::image_accessor_wrap<PixelFormat, wrap_type, wrap_type> img_source_type; 
@@ -517,6 +541,8 @@ public:
       st_type* st = new st_type(bi->get_width(), bi->get_height(),
           bi->get_rowlen(), bi->get_data(), mat, cx);       
         
+      st->m_force_premultiply = true; // important for alpha channel       
+
       m_styles.push_back(st);
     }
         
@@ -525,7 +551,7 @@ public:
 
       // clipped, bilinear method (better quality)   
 
-      typedef agg::pixfmt_rgba32 PixelFormat;
+      typedef agg::pixfmt_rgba32_pre PixelFormat;
       typedef agg::span_allocator<PixelFormat> span_allocator_type;
       typedef image_accessor_clip_transp<PixelFormat> img_source_type; 
       typedef agg::span_interpolator_linear_subdiv<agg::trans_affine> interpolator_type;
@@ -537,6 +563,8 @@ public:
       st_type* st = new st_type(bi->get_width(), bi->get_height(),
           bi->get_rowlen(), bi->get_data(), mat, cx);       
         
+      st->m_force_premultiply = true; // important for alpha channel       
+
       m_styles.push_back(st);
     }
     
@@ -551,7 +579,7 @@ public:
       typedef agg::gradient_x gradient_func_type;
       //typedef agg::gradient_repeat_adaptor<gradient_func_type> gradient_adaptor_type;
       typedef gradient_func_type gradient_adaptor_type;
-      typedef agg::gradient_lut<agg::color_interpolator<agg::rgba8>, 256> color_func_type;
+      typedef agg::gradient_lut<agg::color_interpolator<color_type>, 256> color_func_type;
       typedef agg::span_gradient<color_type,
                                  interpolator_type,
                                  gradient_adaptor_type,
@@ -577,7 +605,7 @@ public:
       typedef agg::span_interpolator_linear<agg::trans_affine> interpolator_type;
       typedef agg::gradient_radial gradient_func_type;
       typedef gradient_func_type gradient_adaptor_type;
-      typedef agg::gradient_lut<agg::color_interpolator<agg::rgba8>, 256> color_func_type;
+      typedef agg::gradient_lut<agg::color_interpolator<color_type>, 256> color_func_type;
       typedef agg::span_gradient<color_type,
                                  interpolator_type,
                                  gradient_adaptor_type,
@@ -586,13 +614,13 @@ public:
       typedef agg_style_gradient<color_type, span_allocator_type, 
         interpolator_type, gradient_func_type, gradient_adaptor_type, 
         color_func_type, sg_type> st_type;
-
+      
       // move the center of the radial fill to where it should be
       gnash::matrix transl;
       transl.concatenate_translation(-32.0f, -32.0f);
-      transl.concatenate(mat);
- 	 
-      st_type* st = new st_type(fs, transl, cx, 64/2);  // div 2 because we need radius, not diameter
+      transl.concatenate(mat);    
+      
+      st_type* st = new st_type(fs, transl, cx, 64/2);  // div 2 because we need radius, not diameter     
         
       // NOTE: The value 64 is based on the bitmap texture used by other
       // Gnash renderers which is normally 64x64 pixels for linear gradients.       
@@ -633,12 +661,12 @@ public:
   {
   }
 
-  bool is_solid(unsigned style) const
+  bool is_solid(unsigned /*style*/) const
   {
     return true;
   }
   
-  const agg::gray8& color(unsigned style) const 
+  const agg::gray8& color(unsigned /*style*/) const 
   {
     return m_color;
   }
