@@ -7,8 +7,6 @@
 
 
 #include <cairo/cairo.h>
-#include <cairo/cairo-xlib.h>
-//#include "gnash.h"
 #include "render_handler.h"
 #include "render_handler_tri.h"
 #include "types.h"
@@ -18,15 +16,48 @@
 #include "tu_config.h"
 #include "log.h"
 
-//using namespace gnash;
 
 namespace gnash {
 namespace renderer {
 namespace cairo {
 
-static cairo_t* g_cr_win = 0;
+static cairo_t* g_cr_output = 0;
 static cairo_t* g_cr = 0;
-Window g_cairo_xwin = 0;
+
+
+// Converts from RGB image to 32-bit pixels in CAIRO_FORMAT_RGB24 format
+static void
+rgb_to_cairo_rgb24(uint8_t* dst, const image::rgb* im)
+{
+    uint32_t* dst32 = reinterpret_cast<uint32_t*>(dst);
+    for (int y = 0;  y < im->m_height;  y++)
+    {
+	const uint8_t* src = image::scanline(im, y);
+	for (int x = 0;  x < im->m_width;  x++, src += 3)
+	{
+	    // 32-bit RGB data in native endian format
+	    *dst32++ = (src[0] << 16) | (src[1] << 8) | src[2];
+	}
+    }
+}
+
+
+// Converts from RGBA image to 32-bit pixels in CAIRO_FORMAT_ARGB32 format
+static void
+rgba_to_cairo_argb32(uint8_t* dst, const image::rgba* im)
+{
+    uint32_t* dst32 = reinterpret_cast<uint32_t*>(dst);
+    for (int y = 0;  y < im->m_height;  y++)
+    {
+	const uint8_t* src = image::scanline(im, y);
+	for (int x = 0;  x < im->m_width;  x++, src += 4)
+	{
+	    // 32-bit ARGB data in native endian format
+	    *dst32++ = (src[0] << 24) | (src[1] << 16) | (src[2] << 8) | src[3];
+	}
+    }
+}
+
 
 // bitmap_info_cairo declaration
 class bitmap_info_cairo : public gnash::bitmap_info
@@ -50,15 +81,20 @@ public:
     }
 };
 
+
 class render_handler_cairo : public gnash::triangulating_render_handler
 {
 public:
     // Some renderer state.
-    cairo_t*         m_cr_offscreen;
     cairo_t*         m_cr_mask;
+    cairo_t*    m_cr_dummy;
     int              m_view_width;
     int              m_view_height;
     
+    // Video buffer
+    uint8_t*    m_video_buffer;
+    int	        m_video_bufsize;
+    
     // Enable/disable antialiasing.
     bool	m_enable_antialias;
     
@@ -86,7 +122,7 @@ public:
 	};
 	mode	m_mode;
 	gnash::rgba	m_color;
-	const gnash::bitmap_info*	m_bitmap_info;
+	const bitmap_info_cairo*    m_bitmap_info;
 	gnash::matrix	m_bitmap_matrix;
 	gnash::cxform	m_bitmap_color_transform;
 	bool	m_has_nonzero_bitmap_additive_color;
@@ -101,7 +137,6 @@ public:
 	// Push our style into cairo.
 	void apply(/*const matrix& current_matrix*/) const
 	{
-//	    GNASH_REPORT_FUNCTION;
 	    assert(m_mode != INVALID);
 	    
 	    if (m_mode == COLOR)
@@ -130,8 +165,7 @@ public:
 			    m_bitmap_color_transform.m_[3][0]);
 		    }
 		    
-		    cairo_pattern_t* pattern =
-		       	(static_cast<const bitmap_info_cairo*>(m_bitmap_info))->m_pattern;
+		    cairo_pattern_t* pattern = m_bitmap_info->m_pattern;
 		    
 		    if (m_mode == BITMAP_CLAMP)
 		    {	
@@ -202,11 +236,11 @@ public:
 
 
 	void	disable()  { m_mode = INVALID; }
-	void	set_color(gnash::rgba color)  { m_mode = COLOR; m_color = color; }
+	void	set_color(const gnash::rgba& color)  { m_mode = COLOR; m_color = color; }
 	void	set_bitmap(const gnash::bitmap_info* bi, const gnash::matrix& m, bitmap_wrap_mode wm, const gnash::cxform& color_transform)
 	    {
 		m_mode = (wm == WRAP_REPEAT) ? BITMAP_WRAP : BITMAP_CLAMP;
-		m_bitmap_info = bi;
+		m_bitmap_info = static_cast<const bitmap_info_cairo*>(bi);
 		m_bitmap_matrix = m;
 		m_bitmap_color_transform = color_transform;
 		m_bitmap_color_transform.clamp();
@@ -281,20 +315,25 @@ public:
 
     // Constructor
     render_handler_cairo() :
-	m_cr_offscreen(0), m_cr_mask(0), m_view_width(0), m_view_height(0)
+	m_cr_mask(0), m_view_width(0), m_view_height(0),
+	m_video_buffer(0), m_video_bufsize(0)
     {
+	cairo_surface_t* dummy = cairo_image_surface_create(
+	    CAIRO_FORMAT_A8, 1, 1);
+	m_cr_dummy = cairo_create(dummy);
+	cairo_surface_destroy(dummy);
     }
 
     // Destructor
     ~render_handler_cairo()
     {
+	if (m_video_buffer)  delete [] m_video_buffer;
 	if (m_cr_mask)  cairo_destroy(m_cr_mask);
-	if (m_cr_offscreen)  cairo_destroy(m_cr_offscreen);
-	if (g_cr == m_cr_offscreen)  g_cr = 0;
+	cairo_destroy(m_cr_dummy);
     }
 
     void	begin_display(
-	gnash::rgba background_color,
+	const gnash::rgba& background_color,
 	int /*viewport_x0*/, int /*viewport_y0*/,
 	int viewport_width, int viewport_height,
 	float x0, float x1, float y0, float y1)
@@ -311,86 +350,34 @@ public:
 	// coordinates of the movie that correspond to the viewport
 	// bounds.
 	{
-//	    GNASH_REPORT_FUNCTION;
+	    g_cr = (g_cr_output ? g_cr_output : m_cr_dummy);
+
 	    m_display_width  = fabsf(x1 - x0);
 	    m_display_height = fabsf(y1 - y0);
 	    m_view_width  = viewport_width;
 	    m_view_height = viewport_height;
 
-	    // Destroy offscreen surface if size is different
-	    if (m_cr_offscreen)
-	    {
-		if (m_view_width != viewport_width ||
-		    m_view_height != viewport_height)
-		{
-		    cairo_destroy(m_cr_offscreen);
-		    m_cr_offscreen = 0;
-		}
-	    }
-
-	    // Create offscreen surface if necessary
-	    if (!m_cr_offscreen)
-	    {
-		cairo_surface_t* offscreen = cairo_image_surface_create(
-		    CAIRO_FORMAT_RGB24, viewport_width, viewport_height);
-		m_cr_offscreen = cairo_create(offscreen);
-		cairo_surface_destroy(offscreen);
-		g_cr = m_cr_offscreen;
-	    }
-
 	    cairo_identity_matrix(g_cr);
+	    cairo_rectangle(g_cr, x0, y0, m_display_width, m_display_height);
+	    cairo_clip(g_cr);
 	    cairo_scale(g_cr, viewport_width / m_display_width,
 		viewport_height / m_display_height);
 	    cairo_translate(g_cr, x0, y0);
 
-	    cairo_rectangle(g_cr, x0, y0, m_display_width, m_display_height);
-	    cairo_clip(g_cr);
-
 	    // Clear the background, if background color has alpha > 0.
 	    if (background_color.m_a > 0)
 		{
 		    // Draw a big quad.
 		    apply_color(background_color);
-		    cairo_rectangle(g_cr, x0, y0, x1, y1);
+		    cairo_rectangle(g_cr, x0, y0, x1 - x0, y1 - y0);
 		    cairo_fill(g_cr);
 		}
 	}
 
 
     void	end_display()
-	// Clean up after rendering a frame.  Client program is still
-	// responsible for calling glSwapBuffers() or whatever.
+	// Clean up after rendering a frame.
 	{
-//	    GNASH_REPORT_FUNCTION;
-#if 0
-	    // This creates a new window for output, which may be useful for
-	    // debugging purposes, but we ordinarily want to use an existing one.
-
-	    if (!g_cr_win)
-	    {
-		Display* xdisp = XOpenDisplay(0);
-		Screen*  screen = XDefaultScreenOfDisplay(xdisp);
-		Visual*  visual = DefaultVisual(xdisp, DefaultScreen(xdisp));
-		int      screen_num = XScreenNumberOfScreen(screen);
-		Window   xwin = XCreateWindow(xdisp,
-		    RootWindow(xdisp, screen_num),
-		    0, 0, m_view_width, m_view_height, 0,
-		    CopyFromParent, CopyFromParent, CopyFromParent, 0, 0);
-		XMapWindow(xdisp, xwin);
-		XRaiseWindow(xdisp, xwin);
-		XSync(xdisp, False);
-		cairo_surface_t* surface = cairo_xlib_surface_create(
-		    xdisp, xwin, visual, m_view_width, m_view_height);
-		g_cr_win = cairo_create(surface);
-	    }
-#endif
-
-	    assert(g_cr_win);
-
-	    // Blit offscreen image onto output window 
-	    cairo_surface_t* offscreen = cairo_get_target(m_cr_offscreen);
-	    cairo_set_source_surface(g_cr_win, offscreen, 0, 0);
-	    cairo_paint(g_cr_win);
 	}
 
 
@@ -439,7 +426,7 @@ public:
 	}
 
 
-    void	fill_style_color(int fill_side, gnash::rgba color)
+    void	fill_style_color(int fill_side, const gnash::rgba& color)
 	// Set fill style for the left interior of the shape.  If
 	// enable is false, turn off fill for the left interior.
 	{
@@ -449,7 +436,7 @@ public:
 	}
 
 
-    void	line_style_color(gnash::rgba color)
+    void	line_style_color(const gnash::rgba& color)
 	// Set the line style of the shape.  If enable is false, turn
 	// off lines for following curve segments.
 	{
@@ -475,7 +462,6 @@ public:
 
     void	draw_mesh_strip(const void* coords, int vertex_count)
 	{
-//	    GNASH_REPORT_FUNCTION;
 	    // Set up current style.
 	    m_current_styles[LEFT_STYLE].apply();
 
@@ -518,7 +504,6 @@ public:
     void	draw_line_strip(const void* coords, int vertex_count)
 	// Draw the line strip formed by the sequence of points.
 	{
-//	    GNASH_REPORT_FUNCTION;
 	    // Set up current style.
 	    m_current_styles[LINE_STYLE].apply();
 
@@ -541,14 +526,13 @@ public:
 	const gnash::bitmap_info* binfo,
 	const gnash::rect& coords,
 	const gnash::rect& /*uv_coords*/,
-	gnash::rgba color)
+	const gnash::rgba& color)
 	// Draw a rectangle textured with the given bitmap, with the
 	// given color.	 Apply given transform; ignore any currently
 	// set transforms.
 	//
 	// Intended for textured glyph rendering.
 	{
-//	    GNASH_REPORT_FUNCTION;
             gnash::bitmap_info* nonconst_binfo = const_cast<gnash::bitmap_info*>(binfo);
 	    bitmap_info_cairo* bi = dynamic_cast<bitmap_info_cairo*>(nonconst_binfo);
 	    assert(bi);
@@ -558,9 +542,8 @@ public:
 	    gnash::point a, b, c, d;
 	    m.transform(&a, gnash::point(coords.get_x_min(), coords.get_y_min()));
 	    m.transform(&b, gnash::point(coords.get_x_max(), coords.get_y_min()));
-	    m.transform(&c, gnash::point(coords.get_x_min(), coords.get_y_max()));
-	    d.m_x = b.m_x + c.m_x - a.m_x;
-	    d.m_y = b.m_y + c.m_y - a.m_y;
+	    m.transform(&c, gnash::point(coords.get_x_max(), coords.get_y_max()));
+	    m.transform(&d, gnash::point(coords.get_x_min(), coords.get_y_max()));
 
 	    // FIXME!!! scaling and offset is wrong
 	    cairo_matrix_t mat;
@@ -590,7 +573,6 @@ public:
 	
     void begin_submit_mask()
 	{
-//	    GNASH_REPORT_FUNCTION;
 	    if (m_cr_mask)  cairo_destroy(m_cr_mask);
 	    cairo_surface_t* mask = cairo_image_surface_create(
 		CAIRO_FORMAT_A8, m_view_width, m_view_height);
@@ -603,8 +585,8 @@ public:
 	
     void end_submit_mask()
 	{	     
-	    // Finished the mask.  Now draw to offscreen buffer
-	    g_cr = m_cr_offscreen;
+	    // Finished with the mask.  Now draw to output
+	    g_cr = (g_cr_output ? g_cr_output : m_cr_dummy);
 	}
 	
     void disable_mask()
@@ -614,10 +596,85 @@ public:
 	    {
 		cairo_destroy(m_cr_mask);
 		m_cr_mask = 0;
-		g_cr = m_cr_offscreen;
+
+		// Prepare to draw to output
+		g_cr = (g_cr_output ? g_cr_output : m_cr_dummy);
 	    }
 	}
 	
+ 	// Returns the format the current renderer wants videoframes in.
+	int videoFrameFormat() {
+		return RGB;
+	}
+	
+	/// Draws the video frames
+	void drawVideoFrame(image::image_base* baseframe, const matrix* m, const rect* bounds){
+	    // Obtain on-stage bounding rectangle
+	    gnash::point a, b, c, d;
+	    m->transform(&a, gnash::point(bounds->get_x_min(), bounds->get_y_min()));
+	    m->transform(&b, gnash::point(bounds->get_x_max(), bounds->get_y_min()));
+	    m->transform(&c, gnash::point(bounds->get_x_max(), bounds->get_y_max()));
+	    m->transform(&d, gnash::point(bounds->get_x_min(), bounds->get_y_max()));
+
+	    // Extract frame attributes
+	    image::rgb* frame = static_cast<image::rgb*>(baseframe);
+	    int         w = frame->m_width;
+	    int         h = frame->m_height;
+
+	    // Compute bounding rectangle size relative to video object
+	    double w_scale = bounds->width()  / w;
+	    double h_scale = bounds->height() / h;
+
+	    // Fit video to bounding rectangle
+	    cairo_matrix_t mat;
+	    cairo_matrix_init_scale(&mat, w_scale, h_scale);
+	    cairo_matrix_translate(&mat,
+		bounds->get_x_min(), bounds->get_y_min());
+
+	    // Now apply transformation to video
+	    cairo_matrix_t frame_mat;
+	    cairo_matrix_init(&frame_mat,
+		m->m_[0][0], m->m_[1][0],
+		m->m_[0][1], m->m_[1][1],
+		m->m_[0][2], m->m_[1][2]);
+	    cairo_matrix_multiply(&mat, &mat, &frame_mat);
+
+	    // Inverse the matrix for pattern space
+	    cairo_matrix_invert(&mat);
+
+	    // Convert RGB frame to cairo format
+	    int buf_size = w * h * 4;
+	    if (m_video_bufsize < buf_size)
+	    {
+		if (m_video_buffer)  delete [] m_video_buffer;
+		m_video_buffer  = new unsigned char[buf_size];
+		m_video_bufsize = buf_size;
+	    }
+	    rgb_to_cairo_rgb24(m_video_buffer, frame);
+
+	    // Create a pattern from the the RGB frame
+	    cairo_surface_t* surface = cairo_image_surface_create_for_data(
+		m_video_buffer, CAIRO_FORMAT_RGB24, w, h, w * 4);
+	    cairo_pattern_t* pattern = cairo_pattern_create_for_surface(surface);
+	    cairo_pattern_set_extend(pattern, CAIRO_EXTEND_NONE);
+	    cairo_pattern_set_matrix(pattern, &mat);
+
+	    // Draw the frame now
+	    cairo_save(g_cr);
+	    cairo_set_source(g_cr, pattern);
+	    cairo_move_to(g_cr, a.m_x, a.m_y);
+	    cairo_line_to(g_cr, b.m_x, b.m_y);
+	    cairo_line_to(g_cr, c.m_x, c.m_y);
+	    cairo_line_to(g_cr, d.m_x, d.m_y);
+	    cairo_clip(g_cr);
+	    cairo_paint(g_cr);
+	    cairo_restore(g_cr);
+
+	    // Clean up
+	    cairo_pattern_destroy(pattern);
+	    cairo_surface_destroy(surface);
+	}
+
 };	// end class render_handler_cairo
 
 
@@ -626,7 +683,6 @@ public:
 bitmap_info_cairo::bitmap_info_cairo()
 // Make a placeholder bitmap_info.  Must be filled in later before using.
 {
-//    GNASH_REPORT_FUNCTION;
     m_buffer = 0;
     m_image = 0;
     m_original_width = 0;
@@ -638,7 +694,6 @@ bitmap_info_cairo::bitmap_info_cairo(int width, int height, uint8_t* data)
 // Initialize this bitmap_info to an alpha image
 // containing the specified data (1 byte per texel).
 {
-//    GNASH_REPORT_FUNCTION;
     assert(width > 0);
     assert(height > 0);
     assert(data);
@@ -662,28 +717,14 @@ bitmap_info_cairo::bitmap_info_cairo(int width, int height, uint8_t* data)
 bitmap_info_cairo::bitmap_info_cairo(image::rgb* im)
 // Version of the constructor that takes an RGB image.
 {
-//    GNASH_REPORT_FUNCTION;
     assert(im);
 
-    // Allocate output buffer
+    // Convert 24-bit BGR data to 32-bit RGB
     int buf_size = im->m_width * im->m_height * 4;
     m_buffer = new unsigned char[buf_size];
+    rgb_to_cairo_rgb24(m_buffer, im);
 
-    // Convert 24-bit BGR data to 32-bit RGB
-    unsigned char* dst = m_buffer;
-    for (int y = 0;  y < im->m_height;  y++)
-    {
-	uint8_t* src = image::scanline(im, y);
-	for (int x = 0;  x < im->m_width;  x++, src += 3)
-	{
-	    *dst++ = src[2]; 	// blue
-	    *dst++ = src[1]; 	// green
-	    *dst++ = src[0];	// red
-	    *dst++;		// alpha not used
-	}
-    }
-
-    // Create the image
+    // Create the cairo image
     m_original_width  = im->m_width;
     m_original_height = im->m_height;
     m_image = cairo_image_surface_create_for_data(m_buffer,
@@ -695,26 +736,12 @@ bitmap_info_cairo::bitmap_info_cairo(image::rgb* im)
 bitmap_info_cairo::bitmap_info_cairo(image::rgba* im)
 // Version of the constructor that takes an image with alpha.
 {
-//    GNASH_REPORT_FUNCTION;
     assert(im);
 
     // Allocate output buffer
     int buf_size = im->m_width * im->m_height * 4;
     m_buffer = new unsigned char[buf_size];
-
-    // Convert BGRA data to ARGB
-    unsigned char* dst = m_buffer;
-    for (int y = 0;  y < im->m_height;  y++)
-    {
-	uint8_t* src = image::scanline(im, y);
-	for (int x = 0;  x < im->m_width;  x++, src += 4)
-	{
-	    *dst++ = src[3]; 	// blue
-	    *dst++ = src[2]; 	// green
-	    *dst++ = src[1];	// red
-	    *dst++ = src[0]; 	// alpha
-	}
-    }
+    rgba_to_cairo_argb32(m_buffer, im);
 
     // Create the image
     m_original_width  = im->m_width;
@@ -728,17 +755,13 @@ DSOEXPORT render_handler*
 create_handler()
 // Factory.
 {
-	//    GNASH_REPORT_FUNCTION;
-	//g_cr_win = (cairo_t*) cairohandle;
 	return new render_handler_cairo();
 }
 
 DSOEXPORT void
 set_handle(cairo_t* handle)
 {
-	assert(handle);
-	assert(!g_cr_win);
-	g_cr_win = handle;
+    g_cr_output = handle;
 }
 
 
