@@ -5,6 +5,14 @@
 
 // A render_handler that uses SDL & OpenGL
 
+/* $Id: render_handler_ogl.cpp,v 1.76 2007/07/26 19:25:35 strk Exp $ */
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#include <cstring>
+#include <cmath>
 
 //#include "gnash.h"
 #include "render_handler.h"
@@ -12,13 +20,27 @@
 #include "types.h"
 #include "image.h"
 #include "utility.h"
-#include "tu_opengl_includes.h"
-
-#include <cstring>
-#include <cmath>
 
 #if defined(_WIN32) || defined(WIN32)
-#	include <Windows.h>
+#  include <Windows.h>
+#endif
+
+#if defined(NOT_SGI_GL) || defined(__APPLE_CC__)
+#include <AGL/agl.h>
+#include <OpenGL/gl.h>
+#include <OpenGL/glu.h>
+#include <OpenGL/glext.h>
+#else
+# include <GL/gl.h>
+# ifdef WIN32
+#  define GL_CLAMP_TO_EDGE 0x812F
+# else
+# include <GL/glx.h>
+# endif
+# include <GL/glu.h>
+# ifndef APIENTRY
+#  define APIENTRY
+# endif
 #endif
 
 using namespace gnash;
@@ -30,15 +52,6 @@ using namespace gnash;
 #define RESAMPLE_METHOD 2
 
 
-// Determines whether to generate mipmaps for smoother rendering of
-// minified textures.  It actually tends to look OK to not use
-// mipmaps, though it is potentially a performance hit if you use big
-// textures.
-//
-// TODO: need to code mipmap LOD bias adjustment, to keep mipmaps from
-// looking too blurry.  (Also applies to text rendering.)
-#define GENERATE_MIPMAPS 0
-
 // bitmap_info_ogl declaration
 class bitmap_info_ogl : public gnash::bitmap_info
 {
@@ -57,6 +70,8 @@ public:
 	virtual void layout_image(image::image_base* im);
 };
 
+// static GLint iquad[] = {-1, 1, 1, 1, 1, -1, -1, -1};
+
 class render_handler_ogl : public gnash::triangulating_render_handler
 {
 public:
@@ -261,7 +276,7 @@ public:
 
 
 	void	disable() { m_mode = INVALID; }
-	void	set_color(gnash::rgba color) { m_mode = COLOR; m_color = color; }
+	void	set_color(const gnash::rgba& color) { m_mode = COLOR; m_color = color; }
 	void	set_bitmap(const gnash::bitmap_info* bi, const gnash::matrix& m, bitmap_wrap_mode wm, const gnash::cxform& color_transform)
 	    {
 		m_mode = (wm == WRAP_REPEAT) ? BITMAP_WRAP : BITMAP_CLAMP;
@@ -339,29 +354,110 @@ public:
 	    delete bi;
 	}
 
+#define GLYUV 0
 
-    ~render_handler_ogl()
+ 	// Returns the format the current renderer wants videoframes in.
+	int videoFrameFormat() {
+#if GLYUV
+		return YUV;
+#else
+		return RGB;
+#endif
+	}
+	
+	/// Draws the video frames
+	void drawVideoFrame(image::image_base* baseframe, const matrix* m, const rect* bounds){
+#if GLYUV
+		image::yuv* frame = static_cast<image::yuv*>(baseframe);
+#else
+		image::rgb* frame = static_cast<image::rgb*>(baseframe);
+#endif
+		glPushAttrib(GL_ENABLE_BIT | GL_COLOR_BUFFER_BIT);
+
+#if GLYUV
+		static GLfloat yuv_rgb[16] = {
+			1, 1, 1, 0,
+			0, -0.3946517043589703515f, 2.032110091743119266f, 0,
+			1.139837398373983740f, -0.5805986066674976801f, 0, 0,
+			0, 0, 0, 1
+		};
+#endif
+
+		glMatrixMode(GL_COLOR);
+		glPushMatrix();
+#if GLYUV
+		glLoadMatrixf(yuv_rgb);
+		glPixelTransferf(GL_GREEN_BIAS, -0.5f);
+		glPixelTransferf(GL_BLUE_BIAS, -0.5f);
+#else
+		glLoadIdentity();
+		glPixelTransferf(GL_GREEN_BIAS, 0.0);
+		glPixelTransferf(GL_BLUE_BIAS, 0.0);
+#endif
+		gnash::point a, b, c, d;
+		m->transform(&a, gnash::point(bounds->get_x_min(), bounds->get_y_min()));
+		m->transform(&b, gnash::point(bounds->get_x_max(), bounds->get_y_min()));
+		m->transform(&c, gnash::point(bounds->get_x_min(), bounds->get_y_max()));
+		d.m_x = b.m_x + c.m_x - a.m_x;
+		d.m_y = b.m_y + c.m_y - a.m_y;
+
+		float w_bounds = TWIPS_TO_PIXELS(b.m_x - a.m_x);
+		float h_bounds = TWIPS_TO_PIXELS(c.m_y - a.m_y);
+
+		unsigned char*   ptr = frame->m_data;
+		float xpos = a.m_x < 0 ? 0.0f : a.m_x;	//hack
+		float ypos = a.m_y < 0 ? 0.0f : a.m_y;	//hack
+		glRasterPos2f(xpos, ypos);	//hack
+
+#if GLYUV
+		GLenum rgb[3] = {GL_RED, GL_GREEN, GL_BLUE}; 
+
+		for (int i = 0; i < 3; ++i)
+		{
+			float zx = w_bounds / (float) frame->planes[i].w;
+			float zy = h_bounds / (float) frame->planes[i].h;
+			glPixelZoom(zx, - zy);	// flip & zoom image
+
+			if (i > 0)
+			{
+				glEnable(GL_BLEND);
+				glBlendFunc(GL_ONE, GL_ONE);
+			}
+
+			glDrawPixels(frame->planes[i].w, frame->planes[i].h, rgb[i], GL_UNSIGNED_BYTE, ptr);
+			ptr += frame->planes[i].size;
+		}
+#else
+		int height = frame->m_height;
+		int width = frame->m_width;
+		float zx = w_bounds / (float) width;
+		float zy = h_bounds / (float) height;
+		glPixelZoom(zx,  -zy);	// flip & zoom image
+		glDrawPixels(width, height, GL_RGB, GL_UNSIGNED_BYTE, ptr);
+#endif
+
+		glMatrixMode(GL_COLOR);
+		glPopMatrix();
+
+		glPopAttrib();
+	
+	}
+   
+	// Ctor stub.
+	render_handler_ogl()
 	{
 	}
 
+	// Dtor stub.
+	~render_handler_ogl()
+	{
+	}
 
     void	begin_display(
-	gnash::rgba background_color,
+	const gnash::rgba& background_color,
 	int viewport_x0, int viewport_y0,
 	int viewport_width, int viewport_height,
 	float x0, float x1, float y0, float y1)
-	// Set up to render a full frame from a movie and fills the
-	// background.	Sets up necessary transforms, to scale the
-	// movie to fit within the given dimensions.  Call
-	// end_display() when you're done.
-	//
-	// The rectangle (viewport_x0, viewport_y0, viewport_x0 +
-	// viewport_width, viewport_y0 + viewport_height) defines the
-	// window coordinates taken up by the movie.
-	//
-	// The rectangle (x0, y0, x1, y1) defines the pixel
-	// coordinates of the movie that correspond to the viewport
-	// bounds.
 	{
 //	    GNASH_REPORT_FUNCTION;
 	    
@@ -394,48 +490,65 @@ public:
 		    glEnd();
 		}
 
-	    // Old unused code.  Might get revived someday.
-// #if 0
-// 	    // See if we want to, and can, use multitexture
-// 	    // antialiasing.
-// 	    s_multitexture_antialias = false;
-// 	    if (m_enable_antialias)
-// 		{
-// 		    int	tex_units = 0;
-// 		    glGetIntegerv(GL_MAX_TEXTURE_UNITS_ARB, &tex_units);
-// 		    if (tex_units >= 2)
-// 			{
-// 			    s_multitexture_antialias = true;
-// 			}
+	    // Markus: Implement anti-aliasing here...
+#if 0
+/*
+Code from Timo Kanera...
+if (gl_antialias)
+  {
+    glShadeModel (GL_SMOOTH);
+    glEnable (GL_POLYGON_SMOOTH);
+    glEnable (GL_LINE_SMOOTH);
+    glEnable (GL_POINT_SMOOTH);
+    mp_msg(MSGT_VO, MSGL_INFO, "[sgi] antialiasing on\n");
+}
+else {
+    glShadeModel (GL_FLAT);
+    glDisable (GL_POLYGON_SMOOTH);
+    glDisable (GL_LINE_SMOOTH);
+    glDisable (GL_POINT_SMOOTH);
+    mp_msg(MSGT_VO, MSGL_INFO, "[sgi] antialiasing off\n");
+}
+*/
+ 	    // See if we want to, and can, use multitexture
+ 	    // antialiasing.
+ 	    bool s_multitexture_antialias = false;
+ 	    if (m_enable_antialias)
+ 		{
+ 		    int	tex_units = 0;
+ 		    glGetIntegerv(GL_MAX_TEXTURE_UNITS_ARB, &tex_units);
+ 		    if (tex_units >= 2)
+ 			{
+ 			    s_multitexture_antialias = true;
+ 			}
+ 		    // Make sure we have an edge texture available.
+ 		    if (s_multitexture_antialias == true)
+ 			{
+ 			    // Very simple texture: 2 texels wide, 1 texel high.
+ 			    // Both texels are white; left texel is all clear, right texel is all opaque.
+ 			    unsigned char	edge_data[8] = { 255, 255, 255, 0, 255, 255, 255, 255 };
+			    GLuint s_edge_texture_id = 0;
 
-// 		    // Make sure we have an edge texture available.
-// 		    if (s_multitexture_antialias == true
-// 			&& s_edge_texture_id == 0)
-// 			{
-// 			    // Very simple texture: 2 texels wide, 1 texel high.
-// 			    // Both texels are white; left texel is all clear, right texel is all opaque.
-// 			    unsigned char	edge_data[8] = { 255, 255, 255, 0, 255, 255, 255, 255 };
+ 			    glActiveTextureARB(GL_TEXTURE1_ARB);
+ 			    glEnable(GL_TEXTURE_2D);
+ 			    glGenTextures(1, (GLuint*)&s_edge_texture_id);
+ 			    glBindTexture(GL_TEXTURE_2D, s_edge_texture_id);
 
-// 			    ogl::active_texture(GL_TEXTURE1_ARB);
-// 			    glEnable(GL_TEXTURE_2D);
-// 			    glGenTextures(1, &s_edge_texture_id);
-// 			    glBindTexture(GL_TEXTURE_2D, s_edge_texture_id);
+ 			    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
+			    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
+ 			    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
+ 			    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
 
-// 			    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
-// 			    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
-// 			    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
-// 			    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
+ 			    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 2, 1, 0, GL_RGBA, GL_UNSIGNED_BYTE, edge_data);
 
-// 			    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 2, 1, 0, GL_RGBA, GL_UNSIGNED_BYTE, edge_data);
+ 			    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);	// @@ should we use a 1D texture???
 
-// 			    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);	// @@ should we use a 1D texture???
-
-// 			    glDisable(GL_TEXTURE_2D);
-// 			    ogl::active_texture(GL_TEXTURE0_ARB);
-// 			    glDisable(GL_TEXTURE_2D);
-// 			}
-// 		}
-// #endif // 0
+ 			    glDisable(GL_TEXTURE_2D);
+ 			    glActiveTextureARB(GL_TEXTURE0_ARB);
+ 			    glDisable(GL_TEXTURE_2D);
+ 			}
+ 		}
+#endif // 0
 	}
 
 
@@ -501,7 +614,7 @@ public:
 	}
 
 
-    void	fill_style_color(int fill_side, gnash::rgba color)
+    void	fill_style_color(int fill_side, const gnash::rgba& color)
 	// Set fill style for the left interior of the shape.  If
 	// enable is false, turn off fill for the left interior.
 	{
@@ -511,7 +624,7 @@ public:
 	}
 
 
-    void	line_style_color(gnash::rgba color)
+    void	line_style_color(const gnash::rgba& color)
 	// Set the line style of the shape.  If enable is false, turn
 	// off lines for following curve segments.
 	{
@@ -686,7 +799,7 @@ public:
 	const gnash::bitmap_info* bi,
 	const gnash::rect& coords,
 	const gnash::rect& uv_coords,
-	gnash::rgba color)
+	const gnash::rgba& color)
 	// Draw a rectangle textured with the given bitmap, with the
 	// given color.	 Apply given transform; ignore any currently
 	// set transforms.
@@ -772,9 +885,14 @@ public:
 
 // bitmap_info_ogl implementation
 
+/*
 
+Markus: A. A. I still miss you and the easter 2006, you know.
+	A. J. I miss you too, but you'll probably not read this code ever... :/
 
-void	hardware_resample(int bytes_per_pixel, int src_width, int src_height, uint8* src_data, int dst_width, int dst_height)
+*/
+
+void	hardware_resample(int bytes_per_pixel, int src_width, int src_height, uint8_t* src_data, int dst_width, int dst_height)
 // Code from Alex Streit
 //
 // Sets the current texture to a resampled/expanded version of the
@@ -817,7 +935,7 @@ void	hardware_resample(int bytes_per_pixel, int src_width, int src_height, uint8
 	}
 	glEnd();
 	glCopyTexImage2D(GL_TEXTURE_2D, 0, out_format, 0,0, dst_width, dst_height, 0);
-	delete temp;
+	delete[] temp;
     }
     glPopAttrib();
     glPopMatrix();
@@ -825,36 +943,13 @@ void	hardware_resample(int bytes_per_pixel, int src_width, int src_height, uint8
 }
 
 
-void	generate_mipmaps(unsigned int internal_format, unsigned int input_format, int bytes_per_pixel, image::image_base* im)
-// DESTRUCTIVELY generate mipmaps of the given image.  The image data
-// and width/height of im are munged in this process.
-{
-//    GNASH_REPORT_FUNCTION;
-    int	level = 1;
-    while (im->m_width > 1 || im->m_height > 1)
-	{
-	    if (bytes_per_pixel == 3)
-		{
-		    image::make_next_miplevel((image::rgb*) im);
-		}
-	    else
-		{
-		    image::make_next_miplevel((image::rgba*) im);
-		}
-
-	    glTexImage2D(GL_TEXTURE_2D, level, internal_format, im->m_width, im->m_height, 0,
-			 input_format, GL_UNSIGNED_BYTE, im->m_data);
-	    level++;
-	}
-}
-
 
 void	software_resample(
     int bytes_per_pixel,
     int src_width,
     int src_height,
     int src_pitch,
-    uint8* src_data,
+    uint8_t* src_data,
     int dst_width,
     int dst_height)
 // Code from Alex Streit
@@ -921,19 +1016,6 @@ void	software_resample(
 			    psrc += 3;
 			}
 		}
-
-#ifdef DEBUG_WRITE_TEXTURES_TO_PPM
-	    static int s_image_sequence = 0;
-	    char temp[256];
-	    sprintf(temp, "image%d.ppm", s_image_sequence++);
-	    FILE* f = fopen(temp, "wb");
-	    if (f)
-		{
-		    fprintf(f, "P6\n# test code\n%d %d\n255\n", dst_width, dst_height);
-		    fwrite(rescaled, dst_width * dst_height * 3, 1, f);
-		    fclose(f);
-		}
-#endif
 	}
     else
 	{
@@ -968,12 +1050,6 @@ void	software_resample(
 
     glTexImage2D(GL_TEXTURE_2D, 0, internal_format, dst_width, dst_height, 0, input_format, GL_UNSIGNED_BYTE, rescaled);
 
-#if GENERATE_MIPMAPS
-    // Build mipmaps.
-    image::image_base	im(rescaled, dst_width, dst_height, dst_width * bytes_per_pixel);
-    generate_mipmaps(internal_format, input_format, bytes_per_pixel, &im);
-#endif // GENERATE_MIPMAPS
-
     delete [] rescaled;
 }
 
@@ -1010,11 +1086,7 @@ void bitmap_info_ogl::layout_image(image::image_base* im)
 		case image::image_base::RGB:
 		{
 
-#if GENERATE_MIPMAPS
-			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
-#else
 			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
-#endif
 
 			int	w = 1; while (w < im->m_width) { w <<= 1; }
 			int	h = 1; while (h < im->m_height) { h <<= 1; }
@@ -1051,9 +1123,6 @@ void bitmap_info_ogl::layout_image(image::image_base* im)
 					im, 0, 0, (float) im->m_width, (float) im->m_height);
 
 			glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, w, h, 0, GL_RGB, GL_UNSIGNED_BYTE, rescaled->m_data);
-	#if GENERATE_MIPMAPS
-			generate_mipmaps(GL_RGB, GL_RGB, 3, rescaled);
-	#endif // GENERATE_MIPMAPS
 
 			delete rescaled;
 				}
@@ -1063,9 +1132,6 @@ void bitmap_info_ogl::layout_image(image::image_base* im)
 			{
 				// Use original image directly.
 				glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, w, h, 0, GL_RGB, GL_UNSIGNED_BYTE, im->m_data);
-#if GENERATE_MIPMAPS
-				generate_mipmaps(GL_RGB, GL_RGB, 3, im);
-#endif // GENERATE_MIPMAPS
 			}
 
 			break;
@@ -1073,11 +1139,7 @@ void bitmap_info_ogl::layout_image(image::image_base* im)
 		case image::image_base::RGBA:
 		{
 
-#if GENERATE_MIPMAPS
-			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
-#else
 			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
-#endif
 
 			int	w = 1; while (w < im->m_width) { w <<= 1; }
 			int	h = 1; while (h < im->m_height) { h <<= 1; }
@@ -1114,9 +1176,6 @@ void bitmap_info_ogl::layout_image(image::image_base* im)
 					im, 0, 0, (float) im->m_width, (float) im->m_height);
 
 			glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE, rescaled->m_data);
-#if GENERATE_MIPMAPS
-			generate_mipmaps(GL_RGBA, GL_RGBA, 4, rescaled);
-#endif // GENERATE_MIPMAPS
 
 			delete rescaled;
 				}
@@ -1126,9 +1185,6 @@ void bitmap_info_ogl::layout_image(image::image_base* im)
 			{
 				// Use original image directly.
 				glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE, im->m_data);
-#if GENERATE_MIPMAPS
-				generate_mipmaps(GL_RGBA, GL_RGBA, 4, im);
-#endif // GENERATE_MIPMAPS
 			}
 
 			break;
@@ -1167,11 +1223,12 @@ void bitmap_info_ogl::layout_image(image::image_base* im)
 	}
 
 }
-
 inline bool opengl_accessible()
 {
 #if defined(_WIN32) || defined(WIN32)
 	return wglGetCurrentContext() != 0;
+#elif defined(__APPLE_CC__)
+	return aglGetCurrentContext() != 0;
 #else
 	return glXGetCurrentContext() != 0;
 #endif
@@ -1242,10 +1299,16 @@ gnash::render_handler*	gnash::create_render_handler_ogl()
 
     // Do some initialisation.
 #define OVERSIZE	1.0f
+    
+    //This makes fonts look nice (actually!)
+#if 0
+    glEnable(GL_POLYGON_SMOOTH);
+    glEnable(GL_POINT_SMOOTH);
+#endif    
         // Turn on alpha blending.
         glEnable(GL_BLEND);
         glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
-        
+;        
         // Turn on line smoothing.  Antialiased lines can be used to
         // smooth the outsides of shapes.
         glEnable(GL_LINE_SMOOTH);
