@@ -3,7 +3,7 @@
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
@@ -18,7 +18,7 @@
 // 
 //
 
-/* $Id: render_handler_tri.h,v 1.10 2007/05/28 15:40:57 ann Exp $ */
+/* $Id: render_handler_tri.h,v 1.14 2007/07/18 10:03:04 udog Exp $ */
 
 #ifndef GNASH_RENDER_HANDLER_TRI_H
 #define GNASH_RENDER_HANDLER_TRI_H
@@ -54,20 +54,32 @@ private:
   /// that's still acceptable when viewed.
   /// To draw these meshes they still have to be transformed using the desired
   /// matrix!
-  std::vector <mesh_set*> m_cached_meshes;
+  ///
+  /// This instance owns the mesh_sets
+  ///
+  typedef std::vector <mesh_set*> MeshSetList;
+  MeshSetList m_cached_meshes;
   
 public:  
 
   /// Searches a mesh set with a error tolerance below max_error.
   /// Returns NULL when no candidate could be found.
+  /// Ownership is NOT transferred
   mesh_set* search_candidate(float max_error);
     
   /// Adds a mesh set to the cache. 
+  //
+  /// Transfer ownerhips of the mesh_set.
+  /// TODO: take an auto_ptr
+  ///
   void add(mesh_set* m);
     
   /// Maintain cached meshes. Clean out mesh_sets that haven't been used 
   /// recently, and make sure they're sorted from high error to low error.
   void sort_and_clean_meshes();  
+
+  /// Drop all cached meshes
+  virtual ~tri_cache_manager();
   
 }; // class tri_cache_manager
 
@@ -111,15 +123,15 @@ public:
 
   /// Triangulating render handlers do not need to support this special version
   /// of draw_line_strip().     
-  void	draw_line_strip(const void* coords, int vertex_count, const rgba color);
+  void	draw_line_strip(const void* coords, int vertex_count, const rgba& color);
   
   /// The given polygon is translated to a mesh strip by this class.
-  void  draw_poly(const point* corners, size_t corner_count, const rgba fill, 
-    const rgba outline);
+  void  draw_poly(const point* corners, size_t corner_count, const rgba& fill, 
+    const rgba& outline);
     
   /// The glyph is drawn just like a normal shape character.
   virtual void draw_glyph(shape_character_def *def,
-    const matrix& mat, rgba color, float pixel_scale);
+    const matrix& mat, const rgba& color, float pixel_scale);
     
   /// Older backends always used glyph textures, so any triangulating render
   /// handler activates glyph textures by default.
@@ -127,6 +139,8 @@ public:
 
   /// Translates world coordinates to pixel coordinates
   geometry::Range2d<int> world_to_pixel(const rect& worldbounds);
+  
+  point pixel_to_world(int x, int y);
     
   // --- helper methods --------------------------------------------------------
   // These methods are called by draw_shape_character() and normally don't need
@@ -155,7 +169,7 @@ public:
 	virtual void	fill_style_disable(int fill_side) = 0;
 	
 	/// Sets the fill style to the given solid color. 
-	virtual void	fill_style_color(int fill_side, rgba color) = 0;
+	virtual void	fill_style_color(int fill_side, const rgba& color) = 0;
 	
 	/// Sets the fill style to the given bitmap. This is also used for gradients
 	/// which are transformed to a bitmap prior to this call. 
@@ -166,7 +180,7 @@ public:
 	virtual void	line_style_disable() = 0;
 	
 	/// Sets the stroke color for subsequent draw_line_strip() calls 
-	virtual void	line_style_color(rgba color) = 0;
+	virtual void	line_style_color(const rgba& color) = 0;
 	
 	/// Sets the stroke width for subsequent draw_line_strip() calls. When 
   /// width==1.0 a "hairline" should be drawn. 
