@@ -3,7 +3,7 @@
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
@@ -38,7 +38,9 @@ public:
 
 	NullGui(bool do_loop)
 		:
-		Gui(0,0,do_loop,0)
+		Gui(0,0,do_loop,0),
+		_timeout(0),
+		_quit(false)
 	{}
 
 	~NullGui() {}
@@ -46,8 +48,9 @@ public:
 	{
 		_interval=interval;
 	}
-	void setTimeout(unsigned int)
+	void setTimeout(unsigned int to)
 	{
+		_timeout=to;
 	}
 	bool init(int, char ***) { return true; }
 	bool createWindow(const char* /*title*/, int /*width*/, int /*height*/)
@@ -58,6 +61,16 @@ public:
 	bool createMenu()  { return true; }
 	bool setupEvents()  { return true; }
 	void renderBuffer()  { }
+
+	void quit() { _quit = true; }
+
+private:
+
+	/// timeout value, in milliseconds
+	unsigned int _timeout;
+
+	/// True when we quit
+	bool _quit;
 };
 
 } // end of gnash namespace
