@@ -3,7 +3,7 @@
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
@@ -14,7 +14,6 @@
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-
 // 
 //
 
@@ -33,6 +32,7 @@
 
 #include <string>
 #include <map>
+#include <iostream> // debugging...
 
 // Forward declarations
 namespace gnash
@@ -81,6 +81,21 @@ public:
 	// milliseconds per frame
 	void setDelay(unsigned int d) { delay=d; }
 
+#ifdef GNASH_FPS_DEBUG
+	/// Set the number of seconds between FPS debugging prints
+	//
+	/// @param time
+	///	Number of seconds between FPS debugging prints.
+	///	A value of 0 disables FPS printing.
+	///	A negative value results in an assertion failure.
+	///
+	void setFpsPrintTime(float time)
+	{
+		assert(time >= 0.0);
+		_fpsDebugTime=time;
+	}
+#endif // def GNASH_FPS_DEBUG
+
 	void setWidth(size_t w) { width=w; }
 	size_t getWidth() { return width; }
 
@@ -130,6 +145,12 @@ private:
 
 	void init_gui();
 
+	/// Create the gui instance
+	//
+	/// Uses the USE_<guiname> macros to find out which one
+	///
+	std::auto_ptr<Gui> getGui();
+
 	static void setFlashVars(sprite_instance& m, const std::string& varstr);
 
 	static void fs_callback(sprite_instance* movie,
@@ -178,6 +199,11 @@ private:
 	/// on failure.
 	///
 	movie_definition* load_movie();
+
+#ifdef GNASH_FPS_DEBUG
+	float _fpsDebugTime;
+#endif
+
 };
 
  
