@@ -3,7 +3,7 @@
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
@@ -18,7 +18,7 @@
 // 
 //
 
-/* $Id: aqua_ogl_glue.cpp,v 1.8 2007/05/28 15:40:59 ann Exp $ */
+/* $Id: aqua_ogl_glue.cpp,v 1.15 2007/07/24 00:04:39 nihilus Exp $ */
 
 
 #include "aqua_ogl_glue.h"
@@ -36,6 +36,9 @@ namespace gnash
 {
 
 AquaOglGlue::AquaOglGlue()
+#ifdef FIX_I810_LOD_BIAS
+  : _tex_lod_bias(-1.2f)
+#endif
 {
 //    GNASH_REPORT_FUNCTION;
 }
@@ -46,51 +49,39 @@ AquaOglGlue::~AquaOglGlue()
 
 }
 
-bool AquaOglGlue::init(int, char***)
+
+bool
+#ifdef FIX_I810_LOD_BIAS
+AquaOglGlue::init(int argc, char** argv[])
+#else
+AquaOglGlue::init(int, char***)
+#endif
 {
 //    GNASH_REPORT_FUNCTION;
-
+#ifdef FIX_I810_LOD_BIAS
+    int c = getopt (argc, *argv, "m:");
+    if (c == 'm') {
+      _tex_lod_bias = (float) atof(optarg);
+    }
+#endif
     return true;
 }
 
-
-render_handler* AquaOglGlue::createRenderHandler(int depth)
+render_handler* AquaOglGlue::createRenderHandler()
 {
-//    GNASH_REPORT_FUNCTION;
-
-    _bpp = depth;
+    GNASH_REPORT_FUNCTION;
     render_handler* renderer = create_render_handler_ogl();
-
+#ifdef FIX_I810_LOD_BIAS
+    glTexEnvf(GL_TEXTURE_FILTER_CONTROL_EXT, GL_TEXTURE_LOD_BIAS_EXT, _tex_lod_bias);
+#endif
     return renderer;
 }
 
 bool AquaOglGlue::prepDrawingArea(int width, int height)
 {
-    if (_bpp == 16) {
-      // 16-bit color, surface creation is likely to succeed.
-      /*SDL_GL_SetAttribute(SDL_GL_RED_SIZE, 5);
-      SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, 5);
-      SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, 5);
-      SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 15);
-      SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);
-      SDL_GL_SetAttribute(SDL_GL_STENCIL_SIZE, 1);*/
-    } else {
-      assert(_bpp == 32);
-
-      // 32-bit color etc, for getting dest alpha,
-      // for MULTIPASS_ANTIALIASING (see
-      // render_handler_ogl.cpp).
-      /*SDL_GL_SetAttribute(SDL_GL_RED_SIZE, 8);
-      SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, 8);
-      SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, 8);
-      SDL_GL_SetAttribute(SDL_GL_ALPHA_SIZE, 8);
-      SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 24);
-      SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);
-      SDL_GL_SetAttribute(SDL_GL_STENCIL_SIZE, 1);*/
-    }
-
+	GNASH_REPORT_FUNCTION;
     //SDL_SetVideoMode(width, height, _bpp, sdl_flags | SDL_OPENGL);
-
+#if 0
      // Turn on alpha blending.
     glEnable(GL_BLEND);
     glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
@@ -109,15 +100,18 @@ bool AquaOglGlue::prepDrawingArea(int width, int height)
     // We don't need lighting effects
     glDisable(GL_LIGHTING);
     glPushAttrib (GL_ALL_ATTRIB_BITS);         
-
+#endif
+#ifdef FIX_I810_LOD_BIAS
+    glTexEnvf(GL_TEXTURE_FILTER_CONTROL_EXT, GL_TEXTURE_LOD_BIAS_EXT, _tex_lod_bias);
+#endif
     return true;
 }
 
-void
-AquaOglGlue::render()
+void AquaOglGlue::render()
 {
-//    GNASH_REPORT_FUNCTION;
+    GNASH_REPORT_FUNCTION;
     //SDL_GL_SwapBuffers();
+    aglUpdateContext(aglGetCurrentContext());
 }
 
 } // namespace gnash
\ No newline at end of file
