@@ -3,7 +3,7 @@
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
@@ -26,25 +26,12 @@
 #endif
 
 #include "gnash.h"
-#include "tu_config.h"
+#include "tu_config.h" // for DSOEXPORT
+#include "gtk_glue.h"
 
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
 
-#ifdef RENDERER_OPENGL
-#include <gtk/gtkgl.h>
-#include "gtk_glue_gtkglext.h"
-#elif defined(RENDERER_CAIRO)
-#include <cairo.h>
-#include "gtk_glue_cairo.h"
-#elif defined(RENDERER_AGG)
-#include "gtk_glue_agg.h"
-#endif
-
-#include <gtk/gtk.h>
-
-#include "gui.h"
-
 
 namespace gnash
 {
@@ -66,6 +53,7 @@ class DSOEXPORT GtkGui : public Gui
 
     virtual bool createMenu();
     virtual bool setupEvents();
+    virtual void beforeRendering();
     virtual void renderBuffer();
     virtual void setInterval(unsigned int interval);
     virtual void setTimeout(unsigned int timeout);
@@ -136,10 +124,11 @@ class DSOEXPORT GtkGui : public Gui
 
     GdkPixbuf* create_pixbuf(const gchar *filename);
     
-#ifdef RENDERER_AGG    
+    void rerenderPixels(int xmin, int ymin, int xmax, int ymax);
+    
     void setInvalidatedRegions(const InvalidatedRanges& ranges);
+
     bool want_multiple_regions() { return true; }
-#endif    
 
     virtual void setCursor(gnash_cursor_type newcursor);
     GtkWidget *getWindow() { return _window; };
@@ -152,15 +141,7 @@ class DSOEXPORT GtkGui : public Gui
     GtkWidget   *_vbox;
     std::vector< geometry::Range2d<int> > _drawbounds;
 
-#ifdef RENDERER_CAIRO
-    cairo_t     *_cairo_handle;
-    GtkCairoGlue glue;
-#elif defined(RENDERER_OPENGL)
-    GdkGLConfig *_glconfig;
-    GtkGlExtGlue glue;
-#elif defined(RENDERER_AGG)
-    GtkAggGlue  glue;
-#endif
+    std::auto_ptr<GtkGlue>     _glue;
 
     static gnash::key::code gdk_to_gnash_key(guint key);
     static int gdk_to_gnash_modifier(int state);
