@@ -4,7 +4,7 @@
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
@@ -30,6 +30,10 @@
 #include "movie_root.h"
 #include "VM.h"
 
+#ifdef GNASH_FPS_DEBUG
+#include "tu_timer.h"
+#endif
+
 #include <cstdio>
 #include <cstring>
 
@@ -78,6 +82,12 @@ Gui::Gui() :
     _renderer(NULL),
     _redraw_flag(true),
     _stopped(false)
+#ifdef GNASH_FPS_DEBUG
+    ,fps_counter(0)
+    ,fps_counter_total(0)
+    ,fps_timer(0)
+    ,fps_timer_interval(0.0)
+#endif
 {
 //    GNASH_REPORT_FUNCTION;
 }
@@ -95,6 +105,12 @@ Gui::Gui(unsigned long xid, float scale, bool loop, unsigned int depth)
     _renderer(NULL),
     _redraw_flag(true),
     _stopped(false)
+#ifdef GNASH_FPS_DEBUG
+    ,fps_counter(0)    
+    ,fps_counter_total(0)    
+    ,fps_timer(0)
+    ,fps_timer_interval(0.0)
+#endif        
 {
 }
 
@@ -102,6 +118,9 @@ Gui::~Gui()
 {
 //    GNASH_REPORT_FUNCTION;
     delete _renderer;
+#ifdef GNASH_FPS_DEBUG
+    printf("Total frame advances: %u\n", fps_counter_total);
+#endif
 }
 
 void
@@ -154,7 +173,7 @@ Gui::resize_view(int width, int height)
 	// set new size ?
 	_width = width;
 	_height = height;
-	_validbounds.setTo(0, 0, _width, _height);
+	_validbounds.setTo(0, 0, _width-1, _height-1);
 	//log_msg(_("new size (in twips) is: %dx%d"), _width*20, _height*20); 
 }
 
@@ -396,6 +415,8 @@ Gui::display(movie_root* m)
 		setInvalidatedRegions(changed_ranges);
 #endif
 
+    beforeRendering();
+
 		// render the frame.
 		// It's up to the GUI/renderer combination
 		// to do any clipping, if desired.     
@@ -452,6 +473,10 @@ Gui::advance_movie(Gui* gui)
 //	GNASH_REPORT_FUNCTION;
 
 	gnash::movie_root* m = gnash::get_current_root();
+	
+#ifdef GNASH_FPS_DEBUG
+	gui->fpsCounterTick(); // will be a no-op if fps_timer_interval is zero
+#endif
 
 // Define REVIEW_ALL_FRAMES to have *all* frames
 // consequencially displaied. Useful for debugging.
@@ -488,7 +513,7 @@ Gui::advance_movie(Gui* gui)
 		gnash::sprite_instance* si = m->get_root_movie();
 		if (curframe + 1 >= si->get_frame_count())
 		{
-		    exit(0); // TODO: quit in a more gentile fashion.
+			gui->quit(); 
 		}
 	}
 
@@ -546,6 +571,77 @@ Gui::setInvalidatedRegions(const InvalidatedRanges& ranges)
 	setInvalidatedRegion(bounds);
 }
 
+#ifdef GNASH_FPS_DEBUG
+void 
+Gui::fpsCounterTick()
+{
+  // GNASH_REPORT_FUNCTION;
+
+  // increment this *before* the early return so that
+  // frame count on exit is still valid
+  ++fps_counter_total;
+
+  if ( ! fps_timer_interval )
+  {
+	  return;
+  }
+
+  uint64_t current_timer = tu_timer::get_ticks();
+
+  // TODO: keep fps_timer_interval in milliseconds to avoid the multiplication
+  //       at each fpsCounterTick call...
+  uint64_t interval_ms = (uint64_t)(fps_timer_interval * 1000.0);
+
+  if (fps_counter_total==1) {
+    fps_timer = current_timer;
+    fps_start_timer = current_timer;
+  }
+  
+  ++fps_counter;
+  
+  if (current_timer - fps_timer >= interval_ms) {
+  
+    float secs = (current_timer - fps_timer) / 1000.0;
+    float secs_total = (current_timer - fps_start_timer)/1000.0;
+        
+    float rate = fps_counter/secs;
+    
+    if (secs > 10000000) {
+      // the timers are unsigned, so when the clock runs "backwards" it leads
+      // to a very high difference value. In theory, this should never happen
+      // with ticks, but it does on my machine (which may have a hw problem?).
+      printf("Time glich detected, need to restart FPS counters, sorry...\n");
+      
+      fps_timer = current_timer;
+      fps_start_timer = current_timer;
+      fps_counter_total = 0;
+      fps_counter = 0;
+      return;
+    } 
+     
+    // first FPS message?
+    if (fps_timer == fps_start_timer) {     // they're ints, so we can compare
+      fps_rate_min = rate;
+      fps_rate_max = rate; 
+    } else {
+      fps_rate_min = fmin(fps_rate_min, rate);
+      fps_rate_max = fmax(fps_rate_max, rate);
+    }
+    
+    float avg = fps_counter_total / secs_total; 
+  
+    //log_msg("Effective frame rate: %0.2f fps", (float)(fps_counter/secs));
+    printf("Effective frame rate: %0.2f fps (min %0.2f, avg %0.2f, max %0.2f, "
+      "%u frames in %0.1f secs total)\n", rate, fps_rate_min, avg, fps_rate_max,
+      fps_counter_total, secs_total);
+      
+    fps_counter = 0;
+    fps_timer = current_timer;
+    
+  }
+   
+}
+#endif
 
 // end of namespace
 }
