@@ -4,7 +4,7 @@
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
@@ -22,12 +22,15 @@
 #endif
 
 
-#include <qgl.h>
-#include <qtimer.h>
 #include <qwidget.h>
 #include <qmessagebox.h>
 #include <qcursor.h>
 #include <qxembed.h>
+#include <qnamespace.h>
+#include <qtimer.h>
+#include <qeventloop.h>
+
+#include "Range2d.h"
 
 #include "gnash.h"
 #include "movie_definition.h" 
@@ -37,9 +40,6 @@
 #include "kdesup.h"
 #include "klash.moc"
 
-//#include <cstdio>
-#include <X11/keysym.h>
-
 using namespace std;
 
 namespace gnash 
@@ -48,49 +48,52 @@ namespace gnash
 KdeGui::~KdeGui()
 {
 //    GNASH_REPORT_FUNCTION;
-    delete _qwidget;
 }
 
 
 KdeGui::KdeGui(unsigned long xid, float scale, bool loop, unsigned int depth)
  : Gui(xid, scale, loop, depth)
 {
-    GNASH_REPORT_FUNCTION;
-
-
 }
 
 bool
 KdeGui::init(int argc, char **argv[])
 {
-    _qapp = new QApplication(argc, *argv);
-    _qwidget = new qwidget(this); 
+//    GNASH_REPORT_FUNCTION;
+    _qapp.reset(new QApplication(argc, *argv));
+    _qwidget.reset(new qwidget(this)); 
     if (_xid) {
         QXEmbed::initialize();
-        QXEmbed::embedClientIntoWindow(_qwidget, _xid);
+        QXEmbed::embedClientIntoWindow(_qwidget.get(), _xid);
     }
 
-//    GNASH_REPORT_FUNCTION;
     _glue.init (argc, argv);
     
     return true;
 }
 
 bool
-KdeGui::createWindow(const char* /*windowtitle*/, int width, int height)
+KdeGui::createWindow(const char* windowtitle, int width, int height)
 {
-    GNASH_REPORT_FUNCTION;
+//    GNASH_REPORT_FUNCTION;
 
-    _qwidget->makeCurrent();
     _qwidget->setGeometry(0, 0, width, height);
-    _qapp->setMainWidget(_qwidget);
+    _qwidget->setCaption(windowtitle);
+
+    _qapp->setMainWidget(_qwidget.get());
     _qwidget->show();
 
-    _glue.prepDrawingArea(_qwidget);
+    _glue.prepDrawingArea(_qwidget.get());
+    _renderer = _glue.createRenderHandler();
+    if ( ! _renderer )
+    {
+        // something went wrong
+        return false;
+    }
+    _glue.initBuffer(width, height);
     
     _width = width;
     _height = height;
-    _renderer = _glue.createRenderHandler();
     set_render_handler(_renderer);
     
     return true;
@@ -103,17 +106,24 @@ KdeGui::renderBuffer()
     _glue.render();
 }
 
+
+void
+KdeGui::setInvalidatedRegions(const InvalidatedRanges& ranges)
+{
+    _glue.setInvalidatedRegions(ranges);
+}
+
 void
 KdeGui::setTimeout(unsigned int timeout)
 {
 //    GNASH_REPORT_FUNCTION;
-//    _timeout = timeout;
+    QTimer::singleShot(timeout, _qapp.get(), SLOT(quit()));
 }
 
 void
 KdeGui::setInterval(unsigned int interval)
 {
-    GNASH_REPORT_FUNCTION;
+//    GNASH_REPORT_FUNCTION;
     _qwidget->setInterval(interval);
     
 }
@@ -121,9 +131,7 @@ KdeGui::setInterval(unsigned int interval)
 bool
 KdeGui::run()
 {
-    GNASH_REPORT_FUNCTION;
- //   _qwidget->connect(&_qapp, SIGNAL(lastWindowClosed()), &_qapp, SLOT(quit()));
-    
+//    GNASH_REPORT_FUNCTION;
     _qapp->exec();
 
     return true;
@@ -241,6 +249,19 @@ KdeGui::handleKeyEvent(QKeyEvent *event, bool down)
     notify_key_event(c, mod, down);
 }
 
+void
+KdeGui::resize(int width, int height)
+{
+    _glue.resize(width, height);
+    resize_view(width, height);
+}
+
+void
+KdeGui::quit()
+{
+    _qapp->eventLoop()->exit();
+}
+
 
 /// \brief restart the movie from the beginning
 void
@@ -255,12 +276,7 @@ void
 qwidget::menuitem_quit_callback()
 {
 //    GNASH_REPORT_FUNCTION;
-#if 0
-    _qapp->closeAllWindows();
-    _qapp->quit();
- #endif
-
-    exit(0);
+    _godfather->quit();
 }
 
 /// \brief Start the movie playing from the current frame.
@@ -328,15 +344,17 @@ qwidget::menuitem_jump_backward_callback()
 void
 qwidget::mouseMoveEvent(QMouseEvent *event)
 {
-    GNASH_REPORT_FUNCTION;
+//    GNASH_REPORT_FUNCTION;
     assert(_godfather);
     QPoint position = event->pos();
 
-    _godfather->notify_mouse_moved(position.x(), position.y());
+    int newX = static_cast<int> (position.x() / _godfather->getXScale());
+    int newY = static_cast<int> (position.y() / _godfather->getYScale());
+
+    _godfather->notify_mouse_moved(newX, newY);
 }
 
 qwidget::qwidget(KdeGui* godfather)
-  : QGLWidget(0, "hi")
 {
     _qmenu.insertItem(_("Play Movie"), this, SLOT(menuitem_play_callback()));
     _qmenu.insertItem(_("Pause Movie"), this, SLOT(menuitem_pause_callback()));
@@ -374,13 +392,13 @@ qwidget::contextMenuEvent(QContextMenuEvent*)
 }
 
 void
-qwidget::mousePressEvent(QMouseEvent *event)
+qwidget::mousePressEvent(QMouseEvent* /* event */)
 {
     _godfather->notify_mouse_clicked(true, 1);
 }
 
 void
-qwidget::mouseReleaseEvent(QMouseEvent *event)
+qwidget::mouseReleaseEvent(QMouseEvent* /* event */)
 {
     _godfather->notify_mouse_clicked(false, 1);
 }
@@ -400,8 +418,27 @@ qwidget::keyReleaseEvent(QKeyEvent *event)
 void
 qwidget::resizeEvent(QResizeEvent *event)
 {
-    _godfather->resize_view(int(event->size().width()), int(event->size().height()));
+    _godfather->resize(event->size().width(), event->size().height());
+}
+
+void 
+qwidget::paintEvent(QPaintEvent *event)
+{
+    const QRegion& region = event->region();
+    QRect rect = region.boundingRect();
+
+    int xmin = static_cast<int> (PIXELS_TO_TWIPS(rect.x()-1)),
+        ymin = static_cast<int> (PIXELS_TO_TWIPS(rect.y()-1)),
+        xmax = static_cast<int> (PIXELS_TO_TWIPS(rect.right()+1)),
+        ymax = static_cast<int> (PIXELS_TO_TWIPS(rect.bottom()+1));
+
+    geometry::Range2d<int> range(xmin, ymin, xmax, ymax);
+    InvalidatedRanges ranges;
+    ranges.add(range);
+
     
+    _godfather->setInvalidatedRegions(ranges);
+    _godfather->renderBuffer();
 }
 
 // end of namespace gnash
