@@ -3,7 +3,7 @@
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
@@ -19,11 +19,12 @@
 //
 
 
-/* $Id: kde_glue.h,v 1.11 2007/05/28 15:41:00 ann Exp $ */
+/* $Id: kde_glue.h,v 1.13 2007/07/01 10:54:03 bjacques Exp $ */
 
 #include "gnash.h"
 
 #include <qwidget.h>
+#include "snappingrange.h"
 
 namespace gnash
 {
@@ -31,12 +32,16 @@ namespace gnash
 class KdeGlue
 {
   public:
-    virtual ~KdeGlue() { };
+    KdeGlue() : _drawing_area(NULL) {}
+    virtual ~KdeGlue() { }
     virtual bool init(int argc, char **argv[]) = 0;
 
     virtual void prepDrawingArea(QWidget *drawing_area) = 0;
     virtual render_handler* createRenderHandler() = 0;
     virtual void render() = 0;
+    virtual void setInvalidatedRegions(const InvalidatedRanges& /* ranges */) {}
+    virtual void resize(int, int) {}
+    virtual void initBuffer(int, int) {}
   protected:
     QWidget     *_drawing_area;
 };
