@@ -1,23 +1,20 @@
 // 
-//   Copyright (C) 2005, 2006 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
+// 
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-// 
-//
-//
-
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -28,7 +25,7 @@
 //#include "rc.h"
 
 #include <string>
-//#include <cstring>
+#include <cstring>
 #include <vector>
 #include <cassert>
 #include <sstream>
@@ -39,7 +36,6 @@
 // these are for stat(2)
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <sys/param.h>
 
 #if defined(_WIN32) || defined(WIN32)
 # define PATH_MAX 255
@@ -48,6 +44,7 @@
 # include <direct.h>
 #else
 # include <unistd.h>
+#	include <sys/param.h>
 #endif
 
 #include <climits>
@@ -58,7 +55,7 @@ namespace gnash {
 
 /*private*/
 void
-URL::init_absolute(const string& in)
+URL::init_absolute(const std::string& in)
 {
 	// Find protocol
 	string::size_type pos = in.find("://");
@@ -106,7 +103,7 @@ URL::init_absolute(const string& in)
 }
 
 /*public*/
-URL::URL(const string& absolute_url)
+URL::URL(const std::string& absolute_url)
 {
 	//cerr << "URL(" << absolute_url << ")" << endl;
 	if ( ( absolute_url.size() && absolute_url[0] == '/' )
@@ -148,7 +145,7 @@ public:
 /*private*/
 // only for UNIX
 void
-URL::normalize_path(string& path)
+URL::normalize_path(std::string& path)
 {
 
 #if defined(_WIN32) || defined(WIN32) 
@@ -174,7 +171,8 @@ URL::normalize_path(string& path)
 			prev = curr;
 
 			if ( comp == "" || comp == "." ) continue;
-			if ( comp == ".." ) components.pop_back();
+			if ( comp == ".." && components.size() )
+				 components.pop_back();
 			else components.push_back(comp);
 		}
 	}
@@ -195,14 +193,14 @@ URL::normalize_path(string& path)
 }
 
 /*public*/
-URL::URL(const string& relative_url, const URL& baseurl)
+URL::URL(const std::string& relative_url, const URL& baseurl)
 {
 	init_relative(relative_url, baseurl);
 }
 
 /*private*/
 void
-URL::init_relative(const string& relative_url, const URL& baseurl)
+URL::init_relative(const std::string& relative_url, const URL& baseurl)
 {
 	// If relative url starts with an hash, it's just
 	// an anchor change
@@ -392,6 +390,8 @@ URL::parse_querystring(const std::string& query_string,
 
 		string name = query_string.substr(start, eq-start);
 		string value = query_string.substr(eq+1, amp-(eq+1));
+		decode(name);
+		decode(value);
 
 		target_map[name] = value;
 
@@ -400,6 +400,63 @@ URL::parse_querystring(const std::string& query_string,
 	}
 }
 
+/* public static */
+void
+URL::encode(std::string& input)
+{
+	const string escapees = " \"#$%&+,/:;<=>?@[\\]^`{|}~";
+	const string hexdigits = "0123456789ABCDEF";
+
+	for (unsigned int i=0;i<input.length(); i++)
+	{
+		unsigned c = input[i] & 0xFF;	// ensure value is 0-255 not -ve
+
+		if (c < 32 || c > 126 || escapees.find((char)c) != string::npos)
+		{
+			input[i] = '%';
+			input.insert(++i, hexdigits.substr(c >> 4, 1));
+			input.insert(++i, hexdigits.substr(c & 0xF, 1));
+	    	}
+		else if ( c == ' ' )
+		{
+			input[i] = '+';
+		}
+	}
+}
+
+/* public static */
+void
+URL::decode(std::string& input)
+{
+	int hexcode;
+
+	for (unsigned int i=0; i<input.length(); i++)
+	{
+		if (input[i] == '%' && (input.length() > i + 2) &&
+			isxdigit(input[i+1]) && isxdigit(input[i+2]))
+		{
+			input[i+1] = toupper(input[i+1]);
+			input[i+2] = toupper(input[i+2]);
+			if (isdigit(input[i+1]))
+				hexcode = (input[i+1] - '0') * 16;
+			else
+				hexcode = (input[i+1] - 'A' + 10) * 16;
+
+			if (isdigit(input[i+2]))
+				hexcode += (input[i+2] - '0');
+			else
+				hexcode += (input[i+2] - 'A' + 10);
+
+			input[i] = (char)hexcode;
+			input.erase(i+1, 2);
+		}
+		else if ( input[i] == '+' )
+		{
+			input[i] = ' ';
+		}
+	}
+}
+
 ostream& operator<< (ostream& o, const URL& u)
 {
 	return o << u.str();
