@@ -4,7 +4,7 @@
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
@@ -17,7 +17,7 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-/* $Id: curl_adapter.cpp,v 1.36 2007/05/24 22:27:09 tgc Exp $ */
+/* $Id: curl_adapter.cpp,v 1.38 2007/07/09 07:31:05 martinwguy Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -75,6 +75,36 @@ tu_file* make_stream(const char * /*url */)
 
 #include <curl/curl.h>
 
+// Adapt to older versions of libcurl.
+//
+// In 7.10.7 and before, CURLINFO_RESPONSE_CODE was called CURLINFO_HTTP_CODE
+//
+#if LIBCURL_VERSION_NUM <= 0x070a07
+# define CURLINFO_RESPONSE_CODE CURLINFO_HTTP_CODE
+#endif
+//
+// curl_multi_strerror() and curl_easy_strerror() came in at 7.12.0
+// Just print the number and tell the user how to decode it.
+//
+#if LIBCURL_VERSION_NUM < 0x070c00
+static char curl_strerror_buf[64];
+static const char *
+curl_easy_strerror(int code)
+{
+	sprintf(curl_strerror_buf,
+		"CurlE error code %d (man libcurl-errors)", code);
+	return curl_strerror_buf;
+}
+
+static const char *
+curl_multi_strerror(int code)
+{
+	sprintf(curl_strerror_buf,
+		"CurlM error code %d (man libcurl-errors)", code);
+	return curl_strerror_buf;
+}
+#endif
+
 namespace curl_adapter
 {
 
