@@ -8,6 +8,7 @@
 #include "geometry.h"
 #include "tu_file.h"
 
+using namespace std;
 
 const vec3	vec3::zero(0, 0, 0);
 const vec3	vec3::x_axis(1, 0, 0);
@@ -606,8 +607,8 @@ void	bitmap32::ProcessForColorKeyZero()
 // Pixels that have alpha > 0.5, but have color 0, should be tweaked slightly
 // so they don't get knocked out when blitting.
 {
-	uint32	Key32 = 0;
-	uint32*	p = GetData();
+	uint32_t	Key32 = 0;
+	uint32_t*	p = GetData();
 	int	pixels = GetHeight() * GetWidth();
 	for (int i = 0; i < pixels; i++, p++) {
 		if ((*p >> 24) >= 128) {
