@@ -1,15 +1,16 @@
 // 
-//   Copyright (C) 2005, 2006 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
+// 
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
@@ -47,9 +48,9 @@ class KXMLGUIClient; // workaround for kde3.3 on sarge with gcc4, kactioncollect
 
 class KlashFactory : public KParts::Factory {
 public:
-    KlashFactory ();
-    virtual ~KlashFactory ();
-    virtual KParts::Part *createPartObject
+    KDE_NO_CDTOR_EXPORT KlashFactory ();
+    KDE_NO_CDTOR_EXPORT virtual ~KlashFactory ();
+    KDE_NO_CDTOR_EXPORT virtual KParts::Part *createPartObject
         (QWidget *wparent, const char *wname,
          QObject *parent, const char *name,
          const char *className, const QStringList &args);
@@ -62,15 +63,15 @@ K_EXPORT_COMPONENT_FACTORY (libklashpart, KlashFactory)
 
 KInstance *KlashFactory::s_instance = 0;
 
-KDE_NO_CDTOR_EXPORT KlashFactory::KlashFactory () {
+KlashFactory::KlashFactory () {
     s_instance = new KInstance ("klash");
 }
 
-KDE_NO_CDTOR_EXPORT KlashFactory::~KlashFactory () {
+KlashFactory::~KlashFactory () {
     delete s_instance;
 }
 
-KDE_NO_EXPORT KParts::Part *KlashFactory::createPartObject
+KParts::Part *KlashFactory::createPartObject
   (QWidget *wparent, const char *wname,
    QObject *parent, const char * name,
    const char * cls, const QStringList & args) {
@@ -155,7 +156,17 @@ KDE_NO_EXPORT bool KlashPart::allowRedir (const KURL & url) const {
     return kapp->authorizeURLAction ("redirect", m_docbase, url);
 }
 
-KDE_NO_EXPORT void KlashPart::play () {
+KDE_NO_EXPORT void KlashPart::play ()
+{
+
+    QString procname;
+    char *gnash_env = getenv("KLASH_PLAYER");
+    if (!gnash_env) {
+      procname = GNASHBINDIR "/kde-gnash";
+    } else {
+      procname = gnash_env;
+    }
+
     stop ();
     if (m_src_url.isEmpty ())
         return;
@@ -166,9 +177,14 @@ KDE_NO_EXPORT void KlashPart::play () {
     if (m_width > 0 && m_height > 0)
         dim = QString ("-j ") + QString::number (m_width) +
             QString (" -k ") + QString::number (m_height);
-    QString cmd = QString ("klash -x ") +
+    QString url = m_url.url();
+    QString url_param;
+    if (!url.isEmpty())
+        url_param = QString ("-u ") + KProcess::quote (url);
+    QString cmd = procname + QString (" -x ") +
         QString::number (static_cast <KlashView *> (widget ())->embedId()) +
         QChar (' ') + dim +
+        QChar (' ') + url_param +
         QChar (' ') + KProcess::quote (m_src_url);
     kdDebug () << cmd << endl;
     *m_process << cmd;
@@ -200,7 +216,7 @@ KDE_NO_EXPORT void KlashPart::stop () {
             m_process->wait(2);
         }
         delete m_process;
-        m_process = 0l;
+        m_process = 0L;
     }
 }
 
@@ -536,12 +552,7 @@ static const int XKeyPress = KeyPress;
 #undef Unsorted
 #undef Bool
 
-KlashEmbed::KlashEmbed (KlashView * view) : QXEmbed (view), m_view (view) {
-    setProtocol (QXEmbed::XPLAIN);
-    int scr = DefaultScreen (qt_xdisplay ());
-    embed (XCreateSimpleWindow (qt_xdisplay(), view->winId (), 0, 0, width(), height(), 1, BlackPixel (qt_xdisplay(), scr), BlackPixel (qt_xdisplay(), scr)));
-    XClearWindow (qt_xdisplay(), embeddedWinId ());
-}
+KlashEmbed::KlashEmbed (KlashView * view) : QXEmbed (view), m_view (view) {}
 
 KlashEmbed::~KlashEmbed () {}
 
@@ -557,7 +568,7 @@ void KlashView::resizeEvent (QResizeEvent *) {
 }
 
 WId KlashView::embedId () {
-    return m_embed->embeddedWinId ();
+    return m_embed->winId ();
 }
 
 #include "klash_part.moc"
