@@ -1,23 +1,21 @@
 // 
-//   Copyright (C) 2005, 2006 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
+// 
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-// 
-//
-
-/* $Id: plugin.cpp,v 1.64.2.3 2006/11/17 10:08:02 tgc Exp $ */
+/* $Id: plugin.cpp,v 1.81.2.3 2007/08/18 16:48:54 strk Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -29,16 +27,24 @@
 #define PLUGIN_NAME     "Shockwave Flash"
 #define MIME_TYPES_DESCRIPTION  MIME_TYPES_HANDLED":swf:"PLUGIN_NAME
 
-#define PLUGIN_DESCRIPTION \
-  "Shockwave Flash 8.0 - Gnash " VERSION ", the GNU Flash Player. Copyright   \
-  &copy; 2006 <a href=\"http://www.fsf.org\">Free Software Foundation</a>,    \
-  Inc.<br> Gnash comes with NO WARRANTY, to the extent permitted by law.  You \
-  may redistribute copies of Gnash under the terms of the                     \
-  <a href=\"http://www.gnu.org/licenses/gpl.html\">GNU General Public License \
-  </a>. For more information about Gnash, see <a \
-  href=\"http://www.gnu.org/software/gnash/\"> \
-  http://www.gnu.org/software/gnash</a>."
+//Some javascript plugin detectors use the description
+//to decide the flash version to display. They expect the
+//form (major version).(minor version) r(revision).
+//e.g. "8.0 r99."
+#define FLASH_VERSION DEFAULT_FLASH_MAJOR_VERSION"."\
+	DEFAULT_FLASH_MINOR_VERSION" r"DEFAULT_FLASH_REV_NUMBER"."
 
+#define PLUGIN_DESCRIPTION \
+  "Shockwave Flash "FLASH_VERSION" Gnash "VERSION", the GNU Flash Player. \
+  Copyright &copy; 2006 <a href=\"http://www.fsf.org\">Free Software \
+  Foundation</a>, Inc. <br> \
+  Gnash comes with NO WARRANTY, to the extent permitted by law. \
+  You may redistribute copies of Gnash under the terms of the \
+  <a href=\"http://www.gnu.org/licenses/gpl.html\">GNU General Public \
+  License</a>. For more information about Gnash, see <a \
+  href=\"http://www.gnu.org/software/gnash/\"> \
+  http://www.gnu.org/software/gnash</a>. \
+  Compatible Shockwave Flash "FLASH_VERSION
 
 #include <sys/param.h>
 #include "plugin.h" //Fixes Warning on redef of MIN/MAX
@@ -55,6 +61,7 @@
 #include <vector>
 #include <iostream>
 
+
 // Mozilla SDK headers
 #include "prinit.h"
 #include "prlock.h"
@@ -71,6 +78,8 @@ NPBool      plugInitialized = FALSE;
 
 static bool  waitforgdb = false;
 
+static const char* getPluginDescription();
+
 void
 PR_CALLBACK Destructor(void * /* data */)
 {
@@ -88,7 +97,7 @@ PR_CALLBACK Destructor(void * /* data */)
 char*
 NPP_GetMIMEDescription(void)
 {
-    return MIME_TYPES_DESCRIPTION;
+    return const_cast<char *>(MIME_TYPES_DESCRIPTION);
 }
 
 //
@@ -132,6 +141,17 @@ NS_PluginInitialize()
 	cout << "Gtk2+ supported in this Mozilla version" << endl;
     }
 
+    char* opts = getenv("GNASH_OPTIONS");
+    if ( opts )
+    {
+	cout << "GNASH_OPTIONS : " << opts << endl;
+	if ( strstr(opts, "waitforgdb") )
+	{
+		waitforgdb = true;
+	}
+
+    }
+
     plugInitialized = TRUE;
 
     return NPERR_NO_ERROR;
@@ -174,7 +194,7 @@ NS_PluginGetValue(NPPVariable aVariable, void *aValue)
       // This becomes the description field you see below the opening
       // text when you type about:plugins
       case NPPVpluginDescriptionString:
-          *static_cast<char **>(aValue) = PLUGIN_DESCRIPTION;
+          *static_cast<const char **>(aValue) = getPluginDescription();
           break;
 
       case NPPVpluginNeedsXEmbed:
@@ -225,6 +245,8 @@ nsPluginInstance::nsPluginInstance(nsPluginCreateData* data)
   : nsPluginInstanceBase(),
     _instance(data->instance),
     _window(0),
+    _width(0),
+    _height(0),
     _childpid(0)
 {
 	for (size_t i=0, n=data->argc; i<n; ++i) {
@@ -288,7 +310,7 @@ nsPluginInstance::init(NPWindow* aWindow)
 void
 nsPluginInstance::shut()
 {
-    if (_childpid) {
+    if (_childpid > 0) {
 	// it seems that waiting after a SIGINT hangs firefox
 	// IFF not run from the console (see bug#17082).
 	// SIGTERM instead solves this problem
@@ -353,7 +375,7 @@ nsPluginInstance::WriteStatus(char *msg) const
 }
 
 NPError
-nsPluginInstance::WriteStatus(string msg) const
+nsPluginInstance::WriteStatus(std::string msg) const
 {
   return WriteStatus( const_cast<char*>(msg.c_str()) );
 }
@@ -372,94 +394,20 @@ NPError
 nsPluginInstance::NewStream(NPMIMEType /* type */, NPStream * stream,
                             NPBool /* seekable */, uint16_t * /* stype */)
 {
-    string url = stream->url;
-    string opts;
-    size_t start, end, eq;
-    bool dumpopts = false;
+    _swf_url = stream->url;
+
+    cout << __FUNCTION__ << ": The full URL is " << _swf_url << endl;
 
-#if 0
-    log_msg("%s: this = %p, URL is %s", __FUNCTION__,
-      (void *)this, stream->url);
-#endif
 
-    // extract the parameters from the URL
 #ifdef WRITE_FILE
+    size_t start, end;
     string fname;
-    end   = url.find(".swf", 0) + 4;
-    start = url.rfind("/", end) + 1;
+    end   = _swf_url.find(".swf", 0) + 4;
+    start = _swf_url.rfind("/", end) + 1;
     fname = "/tmp/";
-    fname += url.substr(start, end - start);
+    fname += _swf_url.substr(start, end - start);
     cout << "The Flash movie name is: " << fname << endl;
-#endif
-    end   = url.find(".swf", 0) + 4;
-    start = url.find("?", end);
-    end   = url.size();
-    if (start != string::npos) {
-	opts = url.substr(start+1, end);
-    }
 
-    string name;
-    string value;
-
-    cout << __FUNCTION__ << ": The full URL is " << url << endl;
-    while (opts.size() > 0) {
-	start = 0; // TODO: An empty name seems useless to me. If this is 
-	           //       indeed the case, we should set `start' to one.
-
-	eq = opts.find("=", start);
-	if (eq == string::npos) {
-	    cout << "INFO: Ignoring URL appendix without name." << endl;
-	    goto process;
-	} 
-
- 	if (opts[0] == '&') {
-	    // A (technically invalid) URL like movie.swf?&option=value.
-	    start++;
-	}
-
-	end = opts.find("&", start);
- 	if (end == string::npos) {
-	    // We have only one name=value pair remaining.
-	    end = opts.size();
- 	}
-	
-	name = opts.substr(start, eq);
-	value = opts.substr(eq+1, end-eq-1);
-	
-	if (dumpopts) {
-	    cout << __FUNCTION__ << "Option " << name << " = "
-		       << value << endl;
-	}
-	// Look for our special debug flags
-	if (name == "debug") {
- 	    cout << __FUNCTION__ << "Debug flag is " << value << endl;
-	    if (value == "waitforgdb") {
-		waitforgdb = true;
-	    }
-	    if (value == "dumpopts") {
-		dumpopts = true;
-	    }
-	} else {
-	    _options[name] = value;
-	}
-	if ((opts.size() > end) && (opts[end] == '&')) {
-		end++;
-	}
-	if (end != string::npos) {
-	    opts.erase(start, end);
-	}
-    }
-
-#if 0
-    log_msg("%s: URL is %s", __PRETTY_FUNCTION__, url.c_str());
-    log_msg("%s: Open stream for %s, this = %p", __FUNCTION__,
-    fname.c_str(), (void *)this);
-#endif
-
-process:
-    _swf_url = url;
-
-#ifdef WRITE_FILE
     _filefd = open(fname.c_str(), O_CREAT | O_WRONLY, S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH);
     if (_filefd < 0) {
         _filefd = open(fname.c_str(), O_TRUNC | O_WRONLY, S_IRUSR|S_IRGRP|S_IROTH);
@@ -500,21 +448,11 @@ nsPluginInstance::DestroyStream(NPStream * /* stream */, NPError /* reason */)
     }
 #endif
 
-    if (waitforgdb) {
-	cout << "Attach GDB to PID " << getpid() << " to debug!" << endl;
-	cout << "This thread will block until then!..." << endl;
-	cout <<"Once blocked here, you can set other breakpoints." << endl;
-	cout << "do a \"set variable waitforgdb=false\" to continue" << endl;
-	while (waitforgdb) {
-	    sleep(1);
-	}
-    }
-
     return NPERR_NO_ERROR;
 }
 
 /// \brief Return how many bytes we can read into the buffer
-int32
+int32_t
 nsPluginInstance::WriteReady(NPStream * /* stream */ )
 {
 #if 0
@@ -526,8 +464,8 @@ nsPluginInstance::WriteReady(NPStream * /* stream */ )
 /// \brief Read the data stream from Mozilla/Firefox
 ///
 /// For now we read the bytes and write them to a disk file.
-int32
-nsPluginInstance::Write(NPStream * /* stream */, int32 /* offset */, int32 len,
+int32_t
+nsPluginInstance::Write(NPStream * /* stream */, int32_t /* offset */, int32_t len,
                         void * buffer)
 {
 #if 0
@@ -546,8 +484,8 @@ nsPluginInstance::startProc(Window win)
     string procname;
     char *gnash_env = getenv("GNASH_PLAYER");
     if (!gnash_env) {
-      procname = PREFIX;
-      procname += "/bin/gnash";
+      procname = GNASHBINDIR;
+      procname += "/gtk-gnash";
     } else {
       procname = gnash_env;
     }
@@ -602,6 +540,22 @@ nsPluginInstance::startProc(Window win)
       cout << "ERROR: dup2() failed: " << strerror(errno) << endl;
     }
 
+    // Close all of the browser's file descriptors that we just 
+    // inherited (including pipefd[0] that we just dup'd to fd 0).
+    // Experiments show seventy or eighty file descriptors open in
+    // typical cases.  Rather than close all the thousands of possible file
+    // descriptors, we start after stderr and keep closing higher numbers
+    // until we encounter ten fd's in a row that
+    // aren't open.  This will tend to close most fd's in most programms.
+    int numfailed = 0, closed = 0;
+    int anfd = fileno(stderr)+1;
+    for ( ; numfailed < 10; anfd++) {
+	ret = close (anfd);
+	if (ret < 0) numfailed++;
+	else         { numfailed = 0; closed++; }
+    }
+    cout << "Closed " << closed << "files." <<endl;
+
     // setup the command line
 
     const char* pageurl = getCurrentPageURL();
@@ -678,6 +632,16 @@ nsPluginInstance::startProc(Window win)
     }
     cout << endl;
 
+    if (waitforgdb) {
+	cout << endl << "  Attach GDB to PID " << getpid() << " to debug!" << endl;
+	cout << "  This thread will block until then!..." << endl;
+	cout << "  Once blocked here, you can set other breakpoints." << endl;
+	cout << "  do a \"set variable waitforgdb=false\" to continue" << endl << endl;
+	while (waitforgdb) {
+	    sleep(1);
+	}
+    }
+
     execv(argv[0], argv);
     // if execv returns, an error has occurred.
     perror(strerror(errno));
@@ -731,6 +695,17 @@ nsPluginInstance::getCurrentPageURL() const
         return propValue.utf8characters; // const char *
 }
 
+static const char* getPluginDescription() 
+{
+	static const char* desc = NULL;
+	if ( ! desc )
+	{
+    		desc = getenv("GNASH_PLUGIN_DESCRIPTION");
+		if ( ! desc ) desc = PLUGIN_DESCRIPTION;
+	}
+	return desc;
+}
+
 // Local Variables:
 // mode: C++
 // indent-tabs-mode: t
