@@ -3,7 +3,7 @@
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
@@ -59,15 +59,23 @@ GetterSetter::operator=(const GetterSetter& s)
 {
 	if ( s._getter != _getter )
 	{
+#ifndef GNASH_USE_GC
 		_getter->drop_ref();
+#endif // ndef GNASH_USE_GC
 		_getter = s._getter;
+#ifndef GNASH_USE_GC
 		_getter->add_ref();
+#endif // ndef GNASH_USE_GC
 	}
 	if ( s._setter != _setter )
 	{
+#ifndef GNASH_USE_GC
 		_setter->drop_ref();
+#endif // ndef GNASH_USE_GC
 		_setter = s._setter;
+#ifndef GNASH_USE_GC
 		_setter->add_ref();
+#endif // ndef GNASH_USE_GC
 	}
 	return *this;
 }
@@ -77,8 +85,10 @@ GetterSetter::GetterSetter(const GetterSetter& s)
 	_getter(s._getter),
 	_setter(s._setter)
 {
+#ifndef GNASH_USE_GC
 	_getter->add_ref();
 	_setter->add_ref();
+#endif // ndef GNASH_USE_GC
 }
 
 GetterSetter::
@@ -87,16 +97,28 @@ GetterSetter(as_function& getter, as_function& setter)
 	_getter(&getter),
 	_setter(&setter)
 {
+#ifndef GNASH_USE_GC
 	_getter->add_ref();
 	_setter->add_ref();
+#endif // ndef GNASH_USE_GC
 }
 
 GetterSetter::~GetterSetter()
 {
+#ifndef GNASH_USE_GC
 	_getter->drop_ref();
 	_setter->drop_ref();
+#endif // ndef GNASH_USE_GC
 }
 
+void
+GetterSetter::setReachable() const
+{
+#ifdef GNASH_USE_GC
+	_getter->setReachable();
+	_setter->setReachable();
+#endif
+}
 
 } // end of gnash namespace
 
