@@ -3,7 +3,7 @@
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
@@ -77,6 +77,9 @@ public:
 
 	/// invoke the setter function
 	void setValue(as_object* this_ptr, const as_value& val) const;
+
+	/// Mark both getter and setter as being reachable (for GC)
+	void setReachable() const;
 };
 
 
