@@ -3,7 +3,7 @@
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
@@ -36,8 +36,6 @@ namespace gnash {
 	class as_object;
 	class as_environment;
 	class as_function;
-	//class as_value;
-	//class Property;
 }
 
 namespace gnash {
@@ -354,6 +352,10 @@ public:
 	///	so this method too is non-const.
 	///
 	void dump(as_object& this_ptr);
+
+	/// Mark all simple properties, getters and setters
+	/// as being reachable (for the GC)
+	void setReachable() const;
 };
 
 
