@@ -1,15 +1,16 @@
 // 
-//   Copyright (C) 2005, 2006 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
+// 
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
@@ -20,6 +21,8 @@
 #ifndef _GNASH_URLACCESSMANAGER_H
 #define _GNASH_URLACCESSMANAGER_H
 
+#include <string>
+
 // Forward declarations
 namespace gnash {
 	class URL;
@@ -37,6 +40,10 @@ namespace URLAccessManager {
 ///
 bool allow(const URL& url);
 
+bool allowHost(const std::string& str);
+
+bool allowHost(const std::string& str, short port);
+
 } // AccessManager
 
 } // namespace gnash
