@@ -4,7 +4,7 @@
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
@@ -17,7 +17,7 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-/* $Id: as_environment.cpp,v 1.78 2007/05/25 13:25:47 strk Exp $ */
+/* $Id: as_environment.cpp,v 1.82 2007/07/11 00:16:38 strk Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -140,10 +140,17 @@ as_environment::get_variable_raw(
     }
 
     // Check built-in constants.
-    if (varname == "_root" || varname == "_level0") {
+    if (varname == "_root") 
+    { 
 	if ( retTarget ) *retTarget = NULL; // correct ??
 	return as_value(m_target->get_root_movie());
     }
+    else if (varname.compare(0, 6, "_level") == 0 && varname.find_first_not_of("0123456789", 7) == string::npos )
+    {
+       if ( retTarget ) *retTarget = NULL; // correct ??
+       unsigned int levelno = atoi(varname.c_str()+6);
+       return VM::get().getRoot().getLevel(levelno).get();
+    }
 
     VM& vm = VM::get();
     as_object* global = vm.getGlobal();
@@ -447,7 +454,9 @@ as_environment::find_target(const as_value& val) const
 		boost::intrusive_ptr<as_object> obj = val.to_object();
 		assert (obj);
 		character* s=dynamic_cast<character*>(obj.get());
+#ifndef GNASH_USE_GC
 		assert(s->get_ref_count() > 1); // or the intrusive_ptr above going out-of-scope will kill it
+#endif // ndef GNASH_USE_GC
 		//log_msg(_("find_target is a character, returning it"));
 		return s; // might be NULL
 	}
@@ -660,7 +669,7 @@ as_environment::find_object_dotsyntax(const std::string& path) const
 		{
 			IF_VERBOSE_ASCODING_ERRORS(
 			log_aserror(_("Member %s for object %p not found (dotsyntax).  Path was %s"),
-				subpart.c_str(), env, path.c_str());
+				subpart.c_str(), (void *)env, path.c_str());
 			);
 			return NULL;
 		}
@@ -669,7 +678,7 @@ as_environment::find_object_dotsyntax(const std::string& path) const
 		{
 			IF_VERBOSE_ASCODING_ERRORS(
 			log_aserror(_("Element '%s' of variable '%s' not found in object %p nor in _global (dotsyntax)"),
-				subpart.c_str(), path.c_str(), env);
+				subpart.c_str(), path.c_str(), (void *)env);
 			);
 			return NULL;
 		}
@@ -680,13 +689,15 @@ as_environment::find_object_dotsyntax(const std::string& path) const
 		IF_VERBOSE_ASCODING_ERRORS(
 		log_aserror(_("Member %s of object %p doesn't cast to an Object (%s) "
 			"evaluating target path %s (dotsyntax)"),
-			subpart.c_str(), env, tmp.to_debug_string().c_str(), path.c_str());
+			subpart.c_str(), (void *)env, tmp.to_debug_string().c_str(), path.c_str());
 		);
 		return NULL;
 	}
 
 	env = tmp.to_object().get();
+#ifndef GNASH_USE_GC
 	assert(env->get_ref_count() > 0); // still alive...
+#endif // ndef GNASH_USE_GC
 
 
 	//@@   _level0 --> root, .. --> parent, . --> this, other == character
@@ -775,7 +786,7 @@ as_environment::find_object_slashsyntax(const std::string& path) const
 		if ( ! ch ) {
 			IF_VERBOSE_ASCODING_ERRORS(
 			log_aserror(_("'..' element in path '%s' follows a non-character object %p"),
-					path.c_str(), env);
+					path.c_str(), (void *)env);
 			);
 			return NULL;
 		}
@@ -784,7 +795,7 @@ as_environment::find_object_slashsyntax(const std::string& path) const
 		{
 			IF_VERBOSE_ASCODING_ERRORS(
 			log_aserror(_("'..' in path '%s' follows a character with no parent (%s : %p) (root is %p)"),
-					path.c_str(), ch->get_text_value().c_str(), ch, VM::get().getRoot().get_root_movie());
+					path.c_str(), ch->get_text_value().c_str(), (void *)ch, (void *)VM::get().getRoot().get_root_movie());
 			);
 			// if we override env, getvariable.as fails [line 57]
 			//env = ch;
@@ -805,7 +816,7 @@ as_environment::find_object_slashsyntax(const std::string& path) const
 			{
 				IF_VERBOSE_ASCODING_ERRORS(
 				log_aserror(_("Member %s for object %p not found (slashsyntax). Path was %s"),
-					subpart.c_str(), env, path.c_str());
+					subpart.c_str(), (void *)env, path.c_str());
 				);
 				return NULL;
 			}
@@ -814,7 +825,7 @@ as_environment::find_object_slashsyntax(const std::string& path) const
 			{
 				IF_VERBOSE_ASCODING_ERRORS(
 				log_aserror(_("Element '%s' of variable '%s' not found in object %p nor in _global (slashsyntax)"),
-					subpart.c_str(), path.c_str(), env);
+					subpart.c_str(), path.c_str(), (void *)env);
 				);
 				return NULL;
 			}
@@ -825,13 +836,15 @@ as_environment::find_object_slashsyntax(const std::string& path) const
 			IF_VERBOSE_ASCODING_ERRORS(
 			log_aserror(_("Member %s of object %p doesn't cast to an Object (%s) "
 				"evaluating target path %s (slashsyntax)"),
-				subpart.c_str(), env, tmp.to_debug_string().c_str(), path.c_str());
+				subpart.c_str(), (void *)env, tmp.to_debug_string().c_str(), path.c_str());
 				);
 			return NULL;
 		}
 
 		env = tmp.to_object().get();
+#ifndef GNASH_USE_GC
 		assert(env->get_ref_count() > 0);
+#endif // ndef GNASH_USE_GC
 	}
 
 	//@@   _level0 --> root, .. --> parent, . --> this, other == character
@@ -1028,6 +1041,51 @@ as_environment::CallFrame::CallFrame(as_function* funcPtr)
 {
 }
 
+#ifdef GNASH_USE_GC
+/// Mark all reachable resources
+//
+/// Reachable resources would be registers and
+/// locals (expected to be empty?) and function.
+void
+as_environment::CallFrame::markReachableResources() const
+{
+	if ( func ) func->setReachable();
+	for (Registers::const_iterator i=registers.begin(), e=registers.end(); i!=e; ++i)
+	{
+		i->setReachable();
+	}
+	locals->setReachable();
+}
+
+void
+as_environment::markReachableResources() const
+{
+	for (size_t i=0, s=4; i<4; ++i)
+	{
+		m_global_register[i].setReachable();
+	}
+
+	if ( m_target ) m_target->setReachable();
+	if ( _original_target ) _original_target->setReachable();
+
+	assert ( _localFrames.empty() );
+#if 1 // I think we expect the stack to be empty !
+	for (CallStack::const_iterator i=_localFrames.begin(), e=_localFrames.end(); i!=e; ++i)
+	{
+		i->markReachableResources();
+	}
+#endif
+
+	assert ( m_stack.empty() );
+#if 1 // I think we expect the stack to be empty !
+	for (std::vector<as_value>::const_iterator i=m_stack.begin(), e=m_stack.end(); i!=e; ++i)
+	{
+		i->setReachable();
+	}
+#endif
+}
+#endif // GNASH_USE_GC
+
 } // end of gnash namespace
 
 
