@@ -3,7 +3,7 @@
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
@@ -25,12 +25,17 @@
 #include "tu_config.h"
 
 #include "container.h"
-#include "ref_counted.h" // for inheritance 
+#include "ref_counted.h" // for inheritance  (to drop)
+#include "GC.h" // for inheritance from GcResource (to complete)
 #include "PropertyList.h"
 #include "as_value.h" // for return of get_primitive_value
 #include "smart_ptr.h"
 #include "as_prop_flags.h" // for enum
 #include "GnashException.h"
+#define NEW_KEY_LISTENER_LIST_DESIGN
+#ifdef NEW_KEY_LISTENER_LIST_DESIGN
+  #include "event_id.h" // for event_id
+#endif
 #include <sstream>
 
 #if defined(__GNUC__) && __GNUC__ > 2
@@ -56,14 +61,19 @@ namespace gnash {
 /// Base-class for ActionScript script-defined objects.
 /// This would likely be ActionScript's 'Object' class.
 ///
-//class as_object : public resource
-class DSOEXPORT as_object : public ref_counted
+class DSOEXPORT as_object
+	:
+#ifdef GNASH_USE_GC
+	public GcResource
+#else
+	public ref_counted
+#endif
 {
 	/// Properties of this objects 
 	PropertyList _members;
 
 	/// Don't allow implicit copy, must think about behaviour
-	as_object& operator==(const as_object&)
+	as_object& operator=(const as_object&)
 	{
 		assert(0);
 		return *this;
@@ -116,12 +126,6 @@ public:
 	/// TODO: write more about this, is it allowed ? is it safe ?
 	///
 	as_object(const as_object& other);
-
-	/// Default destructor for ActionScript objects.
-	//
-	/// Drops reference on prototype member, if any.
-	///
-	virtual ~as_object() {}
 	
 	/// Return a text representation for this object
 	virtual std::string get_text_value() const { return "[object Object]"; }
@@ -162,11 +166,21 @@ public:
 	/// NOTE: This might change in the near future trough use of
 	///       getter/setter properties instead..
 	///
+	/// @param name
+	///	Name of the property. Must be all lowercase
+	///	if the current VM is initialized for a  target
+	///	up to SWF6.
+	///
+	/// @param val
+	///	Value to assign to the named property.
+	///
 	virtual void set_member(const std::string& name, const as_value& val)
 	{
 		return set_member_default(name, val);
 	}
-
+#ifdef NEW_KEY_LISTENER_LIST_DESIGN
+	virtual bool on_event(const event_id& id );
+#endif
 	/// Initialize a member value
 	//
 	/// This method has to be used by built-in classes initialization
@@ -237,13 +251,23 @@ public:
 	/// The default behaviour is to call set_member_default,
 	/// but this function is kept virtual to allow special
 	/// handling of property fetching in derivate class.
-	/// NOTE: This might change in the near future trough use of
-	///       getter/setter properties instead..
 	///
 	/// NOTE that this method is non-const becase a property
 	///      could also be a getter/setter and we can't promise
 	///      that the 'getter' won't change this object trough
-	///	 use of the 'this' reference.
+	///	 use of the 'this' reference. 
+	///
+	/// @param name
+	///	Name of the property. Must be all lowercase
+	///	if the current VM is initialized for a  target
+	///	up to SWF6.
+	///
+	/// @param val
+	///	Variable to assign an existing value to.
+	///	Will be untouched if no property with the given name
+	///	was found.
+	///
+	/// @return true of the named property was found, false otherwise.
 	///
 	virtual bool get_member(const std::string& name, as_value* val)
 	{
@@ -286,10 +310,10 @@ public:
 
 	/// Set member flags (probably used by ASSetPropFlags)
 	//
-	/// @parame name
-	///     Name of the property.
-	///	Case insensitive up to SWF6,
-	///	case *sensitive* from SWF7 up.
+	/// @param name
+	///	Name of the property. Must be all lowercase
+	///	if the current VM is initialized for a  target
+	///	up to SWF6.
 	///
 	/// @param setTrue
 	///	the set of flags to set
@@ -365,6 +389,18 @@ public:
 	///
 	void enumerateProperties(as_environment& env) const;
 
+	/// Enumerate any non-proper properties
+	//
+	/// This function is called by enumerateProperties(as_environment&) 
+	/// to allow for enumeration of properties that are not "proper"
+	/// (not contained in the as_object PropertyList).
+	///
+	/// The default implementation adds nothing
+	///
+	virtual void enumerateNonProperties(as_environment&) const
+	{
+	}
+
 	/// \brief
 	/// Enumerate all non-hidden properties inserting
 	/// their name/value pair to the given map.
@@ -430,7 +466,7 @@ public:
 	static as_value valueof_method(const fn_call& fn);
 
 	/// @} Common ActionScript getter-setters for characters
-
+	
 protected:
 
 	/// Get a property value by name
@@ -447,11 +483,9 @@ protected:
 	///	 use of the 'this' reference.
 	///
 	/// @param name
-	///     Name of the property.
-	///	Case insensitive up to SWF6,
-	///	case *sensitive* from SWF7 up.
-	///	TODO: be *always* case-sensitive, and delegate
-	///	      convertion to lowercase to callers
+	///	Name of the property. Must be all lowercase
+	///	if the current VM is initialized for a  target
+	///	up to SWF6.
 	///
 	/// @param val
 	///     The as_value to store a found variable's value in.
@@ -466,13 +500,39 @@ protected:
         /// The derived class should not override this method,
         /// but instead implement its own gettersetter properties.
 	///
-	/// @parame name
-	///     Name of the property.
-	///	Case insensitive up to SWF6,
-	///	case *sensitive* from SWF7 up.
+	/// @param name
+	///	Name of the property. Must be all lowercase
+	///	if the current VM is initialized for a  target
+	///	up to SWF6.
+	///
+	/// @param val
+	///	Value to assign to the named property.
 	///
 	void set_member_default(const std::string& name, const as_value& val);
 
+#ifdef GNASH_USE_GC
+	/// Mark all reachable resources, override from GcResource.
+	//
+	/// The default implementation marks all properties
+	/// as being reachable, calling markAsObjectReachable().
+	///
+	/// If a derived class provides access to more GC-managed
+	/// resources, it should override this method and call 
+	/// markAsObjectReachable() as the last step.
+	///
+	virtual void markReachableResources() const
+	{
+		markAsObjectReachable();
+	}
+
+	/// Mark properties and __proto__ as reachable (for the GC)
+	void markAsObjectReachable() const
+	{
+		_members.setReachable();
+		if ( m_prototype.get() ) m_prototype->setReachable();
+	}
+#endif // GNASH_USE_GC
+
 	/// The Virtual Machine used to create this object
 	VM& _vm;
 
@@ -497,7 +557,7 @@ ensureType (boost::intrusive_ptr<as_object> obj)
 
 	if (!ret) {
 		std::string     target = typeid(T).name(),
-				source = typeid(obj.get()).name();
+				source = typeid(*obj.get()).name();
 #if defined(__GNUC__) && __GNUC__ > 2
 		int status;
 		char* target_unmangled = 
