@@ -4,7 +4,7 @@
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
@@ -32,6 +32,7 @@
 #include "Number.h" // for automatic as_value::NUMBER => Number as object
 #include "Boolean.h" // for automatic as_value::BOOLEAN => Boolean as object
 #include "action.h" // for call_method0
+#include "utility.h" // for typeName()
 
 #include <boost/algorithm/string/case_conv.hpp>
 
@@ -45,27 +46,24 @@ using namespace std;
 // rather then by "target" ref.
 #define MOVIECLIP_AS_SOFTREF
 
+// Define the macro below to make abstract equality operator verbose
+//#define GNASH_DEBUG_EQUALITY
+
 namespace gnash {
 
 //
 // as_value -- ActionScript value type
 //
 
-static void
-lowercase_if_needed(std::string& str)
-{
-	VM& vm = VM::get();
-	if ( vm.getSWFVersion() >= 7 ) return;
-	boost::to_lower(str, vm.getLocale());
-}
-
 as_value::as_value(as_function* func)
     :
     m_type(AS_FUNCTION),
     m_object_value(func)
 {
     if (m_object_value) {
+#ifndef GNASH_USE_GC
 	m_object_value->add_ref();
+#endif // GNASH_USE_GC
     } else {
         m_type = NULLTYPE;
     }
@@ -132,8 +130,7 @@ as_value::to_string(as_environment* env) const
 			bool gotValidToStringResult = false;
 			if ( env )
 			{
-				std::string methodname = "toString";
-				lowercase_if_needed(methodname);
+				std::string methodname = PROPNAME("toString"); 
 				as_value method;
 				if ( obj->get_member(methodname, &method) )
 				{
@@ -209,8 +206,7 @@ as_value::to_primitive(as_environment& env) const
 	if ( m_type == OBJECT || m_type == AS_FUNCTION )
 	{
 		as_object* obj = m_object_value;
-		std::string methodname = "valueOf";
-		lowercase_if_needed(methodname);
+		std::string methodname = PROPNAME("valueOf"); 
 		as_value method;
 		if ( obj->get_member(methodname, &method) )
 		{
@@ -289,8 +285,7 @@ as_value::to_number(as_environment* env) const
 			as_object* obj = m_object_value; 
 			if ( env )
 			{
-				std::string methodname = "valueOf";
-				lowercase_if_needed(methodname);
+				std::string methodname = PROPNAME("valueOf"); 
 				as_value method;
 				if ( obj->get_member(methodname, &method) )
 				{
@@ -335,6 +330,26 @@ as_value::to_number(as_environment* env) const
 	/* NOTREACHED */
 }
 
+int32_t
+as_value::to_int(as_environment& env) const
+{
+	double d = to_number(&env);
+	int i=0;
+
+	if ( ! isfinite(d) ) return 0;
+
+	if (d < 0)
+	{
+		i = - (uint32_t) fmod (-d, 4294967296.0);
+	}
+	else
+	{
+		i = (uint32_t) fmod (d, 4294967296.0);
+	}
+
+	return i;
+}
+
 // Conversion to boolean for SWF7 and up
 bool
 as_value::to_bool_v7() const
@@ -605,10 +620,12 @@ as_value::set_as_object(as_object* obj)
 		drop_refs();
 		m_type = OBJECT;
 		m_object_value = obj;
+#ifndef GNASH_USE_GC
 		if (m_object_value)
 		{
 			m_object_value->add_ref();
 		}
+#endif // GNASH_USE_GC
 	}
 }
 
@@ -626,7 +643,9 @@ as_value::set_as_function(as_function* func)
 	m_type = AS_FUNCTION;
 	m_object_value = func;
 	if (m_object_value) {
+#ifndef GNASH_USE_GC
 	    m_object_value->add_ref();
+#endif // GNASH_USE_GC
 	} else {
 	    m_type = NULLTYPE;
 	}
@@ -634,15 +653,20 @@ as_value::set_as_function(as_function* func)
 }
 
 bool
-as_value::equals(const as_value& v, as_environment* env) const
+as_value::equals(const as_value& v, as_environment& env) const
 {
-    //log_msg("equals(%s, %s) called", to_debug_string().c_str(), v.to_debug_string().c_str());
+    // Comments starting with numbers refer to the ECMA-262 document
+
+#ifdef GNASH_DEBUG_EQUALITY
+    static int count=0;
+    log_debug("equals(%s, %s) called [%d]", to_debug_string().c_str(), v.to_debug_string().c_str(), count++);
+#endif
 
     bool this_nulltype = (m_type == UNDEFINED || m_type == NULLTYPE);
     bool v_nulltype = (v.get_type() == UNDEFINED || v.get_type() == NULLTYPE);
     if (this_nulltype || v_nulltype)
     {
-	return this_nulltype == v_nulltype;
+        return this_nulltype == v_nulltype;
     }
 
     bool obj_or_func = (m_type == OBJECT || m_type == AS_FUNCTION);
@@ -652,46 +676,82 @@ as_value::equals(const as_value& v, as_environment* env) const
     if ( obj_or_func && v_obj_or_func ) return m_object_value == v.m_object_value;
     if ( m_type == v.m_type ) return equalsSameType(v);
 
-    else if (m_type == STRING)
-    {
-	return m_string_value == v.to_string(env);
-    }
+    // 16. If Type(x) is Number and Type(y) is String,
+    //    return the result of the comparison x == ToNumber(y).
     else if (m_type == NUMBER && v.m_type == STRING)
     {
-	return equalsSameType(v.to_number(env)); // m_number_value == v.to_number(env);
-	//return m_number_value == v.to_number(env);
+        return equalsSameType(v.to_number(&env)); 
     }
+
+    // 17. If Type(x) is String and Type(y) is Number,
+    //     return the result of the comparison ToNumber(x) == y.
     else if (v.m_type == NUMBER && m_type == STRING)
     {
-	return v.equalsSameType(to_number(env)); // m_number_value == v.to_number(env);
-	//return v.m_number_value == to_number(env);
+        return v.equalsSameType(to_number(&env)); 
     }
+
+    // 18. If Type(x) is Boolean, return the result of the comparison ToNumber(x) == y.
     else if (m_type == BOOLEAN)
     {
-	return m_boolean_value == v.to_bool();
+        return as_value(to_number(&env)).equals(v, env); // m_boolean_value == v.to_bool();
     }
 
-    else if (m_type == OBJECT || m_type == AS_FUNCTION)
+    // 19. If Type(y) is Boolean, return the result of the comparison x == ToNumber(y).
+    else if (v.m_type == BOOLEAN)
     {
-    	assert ( ! (v.m_type == OBJECT || v.m_type == AS_FUNCTION) );
-	// convert this value to a primitive and recurse
-	if ( ! env ) return false;
-	as_value v2 = to_primitive(*env); 
-	if ( v2.m_type == OBJECT || v2.m_type == AS_FUNCTION ) return false; // no valid conversion 
-	else return v2.equals(v, env);
+        return as_value(v.to_number(&env)).equals(*this, env); 
     }
 
-    else if (v.m_type == OBJECT || v.m_type == AS_FUNCTION)
+    // 20. If Type(x) is either String or Number and Type(y) is Object,
+    //     return the result of the comparison x == ToPrimitive(y).
+    else if ( (m_type == STRING || m_type == NUMBER ) && ( v.is_object() ) ) // v.m_type == OBJECT || v.m_type == AS_FUNCTION ) )
     {
-    	assert ( ! (m_type == OBJECT || m_type == AS_FUNCTION) );
-	// convert this value to a primitive and recurse
-	if ( ! env ) return false;
-	as_value v2 = v.to_primitive(*env); 
-	if ( v2.m_type == OBJECT || v2.m_type == AS_FUNCTION ) return false; // no valid conversion 
-	else return equals(v2, env);
+        // convert this value to a primitive and recurse
+        as_value v2 = v.to_primitive(env); 
+#ifdef GNASH_DEBUG_EQUALITY
+       log_debug(" convertion to primitive : %s -> %s", v.to_debug_string().c_str(), v2.to_debug_string().c_str());
+#endif
+        if ( v.strictly_equals(v2) ) // returned self ?
+        {
+            return false; // no valid conversion  
+        }
+        else return equals(v2, env);
     }
 
-    return false;
+    // 21. If Type(x) is Object and Type(y) is either String or Number,
+    //    return the result of the comparison ToPrimitive(x) == y.
+    else if ( (v.m_type == STRING || v.m_type == NUMBER ) && ( is_object() ) ) // m_type == OBJECT || m_type == AS_FUNCTION ) )
+    {
+        // convert this value to a primitive and recurse
+        as_value v2 = to_primitive(env); 
+#ifdef GNASH_DEBUG_EQUALITY
+       log_debug(" convertion to primitive : %s -> %s", to_debug_string().c_str(), v2.to_debug_string().c_str());
+#endif
+        if ( strictly_equals(v2) ) // returned self ?
+        {
+            return false; // no valid conversion 
+        }
+        else return v2.equals(v, env);
+    }
+
+    // Both operands are objects (OBJECT,AS_FUNCTION,MOVIECLIP)
+    assert(is_object() && v.is_object());
+
+    // If any of the two converts to a primitive, we recurse
+
+    as_value p = to_primitive(env); 
+    as_value vp = v.to_primitive(env); 
+#ifdef GNASH_DEBUG_EQUALITY
+    log_debug(" convertion to primitive (this): %s -> %s", to_debug_string().c_str(), p.to_debug_string().c_str());
+    log_debug(" convertion to primitive (that): %s -> %s", v.to_debug_string().c_str(), vp.to_debug_string().c_str());
+#endif
+    if ( strictly_equals(p) && v.strictly_equals(vp) ) // both returned self ?
+    {
+            return false; // no valid conversion 
+    }
+
+    return p.equals(vp, env);
+
 }
 	
 // Sets *this to this string plus the given string.
@@ -707,6 +767,7 @@ as_value::string_concat(const std::string& str)
 void
 as_value::drop_refs()
 {
+#ifndef GNASH_USE_GC
     if (m_type == AS_FUNCTION || m_type == OBJECT )
     {
 	if (m_object_value) // should assert here ?
@@ -714,6 +775,7 @@ as_value::drop_refs()
 	    m_object_value->drop_ref();
 	}
     } 
+#endif // GNASH_USE_GC
 }
 
 const char*
@@ -779,7 +841,9 @@ as_value::equalsSameType(const as_value& v) const
 			double b = v.m_number_value;
 
 			// Nan != NaN
-			if ( isnan(a) || isnan(b) ) return false;
+			//if ( isnan(a) || isnan(b) ) return false;
+
+			if ( isnan(a) && isnan(b) ) return true;
 
 			// -0.0 == 0.0
 			if ( (a == -0 && b == 0) || (a == 0 && b == -0) ) return true;
@@ -814,10 +878,10 @@ as_value::to_debug_string() const
 			sprintf(buf, "[bool:%s]", m_boolean_value ? "true" : "false");
 			return buf;
 		case OBJECT:
-			sprintf(buf, "[object:%p]", m_object_value);
+			sprintf(buf, "[object(%s):%p]", typeName(*m_object_value).c_str(), (void *)m_object_value);
 			return buf;
 		case AS_FUNCTION:
-			sprintf(buf, "[function:%p]", m_object_value);
+			sprintf(buf, "[function:%p]", (void *)m_object_value);
 			return buf;
 		case STRING:
 			return "[string:" + m_string_value + "]";
@@ -1026,6 +1090,16 @@ as_value::doubleToString(double _val)
 	return std::string(_str);
 }
 
+void
+as_value::setReachable() const
+{
+#ifdef GNASH_USE_GC
+	if ( m_type == OBJECT || m_type == AS_FUNCTION )
+	{
+		m_object_value->setReachable();
+	}
+#endif // GNASH_USE_GC
+}
 
 } // namespace gnash
 
