@@ -3,7 +3,7 @@
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 
 // This program is distributed in the hope that it will be useful,
@@ -23,6 +23,10 @@
 
 #include "as_object.h" // for inheritance
 
+#ifdef USE_EXTENSIONS
+# include "extension.h" // for composition
+#endif
+
 // Forward declarations
 namespace gnash {
 	class VM;
@@ -36,6 +40,11 @@ class Global: public as_object
 public:
 	Global(VM& vm);
 	~Global() {}
+private:
+
+#ifdef USE_EXTENSIONS
+	Extension et;
+#endif
 };
 
 } // namespace gnash
