@@ -4,7 +4,7 @@
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
@@ -17,7 +17,7 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-/* $Id: Object.cpp,v 1.24 2007/04/18 11:00:30 jgilmore Exp $ */
+/* $Id: Object.cpp,v 1.26 2007/08/08 20:17:28 strk Exp $ */
 
 #include "tu_config.h"
 #include "Object.h"
@@ -250,7 +250,7 @@ object_registerClass(const fn_call& fn)
 	{
 		IF_VERBOSE_ASCODING_ERRORS(
 		log_aserror(_("Invalid call to Object.registerClass() - "
-			"empty symbol id"));
+			"empty symbol id (%s)"), fn.arg(0).to_debug_string().c_str());
 		);
 		return as_value(false);
 	}
@@ -260,7 +260,7 @@ object_registerClass(const fn_call& fn)
 	{
 		IF_VERBOSE_ASCODING_ERRORS(
 		log_aserror(_("Invalid call to Object.registerClass() - "
-			"class is not a function"));
+			"class is not a function (%s)"), fn.arg(1).to_debug_string().c_str());
 		);
 		return as_value(false);
 	}
