@@ -4,7 +4,7 @@
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
@@ -16,7 +16,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-/* $Id: xmlnode.cpp,v 1.35 2007/05/28 15:41:08 ann Exp $ */
+/* $Id: xmlnode.cpp,v 1.38 2007/08/04 04:19:29 strk Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -209,14 +209,18 @@ XMLNode::insertBefore(boost::intrusive_ptr<XMLNode> newnode, boost::intrusive_pt
 void
 XMLNode::removeNode()
 {
+#ifndef GNASH_USE_GC
     assert(get_ref_count() > 1);
+#endif
     boost::intrusive_ptr<XMLNode> oldparent = getParent();
     if ( oldparent )
     {
         oldparent->_children.remove(this);
     }
     _parent = NULL;
+#ifndef GNASH_USE_GC
     assert(get_ref_count() > 0);
+#endif
 }
 
 XMLNode *
@@ -759,6 +763,23 @@ void xmlnode_class_init(as_object& global)
 
 }
 
+#ifdef GNASH_USE_GC
+void
+XMLNode::markReachableResources() const
+{
+	// Mark childs
+	for (ChildList::const_iterator i=_children.begin(), e=_children.end(); i!=e; ++i)
+	{
+		(*i)->setReachable();
+	}
+
+	// Mark parent
+	if ( _parent ) _parent->setReachable();
+
+	markAsObjectReachable();
+}
+#endif // GNASH_USE_GC
+
 } // end of gnash namespace
 
 // Local Variables:
