@@ -4,7 +4,7 @@
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
@@ -171,7 +171,7 @@ DisplayList::place_character(
 	int depth,
 	const cxform& color_xform, 
 	const matrix& mat, 
-	float ratio,
+	int ratio,
 	int clip_depth)
 {
 //	GNASH_REPORT_FUNCTION;
@@ -254,7 +254,7 @@ DisplayList::replace_character(
 	int depth,
 	const cxform* color_xform,
 	const matrix* mat,
-	float ratio,
+	int ratio,
 	int clip_depth)
 {
 	//GNASH_REPORT_FUNCTION;
@@ -333,7 +333,7 @@ DisplayList::move_display_object(
 	int depth,
 	const cxform* color_xform,
 	const matrix* mat,
-	float ratio,
+	int ratio,
 	int /* clip_depth */)
 {
 	//GNASH_REPORT_FUNCTION;
@@ -503,10 +503,11 @@ void DisplayList::reset(movie_definition& movieDef, size_t tgtFrame, bool call_u
 
 	// 1. Find all "timeline depth" for the target frame, querying the
 	//    Timeline object in the sprite/movie definition (see implementation details)
-	std::vector<int> save;
+	std::map<int, int>  save;
 	movieDef.getTimelineDepths(tgtFrame, save);
 
-#define GNASH_DEBUG_TIMELINE 1
+//#define GNASH_DEBUG_TIMELINE 1
+#undef GNASH_DEBUG_TIMELINE
 #ifdef GNASH_DEBUG_TIMELINE
 	cout << "Depths found to save: " << endl;
 	std::ostream_iterator<int> ostrIter(cout, "," ) ;
@@ -516,7 +517,7 @@ void DisplayList::reset(movie_definition& movieDef, size_t tgtFrame, bool call_u
 #endif
 
 
-	typedef std::vector<int>::iterator SeekIter;
+	typedef std::map<int, int>::iterator SeekIter;
 
 	SeekIter startSeek = save.begin();
         SeekIter endSeek = save.end();
@@ -559,9 +560,11 @@ void DisplayList::reset(movie_definition& movieDef, size_t tgtFrame, bool call_u
 		}
 #endif
 
-		/// Only remove if not in the save vector
-		SeekIter match = std::find(startSeek, endSeek, di_depth);
-		if ( match == save.end() )
+		/// Only remove if not in the save vector, 
+		/// or in the save vector but has a different ratio value
+		SeekIter match = save.find(di_depth);
+		if( match == save.end() || 
+			( match != save.end() && match->second != di->get_ratio() ))
 		{
 			// Not to be saved, killing
 			if ( call_unload ) di->unload();
@@ -801,6 +804,8 @@ DisplayList::display()
 		}
 
 		// check whether this object should become mask
+		// (see bug #20527 for the "&& !masked" condition)
+		// if (ch->isMask() && !masked)
 		if (ch->isMask())
 		{
 			//log_msg(_("begin submit mask"));
@@ -812,6 +817,7 @@ DisplayList::display()
 		// if this object should have become a mask,
 		// inform the renderer that it now has all
 		// information about it
+		// if (ch->isMask() && !masked)
 		if (ch->isMask())
 		{
 			//log_msg(_("end submit mask"));
@@ -856,7 +862,9 @@ DisplayList::add_invalidated_bounds(InvalidatedRanges& ranges, bool force) {
 		it != endIt; ++it)
 	{
     DisplayItem& dobj = *it;
+#ifndef GNASH_USE_GC
     assert(dobj->get_ref_count() > 0);
+#endif // ndef GNASH_USE_GC
     dobj->add_invalidated_bounds(ranges, force);
 	}
 	
