@@ -5,7 +5,7 @@
 
 // A module to take care of all of gnash's loaded fonts.
 
-/* $Id: fontlib.cpp,v 1.25 2006/10/18 14:45:02 strk Exp $ */
+/* $Id: fontlib.cpp,v 1.33 2007/07/24 19:43:30 strk Exp $ */
 
 #include "container.h"
 #include "tu_file.h"
@@ -21,10 +21,16 @@
 #include "render.h"
 #include "movie_definition.h"
 
+// Define to the name of a default font.
+#define DEFAULT_FONT_NAME "_sans"
 
 namespace gnash {
 namespace fontlib {
-	std::vector< smart_ptr<font> >	s_fonts;
+
+namespace {
+	std::vector< boost::intrusive_ptr<font> >	s_fonts;
+	boost::intrusive_ptr<font> _defaultFont;
+}
 
 	// Size (in TWIPS) of the box that the glyph should
 	// stay within.
@@ -225,6 +231,9 @@ namespace fontlib {
 
 	void	finish_current_texture(movie_definition* owner)
 	{
+
+		bool embed=true; // use embedded fonts
+
 		if (s_pending_glyphs.size() == 0)
 		{
 			return;
@@ -276,12 +285,12 @@ namespace fontlib {
 
 		if (owner->get_create_bitmaps() == DO_LOAD_BITMAPS)
 		{
-			smart_ptr<bitmap_info>	bi;
+			boost::intrusive_ptr<bitmap_info>	bi;
 			bi = render::create_bitmap_info_alpha(
 				GLYPH_CACHE_TEXTURE_SIZE,
 				GLYPH_CACHE_TEXTURE_SIZE,
 				s_current_cache_image);
-			owner->add_bitmap_info(bi.get_ptr());
+			owner->add_bitmap_info(bi.get());
 
 			// Push finished glyphs into their respective fonts.
 			for (int i = 0, n = s_pending_glyphs.size(); i < n; i++)
@@ -290,9 +299,9 @@ namespace fontlib {
 			assert(pgi->m_glyph_index != -1);
 			assert(pgi->m_source_font != NULL);
 
-			pgi->m_texture_glyph.set_bitmap_info(bi.get_ptr());
-			pgi->m_source_font->add_texture_glyph(pgi->m_glyph_index, pgi->m_texture_glyph);
-			//s_pending_glyphs[i]->set_bitmap_info(bi.get_ptr());
+			pgi->m_texture_glyph.set_bitmap_info(bi.get());
+			pgi->m_source_font->add_texture_glyph(pgi->m_glyph_index, pgi->m_texture_glyph, embed);
+			//s_pending_glyphs[i]->set_bitmap_info(bi.get());
 			}
 		}
 		s_pending_glyphs.clear();
@@ -643,7 +652,7 @@ namespace fontlib {
 
 	bool	try_to_reuse_previous_image(
 		const rendered_glyph_info& rgi,
-		const hash<unsigned int, const rendered_glyph_info*>& image_hash)
+		const map<unsigned int, const rendered_glyph_info*>& image_hash)
 	// See if we've already packed an identical glyph image for
 	// another glyph.  If so, then reuse it, and return true.
 	// If no reusable image, return false.
@@ -652,8 +661,18 @@ namespace fontlib {
 	// fonts that use the same dummy glyph for many undefined
 	// characters.
 	{
+		bool embed=true; // use embedded fonts
+
+		const map<unsigned int, const rendered_glyph_info*>::const_iterator image =
+			image_hash.find(rgi.m_image_hash);
+
+
 		const rendered_glyph_info*	identical_image = NULL;
-		if (image_hash.get(rgi.m_image_hash, &identical_image))
+		if (image != image_hash.end()) {
+		  identical_image = (*image).second;
+		}
+
+		if (identical_image)
 		{
 			// Found a match.  But is it *really* a match?  Do a
 			// bitwise compare.
@@ -664,7 +683,7 @@ namespace fontlib {
 				texture_glyph	identical_tg =
 					identical_image->
 					m_source_font->
-					get_texture_glyph(identical_image->m_glyph_index);
+					get_texture_glyph(identical_image->m_glyph_index, embed); 
 
 				if (identical_tg.is_renderable() == false)
 				{
@@ -706,7 +725,7 @@ namespace fontlib {
 				{
 					// This image is already packed and has a valid bitmap_info.
 					// Push straight into our font.
-					rgi.m_source_font->add_texture_glyph(rgi.m_glyph_index, tg);
+					rgi.m_source_font->add_texture_glyph(rgi.m_glyph_index, tg, embed); // embed font only
 				}
 				else
 				{
@@ -783,7 +802,7 @@ namespace fontlib {
 
 		// Share identical texture data where possible, by
 		// doing glyph image comparisons.
-		hash<unsigned int, const rendered_glyph_info*>	image_hash;
+		map<unsigned int, const rendered_glyph_info*>	image_hash;
 
 		// Pack the glyphs.
 		{for (int i = 0, n = glyph_info.size(); i < n; )
@@ -850,9 +869,11 @@ namespace fontlib {
 							tg));
 
 					// Add this into the hash so it can possibly be reused.
-					if (image_hash.get(rgi.m_image_hash, NULL) == false)
+					map<unsigned int, const rendered_glyph_info*>::const_iterator image =
+						image_hash.find(rgi.m_image_hash);
+					if (image == image_hash.end())
 					{
-						image_hash.add(rgi.m_image_hash, &rgi);
+						image_hash[rgi.m_image_hash] = &rgi;
 					}
 
 					packed[index] = true;
@@ -888,13 +909,15 @@ static void	generate_font_bitmaps(std::vector<rendered_glyph_info>& glyph_info,
 	{
 		assert(f);
 
+		bool embed=true; // use embedded fonts
+
 		f->set_texture_glyph_nominal_size(s_glyph_nominal_size);
 
-		for (int i = 0, n = f->get_glyph_count(); i < n; i++)
+		for (int i = 0, n = f->getEmbedGlyphCount(); i < n; i++)
 		{
-			if (f->get_texture_glyph(i).is_renderable() == false)
+			if (f->get_texture_glyph(i, embed).is_renderable() == false) 
 			{
-				shape_character_def*	sh = f->get_glyph(i);
+				shape_character_def*	sh = f->get_glyph(i, embed); 
 				if (sh)
 				{
 					rect	glyph_bounds;
@@ -998,6 +1021,8 @@ static void	generate_font_bitmaps(std::vector<rendered_glyph_info>& glyph_info,
 	{
 		assert(out);
 
+		bool embed=true; // use embedded glyphs
+
 		// skip number of bitmaps.
 		int	bitmaps_used_base = owner->get_bitmap_info_count();
 		int	size_pos = out->get_position();
@@ -1034,7 +1059,7 @@ static void	generate_font_bitmaps(std::vector<rendered_glyph_info>& glyph_info,
 			out->write_le16(nominal_glyph_size);
 
 			// skip number of glyphs.
-			int ng = fonts[f]->get_glyph_count();
+			int ng = fonts[f]->getEmbedGlyphCount();
 			int ng_position = out->get_position();
 			out->write_le32(0);
 			
@@ -1043,7 +1068,7 @@ static void	generate_font_bitmaps(std::vector<rendered_glyph_info>& glyph_info,
 			// save texture glyphs:
 			for (int g = 0; g < ng; g++)
 			{
-				const texture_glyph& tg = fonts[f]->get_texture_glyph(g);
+				const texture_glyph& tg = fonts[f]->get_texture_glyph(g, embed);
 				if (tg.is_renderable())
 				{
 					// save glyph index.
@@ -1097,6 +1122,8 @@ static void	generate_font_bitmaps(std::vector<rendered_glyph_info>& glyph_info,
 	void	input_cached_data(tu_file* in, const std::vector<font*>& fonts, movie_definition* owner)
 	// Load a stream containing previously-saved font glyph textures.
 	{
+		bool embed=true; // use embedded glyphs
+
 		// load number of bitmaps.
 		int nb = in->read_le16();
 
@@ -1112,7 +1139,7 @@ static void	generate_font_bitmaps(std::vector<rendered_glyph_info>& glyph_info,
 
 			if (owner->get_create_bitmaps() == DO_LOAD_BITMAPS)
 			{
-			smart_ptr<bitmap_info>	bi;
+			boost::intrusive_ptr<bitmap_info>	bi;
 				// load bitmap contents
 				if (s_current_cache_image == NULL || w != pw || h != ph)
 				{
@@ -1128,8 +1155,10 @@ static void	generate_font_bitmaps(std::vector<rendered_glyph_info>& glyph_info,
 					w,
 					h,
 					s_current_cache_image);
-			owner->add_bitmap_info(bi.get_ptr());
+			owner->add_bitmap_info(bi.get());
+#ifndef GNASH_USE_GC
 			assert(bi->get_ref_count() == 2);	// one ref for bi, one for the owner.
+#endif // ndef GNASH_USE_GC
 			}
 			else { 	// Skip image data bytes.
 				in->set_position(in->get_position() + w * h);
@@ -1202,17 +1231,17 @@ static void	generate_font_bitmaps(std::vector<rendered_glyph_info>& glyph_info,
 				tg.m_uv_origin.m_x = in->read_float32();
 				tg.m_uv_origin.m_y = in->read_float32();
 
-				if (glyph_index < 0 || glyph_index >= fnt->get_glyph_count())
+				if (glyph_index < 0 || glyph_index >= fnt->getEmbedGlyphCount())
 				{
 					// Cached data doesn't match this font!
 					log_error("invalid glyph index %d in cached font data, limit is %d, font is '%s'\n",
 						  glyph_index,
-						  fnt->get_glyph_count(),
-						  fnt->get_name());
+						  fnt->getEmbedGlyphCount(),
+						  fnt->get_name().c_str());
 				}
 				else
 				{
-					fnt->add_texture_glyph(glyph_index, tg);
+					fnt->add_texture_glyph(glyph_index, tg, embed);
 				}
 			}
 
@@ -1232,6 +1261,13 @@ static void	generate_font_bitmaps(std::vector<rendered_glyph_info>& glyph_info,
 		s_fonts.clear();
 	}
 
+boost::intrusive_ptr<font>
+get_default_font()
+{
+	if ( _defaultFont ) return _defaultFont;
+	_defaultFont = new font(DEFAULT_FONT_NAME);
+	return _defaultFont;
+}
 
 	int	get_font_count()
 	// Return the number of fonts in our library.
@@ -1248,20 +1284,20 @@ static void	generate_font_bitmaps(std::vector<rendered_glyph_info>& glyph_info,
 			return NULL;
 		}
 
-		return s_fonts[index].get_ptr();
+		return s_fonts[index].get();
 	}
 
 
-	font*	get_font(const char* name)
+	font*	get_font(const std::string& name)
 	// Return the named font.
 	{
 		// Dumb linear search.
 		for (unsigned int i = 0; i < s_fonts.size(); i++)
 		{
-			font*	f = s_fonts[i].get_ptr();
+			font*	f = s_fonts[i].get();
 			if (f != NULL)
 			{
-				if (strcmp(f->get_name(), name) == 0)
+				if (f->get_name() == name)
 				{
 					return f;
 				}
@@ -1269,19 +1305,6 @@ static void	generate_font_bitmaps(std::vector<rendered_glyph_info>& glyph_info,
 		}
 		return NULL;
 	}
-			
-
-	const char*	get_font_name(const font* f)
-	// Return the name of the given font.  (This basically exists
-	// so that font* can be opaque to the host app).
-	{
-		if (f == NULL)
-		{
-			return "<null>";
-		}
-		return f->get_name();
-	}
-
 
 	void	add_font(font* f)
 	// Add the given font to our library.
@@ -1300,7 +1323,7 @@ static void	generate_font_bitmaps(std::vector<rendered_glyph_info>& glyph_info,
 	}
 
 
-	void	draw_glyph(const matrix& mat, const texture_glyph& tg, rgba color, int nominal_glyph_height)
+	void	draw_glyph(const matrix& mat, const texture_glyph& tg, const rgba& color, int nominal_glyph_height)
 	// Draw the given texture glyph using the given transform, in
 	// the given color.
 	{
@@ -1309,6 +1332,12 @@ static void	generate_font_bitmaps(std::vector<rendered_glyph_info>& glyph_info,
 		// @@ worth it to precompute these bounds?
 
 		rect	bounds = tg.m_uv_bounds;
+		if ( bounds.is_null() )
+		{
+			//log_debug("Textured glyph rendering skipped, since it's bounds are null");
+			return;
+		}
+
 		bounds.shift_x (-tg.m_uv_origin.m_x);
 		bounds.shift_y (-tg.m_uv_origin.m_y);
 
@@ -1316,10 +1345,12 @@ static void	generate_font_bitmaps(std::vector<rendered_glyph_info>& glyph_info,
 		// @@ need to factor this out!
 		static float	s_scale = GLYPH_CACHE_TEXTURE_SIZE * s_rendering_box / nominal_glyph_height;
 
+		//log_msg("Scaling bounds %s by factor %g (nominal_glyph_height: %d)", bounds.toString().c_str(), s_scale, nominal_glyph_height);
+
 		bounds.scale_x(s_scale);
 		bounds.scale_y(s_scale);
 		
-		render::draw_bitmap(mat, tg.m_bitmap_info.get_ptr(), bounds, tg.m_uv_bounds, color);
+		render::draw_bitmap(mat, tg.m_bitmap_info.get(), bounds, tg.m_uv_bounds, color);
 	}
 
 
