@@ -4,7 +4,7 @@
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
@@ -17,7 +17,7 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-/* $Id: impl.cpp,v 1.108 2007/06/01 11:02:18 strk Exp $ */
+/* $Id: impl.cpp,v 1.114 2007/08/08 18:26:38 strk Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -53,6 +53,9 @@
 #include "RemoveObjectTag.h"
 #include "DoActionTag.h"
 #include "sound_handler.h" // for get_sound_handler
+#ifdef GNASH_USE_GC
+#include "GC.h"
+#endif
 
 #include <string>
 #include <map>
@@ -61,6 +64,7 @@
 namespace gnash
 {
 
+static void	clear_library();
 
 /// Namespace for global data (will likely turn into a class)
 namespace globals { // gnash::globals
@@ -233,6 +237,8 @@ static void	ensure_loaders_registered()
 	register_tag_loader(SWF::DEFINESCALINGGRID, fixme_loader); // 78
 	register_tag_loader(SWF::DEFINESHAPE4, fixme_loader); // 83
 	register_tag_loader(SWF::DEFINEMORPHSHAPE2, fixme_loader); // 84
+
+	register_tag_loader(SWF::REFLEX, reflex_loader); // 777
 }
 
 
@@ -538,8 +544,6 @@ movie_definition*	create_movie_no_recurse(
 
     s_no_recurse_while_loading = false;
 
-    //m->add_ref();
-    /
     return m;
 }
 #endif
@@ -569,7 +573,7 @@ void	clear()
     // See task task #6959 and depending items
     //
     std::cerr << "Any segfault past this message is likely due to improper threads cleanup." << std::endl;
-    exit(EXIT_SUCCESS);
+    //exit(EXIT_SUCCESS);
 
     clear_library();
     fontlib::clear();
@@ -578,8 +582,12 @@ void	clear()
     // after it's been de-referenced
     set_sound_handler(NULL);
 
-}
+#ifdef GNASH_USE_GC 
+    GC::get().collect();
 
+    GC::cleanup();
+#endif
+}
 
 //
 // library stuff, for sharing resources among different movies.
@@ -621,6 +629,17 @@ public:
 		}
 	}
 
+#ifdef GNASH_USE_GC
+	/// Mark all library elements as reachable (for GC)
+	void markReachableResources() const
+	{
+		for ( container::const_iterator i=_map.begin(), e=_map.end(); i!=e; ++i)
+		{
+			i->second->setReachable();
+		}
+	}
+#endif
+
 	void add(const std::string& key, movie_definition* mov)
 	{
 		_map[key] = mov;
@@ -631,7 +650,8 @@ public:
 
 static MovieLibrary s_movie_library;
 
-static hash< movie_definition*, boost::intrusive_ptr<sprite_instance> >	s_movie_library_inst;
+typedef std::map< movie_definition*, boost::intrusive_ptr<sprite_instance> > library_container_t;
+static library_container_t	s_movie_library_inst;
 static std::vector<sprite_instance*> s_extern_sprites;
 
 static std::string s_workdir;
@@ -641,25 +661,6 @@ void save_extern_movie(sprite_instance* m)
     s_extern_sprites.push_back(m);
 }
 
-//#if 0
-void delete_unused_root()
-{
-    for (unsigned int i = 0; i < s_extern_sprites.size(); i++)
-	{
-	    sprite_instance* root_m = s_extern_sprites[i];
-	    sprite_instance* m = root_m->get_root_movie();
-      
-	    if (m->get_ref_count() < 2)
-		{
-		    log_action(_("extern movie deleted"));
-		    s_extern_sprites.erase(s_extern_sprites.begin() + i);
-		    i--;
-		    root_m->drop_ref();
-		}
-	}
-}
-//#endif // 0
-
 movie_root*
 get_current_root()
 {
@@ -677,7 +678,7 @@ void set_workdir(const char* dir)
     s_workdir = dir;
 }
 
-void	clear_library()
+static void	clear_library()
     // Drop all library references to movie_definitions, so they
     // can be cleaned up.
 {
@@ -702,8 +703,6 @@ movie_definition* create_library_movie(const URL& url, const char* real_url, boo
 	if ( s_movie_library.get(cache_label, &m) )
 	    {
     		log_msg(_("Movie %s already in library"), cache_label.c_str());
-		// Return cached movie.
-		// m->add_ref(); let caller add the ref, if needed
 		return m.get();
 	    }
     }
@@ -748,12 +747,11 @@ sprite_instance* create_library_movie_inst(movie_definition* md)
 {
     // Is the movie instance already in the library?
     {
-	boost::intrusive_ptr<sprite_instance>	m;
-	s_movie_library_inst.get(md, &m);
-	if (m != NULL)
+	library_container_t::const_iterator i = s_movie_library_inst.find(md);
+	if (i != s_movie_library_inst.end())
 	    {
 		// Return cached movie instance.
-		// m->add_ref(); // let caller increment refcount
+		boost::intrusive_ptr<sprite_instance>   m((*i).second);
 		return m.get();
 	    }
     }
@@ -769,10 +767,9 @@ sprite_instance* create_library_movie_inst(movie_definition* md)
 	}
     else
 	{
-	    s_movie_library_inst.add(md, mov);
+	    s_movie_library_inst[md] = mov;
 	}
 
-    // mov->add_ref(); // let caller increment refcount
     return mov;
 }
 
@@ -873,9 +870,42 @@ void	precompute_cached_data(movie_definition* movie_def)
 		}
 	}
 
+#ifndef GNASH_USE_GC
     m->drop_ref();
+#endif //ndef GNASH_USE_GC
 }
 
+#ifdef GNASH_USE_GC
+/// A GC root used to mark all reachable collectable pointers
+class GnashGcRoot : public GcRoot 
+{
+
+public:
+
+	GnashGcRoot()
+	{
+	}
+
+	void markReachableResources() const
+	{
+		VM::get().markReachableResources();
+
+		// Mark library movies (TODO: redesign this part)
+		s_movie_library.markReachableResources();
+	}
+};
+#endif
+
+void gnashInit()
+{
+#ifdef GNASH_USE_GC
+	static GnashGcRoot gcRoot;
+	GC::init(gcRoot);
+#endif
+}
+
+
+
 
 } // namespace gnash
 
