@@ -4,7 +4,7 @@
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
@@ -30,6 +30,11 @@
 #include "tu_random.h"
 #include "ExecutableCode.h"
 #include "Stage.h"
+#include "utility.h"
+#include "URL.h"
+#ifdef NEW_KEY_LISTENER_LIST_DESIGN
+  #include "action.h"
+#endif
 
 #include <iostream>
 #include <string>
@@ -46,7 +51,7 @@ inline bool
 movie_root::testInvariant() const
 {
 	// TODO: fill this function !
-	assert(_movie.get());
+	assert( ! _movies.empty() );
 
 	return true;
 }
@@ -59,7 +64,7 @@ movie_root::movie_root()
 	m_viewport_width(1),
 	m_viewport_height(1),
 	m_pixel_scale(1.0f),
-	m_background_color(0, 0, 0, 255),
+	m_background_color(255, 255, 255, 255),
 	m_timer(0.0f),
 	m_mouse_x(0),
 	m_mouse_y(0),
@@ -68,6 +73,7 @@ movie_root::movie_root()
 	m_on_event_xmlsocket_ondata_called(false),
 	m_on_event_xmlsocket_onxml_called(false),
 	m_on_event_load_progress_called(false),
+	_lastTimerId(0),
 	m_active_input_text(NULL),
 	m_time_remainder(0.0f),
 	m_drag_state(),
@@ -83,24 +89,108 @@ movie_root::~movie_root()
 	{
 		delete *it;
 	}
+
+	for (TimerMap::iterator it=_intervalTimers.begin(),
+			itE=_intervalTimers.end();
+			it != itE; ++it)
+	{
+		delete it->second;
+	}
 	assert(testInvariant());
 }
 
 void
 movie_root::setRootMovie(movie_instance* movie)
 {
-	assert(movie != NULL);
-	_movie = movie;
+	setLevel(0, movie);
+}
 
-	_movie->set_invalidated();
+/* private */
+void
+movie_root::setLevel(unsigned int num, boost::intrusive_ptr<movie_instance> movie)
+{
+	assert(movie != NULL);
+	assert(static_cast<unsigned int>(movie->get_depth()) == num);
+
+	//movie->setLevel(num)
+	//movie->set_depth(num);
+	//movie->set_name(ss.str().c_str());
+
+	//if ( _movies.size() < num+1 ) _movies.resize(num+1);
+	_movies[num] = movie; // [num] = movie;
+
+	movie->set_invalidated();
 	
 	set_display_viewport(0, 0,
-		(int) _movie->get_movie_definition()->get_width_pixels(),
-		(int) _movie->get_movie_definition()->get_height_pixels());
+		(int) movie->get_movie_definition()->get_width_pixels(),
+		(int) movie->get_movie_definition()->get_height_pixels());
 
 	assert(testInvariant());
 }
 
+bool
+movie_root::loadLevel(unsigned int num, const URL& url)
+{
+	boost::intrusive_ptr<movie_definition> md ( create_library_movie(url) );
+	if (md == NULL)
+	{
+		log_error(_("can't create movie_definition for %s"),
+			url.str().c_str());
+		return false;
+	}
+
+	boost::intrusive_ptr<movie_instance> extern_movie;
+	extern_movie = md->create_movie_instance();
+	if (extern_movie == NULL)
+	{
+		log_error(_("can't create extern movie_instance "
+			"for %s"), url.str().c_str());
+		return false;
+	}
+
+	// Parse query string
+	sprite_instance::VariableMap vars;
+	url.parse_querystring(url.querystring(), vars);
+	extern_movie->setVariables(vars);
+
+	character* ch = extern_movie.get();
+	ch->set_depth(num);
+
+	save_extern_movie(extern_movie.get());
+
+	setLevel(num, extern_movie);
+
+	return true;
+}
+
+boost::intrusive_ptr<movie_instance>
+movie_root::getLevel(unsigned int num) const
+{
+	Levels::const_iterator i = _movies.find(num);
+	if ( i == _movies.end() ) return 0;
+
+	assert(boost::dynamic_pointer_cast<movie_instance>(i->second));
+	return boost::static_pointer_cast<movie_instance>(i->second);
+}
+
+void
+movie_root::restart()
+{
+	for (Levels::iterator i=_movies.begin(), e=_movies.end(); i!=e; ++i)
+	{
+		i->second->restart();
+	}
+}
+
+void
+movie_root::clear_invalidated()
+{
+	for (Levels::iterator i=_movies.begin(), e=_movies.end(); i!=e; ++i)
+	{
+		i->second->clear_invalidated();
+	}
+}
+
 boost::intrusive_ptr<Stage>
 movie_root::getStageObject()
 {
@@ -108,7 +198,7 @@ movie_root::getStageObject()
 	if ( ! VM::isInitialized() ) return NULL;
 	as_object* global = VM::get().getGlobal();
 	if ( ! global ) return NULL;
-	if ( ! global->get_member("Stage", &v) ) return NULL;
+	if ( ! global->get_member(PROPNAME("Stage"), &v) ) return NULL;
 	return boost::dynamic_pointer_cast<Stage>(v.to_object());
 }
 		
@@ -127,7 +217,7 @@ movie_root::set_display_viewport(int x0, int y0, int w, int h)
 		//log_msg("Rescaling allowed");
 
 		// should we cache this ? it's immutable after all !
-		const rect& frame_size = _movie->get_frame_size();
+		const rect& frame_size = _movies[0]->get_frame_size();
 
 		float	scale_x = m_viewport_width / TWIPS_TO_PIXELS(frame_size.width());
 		float	scale_y = m_viewport_height / TWIPS_TO_PIXELS(frame_size.height());
@@ -157,23 +247,19 @@ movie_root::notify_mouse_moved(int x, int y)
 
 }
 
-
-
-key_as_object *
-movie_root::notify_global_key(key::code k, bool down)
+boost::intrusive_ptr<key_as_object>
+movie_root::getKeyObject()
 {
 	VM& vm = VM::get();
-	if ( vm.getSWFVersion() < 6 )
-	{
-		// _global.Key was added in SWF6
-		return NULL; 
-	}
 
-	static boost::intrusive_ptr<key_as_object> keyobject = NULL;
-	if ( ! keyobject )
+	// TODO: test what happens with the global "Key" object
+	//       is removed or overridden by the user
+
+	if ( ! _keyobject )
 	{
-		// This isn't very performant... do we allow user override
-		// of _global.Key, btw ?
+		// This isn't very performant... 
+		// it will keep trying to find it even if impossible
+		// to find.
 
 		as_value kval;
 		as_object* global = VM::get().getGlobal();
@@ -188,21 +274,36 @@ movie_root::notify_global_key(key::code k, bool down)
 			//log_msg("Found member 'Key' in _global: %s", kval.to_string());
 			boost::intrusive_ptr<as_object> obj = kval.to_object();
 			//log_msg("_global.Key to_object() : %s @ %p", typeid(*obj).name(), obj);
-			keyobject = boost::dynamic_pointer_cast<key_as_object>( obj );
+			_keyobject = boost::dynamic_pointer_cast<key_as_object>( obj );
 		}
 	}
 
+	return _keyobject;
+}
+
+
+key_as_object *
+movie_root::notify_global_key(key::code k, bool down)
+{
+	VM& vm = VM::get();
+	if ( vm.getSWFVersion() < 5 )
+	{
+		// Key was added in SWF5
+		return NULL; 
+	}
+
+	boost::intrusive_ptr<key_as_object> keyobject = getKeyObject();
 	if ( keyobject )
 	{
-		if (down) keyobject->set_key_down(k);
-		else keyobject->set_key_up(k);
+		if (down) _keyobject->set_key_down(k);
+		else _keyobject->set_key_up(k);
 	}
 	else
 	{
 		log_error("gnash::notify_key_event(): _global.Key doesn't exist, or isn't the expected built-in\n");
 	}
 
-	return keyobject.get();
+	return _keyobject.get();
 }
 
 bool
@@ -217,7 +318,8 @@ movie_root::notify_key_event(key::code k, bool down)
 
 	// Notify character key listeners.
 	notify_key_listeners(k, down);
-	
+
+#ifndef NEW_KEY_LISTENER_LIST_DESIGN
 	// Notify both character and non-character Key listeners
 	//	for user defined handerlers.
 	// FIXME: this may violates the event order
@@ -231,7 +333,7 @@ movie_root::notify_key_event(key::code k, bool down)
 		else
 			global_key->notify_listeners(event_id::KEY_UP);
 	}
-	
+#endif	
 	processActionQueue();
 
 	return false; // should return true if needs update ...
@@ -315,7 +417,9 @@ generate_mouse_button_events(mouse_button_state* ms)
 				// onDragOut
 				if (active_entity != NULL)
 				{
+#ifndef GNASH_USE_GC
 					assert(active_entity->get_ref_count() > 1); // we are NOT the only object holder !
+#endif // GNASH_USE_GC
 					active_entity->on_button_event(event_id::DRAG_OUT);
 					// TODO: have on_button_event return
 					//       wheter the action must trigger
@@ -454,8 +558,7 @@ movie_root::fire_mouse_event()
 	assert(testInvariant());
 
     // Generate a mouse event
-    m_mouse_button_state.m_topmost_entity =
-        _movie->get_topmost_mouse_entity(PIXELS_TO_TWIPS(m_mouse_x), PIXELS_TO_TWIPS(m_mouse_y));
+    m_mouse_button_state.m_topmost_entity = getTopmostMouseEntity(PIXELS_TO_TWIPS(m_mouse_x), PIXELS_TO_TWIPS(m_mouse_y));
     m_mouse_button_state.m_mouse_button_state_current = (m_mouse_buttons & 1);
 
     bool need_redraw = generate_mouse_button_events(&m_mouse_button_state);
@@ -502,52 +605,43 @@ movie_root::set_drag_state(const drag_state& st)
 	assert(testInvariant());
 }
 
-#if 0 // see comments in movie_root.h
-void
-movie_root::get_url(const char *url)
-{
-    GNASH_REPORT_FUNCTION;
-
-	// nobody should use this function
-	assert(0);
-    
-    string command = "mozilla -remote \"openurl";
-    command += url;
-    command += ")\"";
-    log_msg (_("Launching URL... %s"), command.c_str());
-    system(command.c_str());
-}
-#endif
-
 unsigned int
-movie_root::add_interval_timer(const Timer& timer)
+movie_root::add_interval_timer(std::auto_ptr<Timer> timer, bool internal)
 {
+	assert(timer.get());
 	assert(testInvariant());
 			
-	int id = _intervalTimers.size()+1;
+	int id = ++_lastTimerId;
+	if ( internal ) id = -id;
+
 	if ( _intervalTimers.size() >= 255 )
 	{
+		// TODO: Why this limitation ? 
 		log_error("FIXME: " SIZET_FMT " timers currently active, won't add another one", _intervalTimers.size());
 	}
 
-	// TODO: find first NULL element in vector for reuse ?
-	_intervalTimers.push_back(timer);
+	assert(_intervalTimers.find(id) == _intervalTimers.end());
+	_intervalTimers[id] = timer.release(); 
 	return id;
 }
 	
 bool
 movie_root::clear_interval_timer(unsigned int x)
 {
-	if ( ! x || x > _intervalTimers.size() ) return false;
+	TimerMap::iterator it = _intervalTimers.find(x);
+	if ( it == _intervalTimers.end() ) return false;
 
-	Timer& timer = _intervalTimers[x-1];
-
-	// will make sure next expire() will always return false!
-	timer.clearInterval();
-
-	assert(testInvariant());
+	// We do not remove the element here because
+	// we might have been called during execution
+	// of another timer, thus during a scan of the _intervalTimers
+	// container. If we use erase() here, the iterators in executeTimers
+	// would be invalidated. Rather, executeTimers() would check container
+	// elements for being still active and remove the cleared one in a safe way
+	// at each iteration.
+	it->second->clearInterval();
 
 	return true;
+
 }
 	
 void
@@ -555,59 +649,30 @@ movie_root::advance(float delta_time)
 {
 	// GNASH_REPORT_FUNCTION;
 
-	// TODO: wrap this in a executeTimers() method 
-	for (TimerList::iterator it=_intervalTimers.begin(),
-			itEnd=_intervalTimers.end();
-			it != itEnd;
-			++it)
-	{
-		Timer& timer = *it;
-		if ( timer.expired() )
-		{
-			// log_msg("FIXME: Interval Timer Expired!\n");
-			//_movie->on_event_interval_timer();
-			timer();
-		}
-	}
+	// Execute expired timers
+	executeTimers();
 
+#ifndef NEW_KEY_LISTENER_LIST_DESIGN
 	// Cleanup key listeners (remove unloaded characters)
 	// FIXME: not all key listeners could be cleaned here!
 	// (eg. characters unloaded by loop-back won't be cleared until next advancement)
 	cleanup_key_listeners();
-
+#endif
 	// random should go continuously that:
 	// 1. after restart of the player the situation has not repeated
 	// 2. by different machines the random gave different numbers
 	tu_random::next_random();
 			
-#ifdef GNASH_DEBUG
-	size_t totframes = _movie->get_frame_count();
-	size_t prevframe = _movie->get_current_frame();
+	advanceAllLevels(delta_time);
+
+#ifdef NEW_KEY_LISTENER_LIST_DESIGN
+	cleanup_key_listeners();
 #endif
-
-	// Keep root sprite alive during actions execution.
-	//
-	// This is *very* important, as actions in the movie itself
-	// could get rid of it. A simple example:
-	//
-	// 	_root.loadMovie(other);
-	//
-	boost::intrusive_ptr<sprite_instance> keepMovieAlive(_movie.get());
-
-	_movie->advance(delta_time);
-
 	processActionQueue();
 
-#ifdef GNASH_DEBUG
-	size_t curframe = _movie->get_current_frame();
-
-	log_msg("movie_root::advance advanced top-level movie from "
-			SIZET_FMT "/" SIZET_FMT
-			" to " SIZET_FMT "/" SIZET_FMT
-			" (_movie is %s%s)",
-			prevframe, totframes, curframe, totframes,
-			typeid(*_movie).name(),
-			_movie->get_play_state() == sprite_instance::STOP ? " - now in STOP mode" : "");
+#ifdef GNASH_USE_GC
+	// Run the garbage collector (step back !!)
+	GC::get().collect();
 #endif
 
 	assert(testInvariant());
@@ -621,20 +686,8 @@ movie_root::display()
 
 	assert(testInvariant());
 
-	_movie->clear_invalidated();
-
-//  	    GNASH_REPORT_FUNCTION;
-	if (_movie->get_visible() == false)
-        {
-            // Don't display.
-            return;
-        }
-
 	// should we cache this ? it's immutable after all !
-	const rect& frame_size = _movie->get_frame_size();
-
-	// null frame size ? don't display !
-	if ( frame_size.is_null() ) return;
+	const rect& frame_size = getLevel(0)->get_frame_size();
 
 	render::begin_display(
 		m_background_color,
@@ -643,7 +696,27 @@ movie_root::display()
 		frame_size.get_x_min(), frame_size.get_x_max(),
 		frame_size.get_y_min(), frame_size.get_y_max());
 
-	_movie->display();
+
+	for (Levels::iterator i=_movies.begin(), e=_movies.end(); i!=e; ++i)
+	{
+		boost::intrusive_ptr<sprite_instance> movie = i->second;
+
+		movie->clear_invalidated();
+
+		if (movie->get_visible() == false) continue;
+
+		// null frame size ? don't display !
+		const rect& sub_frame_size = movie->get_frame_size();
+
+		if ( frame_size.is_null() )
+		{
+			log_debug("_level%u has null frame size, skipping", i->first);
+			continue;
+		}
+
+		movie->display();
+
+	}
 
 	render::end_display();
 }
@@ -657,7 +730,7 @@ movie_root::call_method(const char* method_name,
 
 	va_list	args;
 	va_start(args, method_arg_fmt);
-	const char* result = _movie->call_method_args(method_name,
+	const char* result = getLevel(0)->call_method_args(method_name,
 		method_arg_fmt, args);
 	va_end(args);
 
@@ -669,18 +742,153 @@ char* movie_root::call_method_args(const char* method_name,
 		const char* method_arg_fmt, va_list args)
 {
 	assert(testInvariant());
-	return _movie->call_method_args(method_name, method_arg_fmt, args);
+	return getLevel(0)->call_method_args(method_name, method_arg_fmt, args);
 }
 
+#ifdef NEW_KEY_LISTENER_LIST_DESIGN
+
 void movie_root::cleanup_key_listeners()
 {
 #ifdef KEY_LISTENERS_DEBUG
-	size_t prevsize = m_key_listeners.size();
-	log_msg("Cleaning up %u key listeners", m_key_listeners.size());
+    size_t prevsize = _keyListeners.size();
+    log_msg("Cleaning up %u key listeners", _keyListeners.size());
 #endif
 
-	for (ListenerSet::iterator iter = m_key_listeners.begin();
-			 iter != m_key_listeners.end(); )
+    for (KeyListeners::iterator iter = _keyListeners.begin();
+        iter != _keyListeners.end(); )
+    {
+        // The listener object has no registered key event handlers, remove it.
+        if( !iter->hasUserRegistered() && !iter->hasOnClipRegistered() )
+        {
+            _keyListeners.erase(iter++);
+        }
+        else 
+        {
+            boost::intrusive_ptr<as_object> obj = iter->get();
+			character* ch = dynamic_cast<character*>(obj.get());
+            // The listener object is unloaded, remove it.
+            // TODO: Don't do this again in the character destructors. should we?
+            if ( ch && ch->isUnloaded() ) 
+            {
+                _keyListeners.erase(iter++);
+            }
+            else
+                ++iter;
+        }
+    }
+
+#ifdef KEY_LISTENERS_DEBUG
+    size_t currsize = _keyListeners.size();
+    log_msg("Cleaned up %u listeners (from %u to %u)", prevsize-currsize, prevsize, currsize);
+#endif
+}
+
+void movie_root::notify_key_listeners(key::code k, bool down)
+{
+    //log_msg("Notifying " SIZET_FMT " keypress listeners", _keyListeners.size());
+
+    as_environment env;
+
+    for (KeyListeners::iterator iter = _keyListeners.begin();
+        iter != _keyListeners.end(); ++iter)
+    {
+		boost::intrusive_ptr<as_object> obj = iter->get();
+        character* ch = dynamic_cast<character*>(obj.get());
+        // notify character listeners
+        if ( ch && ! ch->isUnloaded() ) 
+        {
+            if(down)
+            {
+                // invoke onClipKeyDown handler
+                ch->on_event(event_id(event_id::KEY_DOWN, key::INVALID));
+                
+                if(iter->hasUserRegistered())
+                // invoke onKeyDown handler
+                {
+                    boost::intrusive_ptr<as_function> 
+                        method = ch->getUserDefinedEventHandler("onKeyDown");
+                    if ( method )
+                    {
+                        call_method0(as_value(method.get()), &env, ch);
+                    }
+                }
+                // invoke onClipKeyPress handler
+                ch->on_event(event_id(event_id::KEY_PRESS, k));
+            }
+            else
+            {
+                //invoke onClipKeyUp handler
+                ch->on_event(event_id(event_id::KEY_UP, key::INVALID));   
+
+                if(iter->hasUserRegistered())
+                // invoke onKeyUp handler
+                {
+                    boost::intrusive_ptr<as_function> 
+                        method = ch->getUserDefinedEventHandler("onKeyUp");
+                    if ( method )
+                    {
+                        call_method0(as_value(method.get()), &env, ch);
+                    }
+                }
+            }
+        }
+        // notify non-character listeners
+        else 
+        {
+            if(down) 
+            {
+                iter->get()->on_event(event_id(event_id::KEY_DOWN, key::INVALID));
+            }
+            else 
+            {
+                iter->get()->on_event(event_id(event_id::KEY_UP, key::INVALID));
+            }
+        }
+    }
+    assert(testInvariant());
+}
+
+void movie_root::add_key_listener(const KeyListener & listener)
+{
+    KeyListeners::iterator target = _keyListeners.find(listener);
+    if(target == _keyListeners.end())
+    // The key listener is not in the container, then add it.
+    {
+        _keyListeners.insert(listener);
+    }
+    else
+    // The key listener is already in the container, then register it(again).
+    {
+        if(listener.hasUserRegistered())
+        {
+            target->registerUserHandler();
+        }
+        if(listener.hasOnClipRegistered())
+        {
+            target->registerOnClipHandler();
+        }
+    }
+
+    assert(testInvariant());
+}
+
+void movie_root::remove_key_listener(as_object* listener)
+{
+    _keyListeners.erase(KeyListener(listener));
+
+    assert(testInvariant());
+}
+
+#else // ndef NEW_KEY_LISTENER_LIST_DESIGN
+
+void movie_root::cleanup_key_listeners()
+{
+#ifdef KEY_LISTENERS_DEBUG
+	size_t prevsize = _keyListeners.size();
+	log_msg("Cleaning up %u key listeners", _keyListeners.size());
+#endif
+
+	for (ListenerSet::iterator iter = m_key_listeners.begin(); iter != m_key_listeners.end(); )
 	{
 		// TODO: handle non-character objects too !
 		character* ch = dynamic_cast<character*>(iter->get());
@@ -698,7 +906,7 @@ void movie_root::cleanup_key_listeners()
 	}
 
 #ifdef KEY_LISTENERS_DEBUG
-	size_t currsize = m_key_listeners.size();
+	size_t currsize = _keyListeners.size();
 	log_msg("Cleaned up %u listeners (from %u to %u)", prevsize-currsize, prevsize, currsize);
 #endif
 }
@@ -750,6 +958,7 @@ void movie_root::remove_key_listener(as_object* listener)
 	//log_msg("After removing key listener %p, %u listeners are left", (void*)listener, m_key_listeners.size());
 	assert(testInvariant());
 }
+#endif // ndef NEW_KEY_LISTENER_LIST_DESIGN
 
 void movie_root::add_mouse_listener(as_object* listener)
 {
@@ -816,7 +1025,10 @@ movie_root::isMouseOverActiveEntity() const
 void
 movie_root::add_invalidated_bounds(InvalidatedRanges& ranges, bool force)
 {
-	_movie->add_invalidated_bounds(ranges, force);
+	for (Levels::reverse_iterator i=_movies.rbegin(), e=_movies.rend(); i!=e; ++i)
+	{
+		i->second->add_invalidated_bounds(ranges, force);
+	}
 }
 
 void
@@ -871,5 +1083,142 @@ movie_root::pushAction(boost::intrusive_ptr<as_function> func, boost::intrusive_
 	_actionQueue.push_back(new FunctionCode(func, target));
 }
 
+/* private */
+void
+movie_root::executeTimers()
+{
+	for (TimerMap::iterator it=_intervalTimers.begin(), itEnd=_intervalTimers.end();
+			it != itEnd; )
+	{
+		// Get an iterator to next element, as we'll use
+		// erase to drop cleared timers, and that would
+		// invalidate the current iterator.
+		//
+		// FYI: it's been reported on ##iso-c++ that next
+		//      C++ version will fix std::map<>::erase(iterator)
+		//      to return the next valid iterator,
+		//      like std::list<>::erase(iterator) does.
+		//      For now, we'll have to handle this manually)
+		//
+		TimerMap::iterator nextIterator = it;
+		++nextIterator;
+
+		Timer* timer = it->second;
+
+		if ( timer->cleared() )
+		{
+			// this timer was cleared, erase it
+			delete timer;
+			_intervalTimers.erase(it);
+		}
+		else
+		{
+			if ( timer->expired() )
+			{
+				//cout << " EXPIRED, start time is now " << timer.getStart() << endl;
+				(*timer)();
+			}
+		}
+
+		it = nextIterator;
+	}
+
+}
+
+#ifdef GNASH_USE_GC
+void
+movie_root::markReachableResources() const
+{
+	// Mark movie levels as reachable
+	for (Levels::const_reverse_iterator i=_movies.rbegin(), e=_movies.rend(); i!=e; ++i)
+	{
+		i->second->setReachable();
+	}
+
+	// Mark mouse entities 
+	m_mouse_button_state.markReachableResources();
+	
+	// Mark timer targets
+	for (TimerMap::const_iterator i=_intervalTimers.begin(), e=_intervalTimers.end();
+			i != e; ++i)
+	{
+		i->second->markReachableResources();
+	}
+
+	// Mark resources reachable by queued action code
+	for (ActionQueue::const_iterator i=_actionQueue.begin(), e=_actionQueue.end();
+			i != e; ++i)
+	{
+		(*i)->markReachableResources();
+	}
+
+#ifdef NEW_KEY_LISTENER_LIST_DESIGN
+	// Mark key listeners
+	for (KeyListeners::const_iterator i=_keyListeners.begin(), e=_keyListeners.end();
+			i != e; ++i)
+	{
+		i->setReachable();
+	}
+#else
+	// Mark key listeners
+	for (ListenerSet::const_iterator i=m_key_listeners.begin(), e=m_key_listeners.end();
+			i != e; ++i)
+	{
+		(*i)->setReachable();
+	}
+#endif
+
+	// Mark global key object
+	if ( _keyobject ) _keyobject->setReachable();
+}
+#endif // GNASH_USE_GC
+
+character *
+movie_root::getTopmostMouseEntity(float x, float y)
+{
+	for (Levels::reverse_iterator i=_movies.rbegin(), e=_movies.rend(); i!=e; ++i)
+	{
+		character* ret = i->second->get_topmost_mouse_entity(x, y);
+		if ( ret ) return ret;
+	}
+	return NULL;
+}
+
+void
+movie_root::advanceAllLevels(float delta_time)
+{
+	// scan a backup copy of the levels, so that movies advancement won't
+	// invalidate iterators
+	Levels cached = _movies;
+	for (Levels::reverse_iterator i=cached.rbegin(), e=cached.rend(); i!=e; ++i)
+	{
+		advanceMovie(i->second, delta_time);
+	}
+}
+
+void
+movie_root::advanceMovie(boost::intrusive_ptr<sprite_instance> movie, float delta_time)
+{
+#ifdef GNASH_DEBUG
+	size_t totframes = movie->get_frame_count();
+	size_t prevframe = movie->get_current_frame();
+#endif
+
+	movie->advance(delta_time);
+
+#ifdef GNASH_DEBUG
+	size_t curframe = movie->get_current_frame();
+
+	log_msg("movie_root::advance advanced level %d movie from "
+			SIZET_FMT "/" SIZET_FMT
+			" to " SIZET_FMT "/" SIZET_FMT
+			" (movie is %s%s)",
+			movie->get_depth(), prevframe, totframes, curframe, totframes,
+			typeName(*movie).c_str(),
+			movie->get_play_state() == sprite_instance::STOP ? " - now in STOP mode" : "");
+#endif
+
+}
+
 } // namespace gnash
 
