@@ -4,7 +4,7 @@
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
@@ -17,7 +17,7 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-/* $Id: button_character_def.cpp,v 1.12 2007/05/24 13:46:24 strk Exp $ */
+/* $Id: button_character_def.cpp,v 1.17 2007/07/10 16:16:00 strk Exp $ */
 
 // Based on the public domain work of Thatcher Ulrich <tu@tulrich.com> 2003
 
@@ -38,7 +38,22 @@ namespace gnash {
 
 button_action::~button_action()
 {
-	m_actions.clear();
+	for (ActionList::iterator i=m_actions.begin(), e=m_actions.end();
+			i != e; ++i)
+	{
+		// We can NOT delete action_buffers here becase they 
+		// may contain the action currently being executed and
+		// triggering the deletion.
+		// I'm not really sure about whether this is the problem,
+		// anyway clip_as_button2.swf fails on segfault when clicking
+		// the upper-right button if we delete here.
+		//
+		// TODO: properly implement management of these resources
+		//       which are otherwise just leaking..
+		//
+		//delete (*i);
+	}
+	m_actions.clear(); // this is useless, will be done automatically
 }
 
 void	button_action::read(stream* in, int tag_type)
@@ -92,16 +107,25 @@ button_record::read(stream* in, int tag_type,
 
 	// Get character definition now (safer)
 	m_character_def = m->get_character_def(m_character_id);
+
 	// If no character with given ID is found in the movie
 	// definition, we print an error, but keep parsing.
 	if ( ! m_character_def )
 	{
 		IF_VERBOSE_MALFORMED_SWF(
-		log_swferror(_("button record refer to "
+		log_swferror(_("   button record refer to "
 			"character with id %d, which is not found "
 			"in the chars dictionary"), m_character_id);
 		);
 	}
+	else
+	{
+		IF_VERBOSE_PARSE(
+		log_parse(_("   button record for states %x contain "
+			"character %d (%s)"), flags, m_character_id,
+		        typeName(*m_character_def).c_str());
+		);
+	}
 
 	m_button_layer = in->read_u16();
 	m_button_matrix.read(in);
@@ -333,7 +357,15 @@ button_character_definition::create_character_instance(
 	return ch;
 }
 
+#ifdef GNASH_USE_GC
+void
+button_character_definition::button_sound_info::markReachableResources() const
+{
+	if ( m_sam ) m_sam->setReachable();
 }
+#endif // GNASH_USE_GC
+
+} // namespace gnash
 
 // Local Variables:
 // mode: C++
