@@ -3,7 +3,7 @@
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
@@ -19,7 +19,7 @@
 //
 //
 
-/* $Id: character_def.cpp,v 1.5 2007/05/28 15:41:09 ann Exp $ */
+/* $Id: character_def.cpp,v 1.7 2007/07/03 05:46:02 strk Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -27,6 +27,7 @@
 
 #include "character_def.h"
 #include "generic_character.h"
+#include "render_handler.h" // for destruction of render_cache_manager
 
 namespace gnash
 {
@@ -37,8 +38,13 @@ character_def::create_character_instance(character* parent, int id)
 	return new generic_character(this, parent, id);
 }
 
+character_def::~character_def()
+{
+	delete m_render_cache;
 }
 
+} // end of namespace gnash
+
 // Local Variables:
 // mode: C++
 // indent-tabs-mode: t
