@@ -3,7 +3,7 @@
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
@@ -62,10 +62,14 @@ public:
 		m_no_select(false),
 		m_border(false),
 		m_html(false),
-		m_use_outlines(false),
+		m_use_outlines(false), // For an SWF-defined textfield we'll read
+		                       // this from the tag. Dynamic textfields should
+		                       // use device fonts by default (so not use outline ones)
 		m_font_id(-1),
 		m_font(NULL),
-		m_text_height(1),
+		m_text_height(1), // TODO: initialize to a meaningful value (see sprite_instance::add_textfield)
+		                  //       and make sure get_font_height is not called for rendering purposes
+		                  //       (instead call a method of edit_text_character_def)
 		m_max_length(0),
 		m_alignment(ALIGN_LEFT),
 		m_left_margin(0),
@@ -133,6 +137,16 @@ public:
 		return m_rect;
 	}
 
+	/// Set boundaries of this textfield
+	//
+	/// This method is used for dynamic textfields
+	/// (actionscript created)
+	///
+	void set_bounds(const rect& bounds)
+	{
+		m_rect = bounds;
+	}
+
 	/// Get right margin in twips
 	uint16_t get_right_margin() const {
 		return m_right_margin;
@@ -154,6 +168,14 @@ public:
 		return m_text_height;
 	}
 
+	/// Set height of font  in twips.
+	// 
+	/// Used by dynamically created textfields.
+	///
+	void set_font_height(uint16_t h) {
+		m_text_height = h;
+	}
+
 	/// Get font.
 	//
 	/// Note: use add_ref() on the return if you need to keep
@@ -166,6 +188,11 @@ public:
 		return m_color;
 	}
 
+	/// Set color of the text
+	void set_text_color(const rgba& col) {
+		m_color = col;
+	}
+
 	/// \brief
 	/// Get extra space between lines (in twips).
 	//
@@ -210,7 +237,30 @@ public:
     // there. Should be fixed (remove get_bounds). 
     return get_bounds(); 
   }
+
+	/// Return true if HTML was allowed by definition
+	bool htmlAllowed() const { return m_html; }
+
+	/// Return true if this character definition requested use of device fonts
+	// 
+	/// Used by edit_text_character constructor to set it's default.
+	///
+	bool getUseEmbeddedGlyphs() const 
+	{
+		return m_use_outlines;
+	}
 	
+protected:
+
+#ifdef GNASH_USE_GC
+	/// Mark all reachable resources (for GC)
+	//
+	/// Reachable resources are:
+	///  - The root movie definition (m_root_def) @@ what do we use this for ?
+	///  - The font being used (m_font) 
+	///
+	void markReachableResources() const;
+#endif // GNASH_USE_GC
 
 private:
 
@@ -262,8 +312,11 @@ private:
 
 
 	/// \brief
-	/// When true, use specified SWF internal font. 
-	/// Otherwise, renderer picks a default font
+	/// When true, use specified SWF internal font (embed fonts)
+	/// Otherwise, use specified device font (or a default one if m_font_id is -1)
+	///
+	/// Also known as USE_GLYPH (from Ming)
+	///
 	bool	m_use_outlines;
 
 	int	m_font_id;
