@@ -1,18 +1,21 @@
-// 
-//   Copyright (C) 2005, 2006 Free Software Foundation, Inc.
-// 
+// movie_def_impl.cpp:  Load in Movie definitions, for Gnash.
+//
+//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
-// 
+//
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
+//
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+//
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -20,7 +23,7 @@
 
 #include "movie_def_impl.h"
 #include "movie_definition.h" // for inheritance
-#include "movie_instance.h" // for create_instance()
+#include "sprite_instance.h" // for create_instance()
 #include "tu_file.h"
 #include "zlib_adapter.h"
 #include "stream.h"
@@ -29,16 +32,20 @@
 #include "font.h"
 #include "log.h"
 #include "sprite_instance.h"
+#include "movie_instance.h"
 #include "bitmap_character_def.h"
 #include "swf/TagLoadersTable.h"
 #include "movie_root.h"
+#include "VM.h" // for assertions
+#include "GnashException.h" // for parser exception
+#include "execute_tag.h"
+#include "sound_definition.h" // for sound_sample
+#include <boost/bind.hpp>
 
+#include <memory>
 #include <string>
-#include <unistd.h> 
+#include <unistd.h>
 
-#ifdef HAVE_PTHREADS
-#include <pthread.h>
-#endif
 
 // Increment this when the cache data format changes.
 #define CACHE_FILE_VERSION 4
@@ -48,56 +55,67 @@
 #define FRAMELOAD_CHUNK 0
 
 // Debug frames load
-#undef DEBUG_FRAMES_LOAD 
+#undef DEBUG_FRAMES_LOAD
 
 // Define this this to load movies using a separate thread
 // (undef and it will fully load a movie before starting to play it)
 #define LOAD_MOVIES_IN_A_SEPARATE_THREAD 1
 
 // Debug threads locking
-#undef DEBUG_THREADS_LOCKING 
+#undef DEBUG_THREADS_LOCKING
+
+#if defined(_WIN32) || defined(WIN32)
+#	include <windows.h>
+# define usleep(x) Sleep(x/1000)
+#endif
 
 namespace gnash
 {
 
 MovieLoader::MovieLoader(movie_def_impl& md)
 	:
-	_waiting_for_frame(0),
 	_movie_def(md)
+	,_thread(NULL)
 {
-#ifdef LOAD_MOVIES_IN_A_SEPARATE_THREAD
-	pthread_cond_init(&_frame_reached_condition, NULL);
-	pthread_mutex_init(&_mutex, NULL);
-#endif
 }
 
 MovieLoader::~MovieLoader()
 {
-#ifdef LOAD_MOVIES_IN_A_SEPARATE_THREAD
-	if ( pthread_cond_destroy(&_frame_reached_condition) != 0 )
-	{
-		log_error("Error destroying MovieLoader condition");
-	}
+	//cout << "MovieLoader dtor called" << endl;
 
-	if ( pthread_mutex_destroy(&_mutex) != 0 )
+	// we should assert _movie_def._loadingCanceled
+	// but we're not friend yet (anyone introduce us ?)
+	if ( _thread.get() )
 	{
-		log_error("Error destroying MovieLoader mutex");
+		//cout << "Joining thread.." << endl;
+		_thread->join();
 	}
-#endif
 }
 
-void*
-MovieLoader::execute(void* arg)
+bool
+MovieLoader::started() const
+{
+	boost::mutex::scoped_lock lock(_mutex);
+
+	return _thread.get() != NULL;
+}
+
+bool
+MovieLoader::isSelfThread() const
+{
+	boost::mutex::scoped_lock lock(_mutex);
+
+	if (!_thread.get()) {
+		return false;
+	}
+	boost::thread this_thread;
+	return this_thread == *_thread;
+}
+
+void
+MovieLoader::execute(movie_def_impl* md)
 {
-	movie_def_impl* md = static_cast<movie_def_impl*>(arg);
 	md->read_all_swf();
-	// maybe this frees all resources and that's bad !
-	//pthread_exit(NULL);
-	
-	/* Better to cancel yourself methinks: 'man 3p pthread_cancel' */
-	pthread_cancel(pthread_self());
-        pthread_testcancel();
-	return NULL;
 }
 
 bool
@@ -107,115 +125,21 @@ MovieLoader::start()
 	// don't start MovieLoader thread !
 	assert(0);
 #endif
-	if ( pthread_create(&_thread, NULL, execute, &_movie_def) )
-	{
-		return false;
-	}
+	// We have two sanity checks, started() and isSelfThread() which rely
+	// on boost::thread() returning before they are executed. Therefore,
+	// we must employ locking.
+	// Those tests do seem a bit redundant, though...
+	boost::mutex::scoped_lock lock(_mutex);
 
-	// should set some mutexes ?
+	_thread.reset( new boost::thread(boost::bind(execute, &_movie_def)) );
 
 	return true;
 }
 
-inline void
-MovieLoader::signal_frame_loaded(size_t frameno)
-{
-#ifndef LOAD_MOVIES_IN_A_SEPARATE_THREAD
-	return; 
-#endif
-	if (_waiting_for_frame &&
-		frameno >= _waiting_for_frame )
-	{
-		pthread_cond_signal(&_frame_reached_condition);
-	}
-}
-
-inline void
-MovieLoader::lock()
-{
-#ifndef LOAD_MOVIES_IN_A_SEPARATE_THREAD
-	return; // nothing to do as there's no mutex
-#endif
-
-#ifdef DEBUG_THREADS_LOCKING
-	// debugging
-	if ( pthread_equal(pthread_self(), _thread) ) {
-		log_msg("MovieLoader locking itself");
-	} else {
-		log_msg("MovieLoader being locked by another thread");
-	}
-#endif
-
-	if ( pthread_mutex_lock(&_mutex) != 0 )
-	{
-		log_error("Error locking MovieLoader");
-	}
-
-#ifdef DEBUG_THREADS_LOCKING
-	// debugging
-	if ( pthread_equal(pthread_self(), _thread) ) {
-		log_msg("MovieLoader locked by itself");
-	} else {
-		log_msg("MovieLoader locked by another thread");
-	}
-#endif
-}
-
-inline void
-MovieLoader::unlock()
-{
-#ifndef LOAD_MOVIES_IN_A_SEPARATE_THREAD
-	return;
-#endif
-
-#ifdef DEBUG_THREADS_LOCKING
-	// debugging
-	if ( pthread_equal(pthread_self(), _thread) ) {
-		log_msg("MovieLoader unlocking itself");
-	} else {
-		log_msg("MovieLoader being unlocked by another thread");
-	}
-#endif
-
-	if ( pthread_mutex_unlock(&_mutex) != 0 )
-	{
-		log_error("Error unlocking MovieLoader");
-	}
-
-#ifdef DEBUG_THREADS_LOCKING
-	// debugging
-	if ( pthread_equal(pthread_self(), _thread) ) {
-		log_msg("MovieLoader unlocked itself");
-	} else {
-		log_msg("MovieLoader unlocked by another thread");
-	}
-#endif
-}
-
-void
-MovieLoader::wait_for_frame(size_t framenum)
-{
-#ifndef LOAD_MOVIES_IN_A_SEPARATE_THREAD
-	assert(0); // don't call wait_for_frame !
-#endif
-
-	// lock the loader so we can rely on m_loading_frame
-	lock();
-
-	if ( _movie_def.get_loading_frame() < framenum )
-	{
-		assert(_waiting_for_frame == 0);
-		_waiting_for_frame = framenum;
-		pthread_cond_wait(&_frame_reached_condition, &_mutex);
-		_waiting_for_frame = 0;
-	}
-
-	unlock();
-}
-
 //
 // some utility stuff
 //
+// FIXME, this needs work now that log_msg always ends with a newline.
 
 void	dump_tag_bytes(stream* in)
     // Log the contents of the current tag, in hex.
@@ -232,7 +156,7 @@ void	dump_tag_bytes(stream* in)
             if (c < 32) c = '.';
             if (c > 127) c = '.';
             row_buf[row_count] = c;
-				
+
             row_count++;
             if (row_count >= ROW_BYTES)
                 {
@@ -259,7 +183,7 @@ void	dump_tag_bytes(stream* in)
 
 
 //
-// progress callback stuff (from Vitaly)
+// progress callback stuff
 //
 progress_callback	s_progress_function = NULL;
 
@@ -278,20 +202,30 @@ register_progress_callback(progress_callback progress_handle)
 movie_def_impl::movie_def_impl(create_bitmaps_flag cbf,
 		create_font_shapes_flag cfs)
 	:
-	_tag_loaders(s_tag_loaders), // FIXME: use a class-static TagLoadersTable for movie_def_impl
+	// FIXME: use a class-static TagLoadersTable for movie_def_impl
+	_tag_loaders(SWF::TagLoadersTable::getInstance()),
 	m_create_bitmaps(cbf),
 	m_create_font_shapes(cfs),
 	m_frame_rate(30.0f),
 	m_frame_count(0u),
 	m_version(0),
-	m_loading_frame(0u),
+	_frames_loaded(0u),
+	_frames_loaded_mutex(),
+	_frame_reached_condition(),
+	_waiting_for_frame(0),
 	m_jpeg_in(0),
-	_loader(*this)
+	_loader(*this),
+	_loadingCanceled(false)
 {
 }
 
 movie_def_impl::~movie_def_impl()
 {
+	//cout << "movie_def_impl dtor called" << endl;
+
+	// Request cancelation of the loading thread
+	_loadingCanceled = true;
+
     // Release our playlist data.
     {for (size_t i = m_playlist.size() - 1; i != static_cast<size_t>(-1); i--) // Optimized
         {
@@ -300,7 +234,7 @@ movie_def_impl::~movie_def_impl()
                     delete m_playlist[i][j];
                 }
         }}
-	
+
     // Release init action data.
     {for (size_t i = m_init_action_list.size() - 1; i != static_cast<size_t>(-1); i--) //Optimized
         {
@@ -309,13 +243,13 @@ movie_def_impl::~movie_def_impl()
                     delete m_init_action_list[i][j];
                 }
         }}
-		
+
 	// It's supposed to be cleaned up in read()
 	// TODO: join with loader thread instead ?
 	//assert(m_jpeg_in.get() == NULL);
 }
 
-bool movie_def_impl::in_import_table(int character_id)
+bool movie_def_impl::in_import_table(int character_id) const
 {
     for (size_t i = 0, n = m_imports.size(); i < n; i++)
         {
@@ -327,29 +261,24 @@ bool movie_def_impl::in_import_table(int character_id)
     return false;
 }
 
-void movie_def_impl::visit_imported_movies(import_visitor* visitor)
+void movie_def_impl::visit_imported_movies(import_visitor& visitor)
 {
-    stringi_hash<bool>	visited;	// ugh!
+    // don't call the visitor twice for a single URL
+    std::set<std::string> visited;
 
     for (size_t i = 0, n = m_imports.size(); i < n; i++)
+    {
+        const import_info& inf = m_imports[i];
+        if (visited.insert(inf.m_source_url).second)
         {
-            const import_info&	inf = m_imports[i];
-            if (visited.find(inf.m_source_url) == visited.end())
-                {
-                    // Call back the visitor.
-                    visitor->visit(inf.m_source_url.c_str());
-                    visited[inf.m_source_url] = true;
-                }
+            // Call back the visitor.
+            visitor.visit(inf.m_source_url);
         }
+    }
 }
 
-void movie_def_impl::resolve_import(const char* source_url, movie_definition* source_movie)
+void movie_def_impl::resolve_import(const std::string& source_url, movie_definition* source_movie)
 {
-    // @@ should be safe, but how can we verify
-    // it?  Compare a member function pointer, or
-    // something?
-    movie_def_impl*	def_impl = static_cast<movie_def_impl*>(source_movie);
-    movie_definition*	def = static_cast<movie_definition*>(def_impl);
 
     // Iterate in reverse, since we remove stuff along the way.
     for (size_t i = m_imports.size(); i > 0; i--)
@@ -358,13 +287,13 @@ void movie_def_impl::resolve_import(const char* source_url, movie_definition* so
             if (inf.m_source_url == source_url)
                 {
                     // Do the import.
-                    smart_ptr<resource> res = def->get_exported_resource(inf.m_symbol);
-                    bool	 imported = true;
+                    boost::intrusive_ptr<resource> res = source_movie->get_exported_resource(inf.m_symbol);
+                    bool	 imported = false;
 
                     if (res == NULL)
                         {
-                            log_error("import error: resource '%s' is not exported from movie '%s'\n",
-                                      inf.m_symbol.c_str(), source_url);
+                            log_error(_("import error: resource '%s' is not exported from movie '%s'"),
+                                      inf.m_symbol.c_str(), source_url.c_str());
                         }
                     else if (font* f = res->cast_to_font())
                         {
@@ -380,12 +309,13 @@ void movie_def_impl::resolve_import(const char* source_url, movie_definition* so
                         }
                     else
                         {
-                            log_error("import error: resource '%s' from movie '%s' has unknown type\n",
-                                      inf.m_symbol.c_str(), source_url);
+                            log_error(_("import error: resource '%s' from movie '%s' has unknown type"),
+                                      inf.m_symbol.c_str(), source_url.c_str());
                         }
 
                     if (imported)
                         {
+				// TODO: a std::list would be faster here
                             m_imports.erase(m_imports.begin() + i);
 
                             // Hold a ref, to keep this source movie_definition alive.
@@ -408,45 +338,47 @@ movie_def_impl::get_character_def(int character_id)
     // make sure character_id is resolved
     if (in_import_table(character_id))
         {
-            log_error("get_character_def(): character_id %d is still waiting to be imported\n",
+            log_error(_("get_character_def(): character_id %d is still waiting to be imported"),
                       character_id);
         }
 #endif // not NDEBUG
 
-	smart_ptr<character_def> ch = _dictionary.get_character(character_id);
+	boost::intrusive_ptr<character_def> ch = _dictionary.get_character(character_id);
+#ifndef GNASH_USE_GC
 	assert(ch == NULL || ch->get_ref_count() > 1);
-	return ch.get_ptr(); // mm... why don't we return the smart_ptr?
+#endif // ndef GNASH_USE_GC
+	return ch.get(); // mm... why don't we return the boost::intrusive_ptr?
 }
 
 void movie_def_impl::add_font(int font_id, font* f)
 {
     assert(f);
-    m_fonts.add(font_id, f);
+    m_fonts.insert(make_pair(font_id, boost::intrusive_ptr<font>(f)));
 }
 
-font* movie_def_impl::get_font(int font_id)
+font* movie_def_impl::get_font(int font_id) const
 {
 #ifndef NDEBUG
     // make sure font_id is resolved
     if (in_import_table(font_id))
         {
-            log_error("get_font(): font_id %d is still waiting to be imported\n",
+            log_error(_("get_font(): font_id %d is still waiting to be imported"),
                       font_id);
         }
 #endif // not NDEBUG
 
-    smart_ptr<font>	f;
-    m_fonts.get(font_id, &f);
-    assert(f == NULL || f->get_ref_count() > 1);
-    return f.get_ptr();
+    FontMap::const_iterator it = m_fonts.find(font_id);
+    if ( it == m_fonts.end() ) return NULL;
+    boost::intrusive_ptr<font> f = it->second;
+    assert(f->get_ref_count() > 1);
+    return f.get();
 }
 
 bitmap_character_def* movie_def_impl::get_bitmap_character_def(int character_id)
 {
-    smart_ptr<bitmap_character_def>	ch;
-    m_bitmap_characters.get(character_id, &ch);
-    assert(ch == NULL || ch->get_ref_count() > 1);
-    return ch.get_ptr();
+    BitmapMap::iterator it = m_bitmap_characters.find(character_id);
+    if ( it == m_bitmap_characters.end() ) return NULL;
+    else return it->second.get();
 }
 
 void
@@ -454,8 +386,9 @@ movie_def_impl::add_bitmap_character_def(int character_id,
 		bitmap_character_def* ch)
 {
     assert(ch);
-    //log_msg("Add bitmap character %d", character_id);
-    m_bitmap_characters.add(character_id, ch);
+    //log_msg(_("Add bitmap character %d"), character_id);
+    //m_bitmap_characters.add(character_id, ch);
+    m_bitmap_characters.insert(make_pair(character_id, boost::intrusive_ptr<bitmap_character_def>(ch)));
 
 	// we can *NOT* generate bitmap_info until
 	// a renderer is present
@@ -464,34 +397,44 @@ movie_def_impl::add_bitmap_character_def(int character_id,
 
 sound_sample* movie_def_impl::get_sound_sample(int character_id)
 {
-    smart_ptr<sound_sample>	ch;
-    m_sound_samples.get(character_id, &ch);
-    assert(ch == NULL || ch->get_ref_count() > 1);
-    return ch.get_ptr();
+    SoundSampleMap::iterator it = m_sound_samples.find(character_id);
+    if ( it == m_sound_samples.end() ) return NULL;
+
+    boost::intrusive_ptr<sound_sample> ch = it->second;
+#ifndef GNASH_USE_GC
+    assert(ch->get_ref_count() > 1);
+#endif // ndef GNASH_USE_GC
+
+    return ch.get();
 }
 
 void movie_def_impl::add_sound_sample(int character_id, sound_sample* sam)
 {
     assert(sam);
-	log_msg("Add sound sample %d", character_id);
-    m_sound_samples.add(character_id, sam);
+    IF_VERBOSE_PARSE(
+    log_parse(_("Add sound sample %d assigning id %d"),
+		character_id, sam->m_sound_handler_id);
+    );
+    m_sound_samples.insert(make_pair(character_id,
+			    boost::intrusive_ptr<sound_sample>(sam)));
 }
 
-
-// Read a .SWF movie.
+// Read header and assign url
 bool
-movie_def_impl::read(tu_file* in, const std::string& url)
+movie_def_impl::readHeader(std::auto_ptr<tu_file> in, const std::string& url)
 {
 
-	// we only read a movie once (well, headers at least)
+	_in = in;
+
+	// we only read a movie once
 	assert(_str.get() == NULL);
 
 	if ( url == "" ) _url = "<anonymous>";
 	else _url = url;
 
-	uint32_t file_start_pos = in->get_position();
-	uint32_t header = in->read_le32();
-	m_file_length = in->read_le32();
+	uint32_t file_start_pos = _in->get_position();
+	uint32_t header = _in->read_le32();
+	m_file_length = _in->read_le32();
 	_swf_end_pos = file_start_pos + m_file_length;
 
 	m_version = (header >> 24) & 255;
@@ -499,74 +442,98 @@ movie_def_impl::read(tu_file* in, const std::string& url)
 		&& (header & 0x0FFFFFF) != 0x00535743)
         {
 		// ERROR
-		log_error("gnash::movie_def_impl::read() -- "
-			"file does not start with a SWF header!\n");
+		log_error(_("gnash::movie_def_impl::read() -- "
+			"file does not start with a SWF header"));
 		return false;
         }
 	bool	compressed = (header & 255) == 'C';
-    
+
 	IF_VERBOSE_PARSE(
-		log_parse("version = %d, file_length = %d",
+		log_parse(_("version = %d, file_length = %d"),
 			m_version, m_file_length);
 	);
 
 	if ( m_version > 7 )
 	{
-		log_warning("SWF%d is not fully supported, trying anyway "
-			"but don't expect it to work", m_version);
+		log_unimpl(_("SWF%d is not fully supported, trying anyway "
+			"but don't expect it to work"), m_version);
 	}
 
-	tu_file* original_in = NULL;
 	if (compressed)
         {
-#if TU_CONFIG_LINK_TO_ZLIB == 0
-		log_error("movie_def_impl::read(): unable to read "
-			"zipped SWF data; TU_CONFIG_LINK_TO_ZLIB is 0\n");
+#ifndef HAVE_ZLIB_H
+		log_error(_("movie_def_impl::read(): unable to read "
+			"zipped SWF data; gnash was compiled without zlib support"));
 		return false;
-#endif
-
+#else
 		IF_VERBOSE_PARSE(
-			log_parse("file is compressed.");
+			log_parse(_("file is compressed"));
 		);
 
-		original_in = in;
-
 		// Uncompress the input as we read it.
-		_zlib_file.reset(zlib_adapter::make_inflater(original_in));
-		in = _zlib_file.get();
-
+		_in = zlib_adapter::make_inflater(_in);
+#endif
         }
 
-	//stream str(in);
-	_str.reset(new stream(in));
+	assert(_in.get());
+
+	_str.reset(new stream(_in.get()));
 
 	m_frame_size.read(_str.get());
+	// If the rect is malformed, rect::read would already 
+	// print an error. We check again here just to give 
+	// the error are better context.
+	if ( m_frame_size.is_null() )
+	{
+		IF_VERBOSE_MALFORMED_SWF(
+		log_swferror("non-finite movie bounds");
+		);
+	}
 	m_frame_rate = _str->read_u16() / 256.0f;
 	m_frame_count = _str->read_u16();
 
-	// hack
-	// Vitaly: I am not assured that it is correctly
-	//m_frame_count = (m_frame_count == 0) ? 1 : m_frame_count;
-
 	/* Markus: Probably this is better anyways */
-	
-	if(m_frame_count == 0)m_frame_count++;
-	
-	m_playlist.resize(m_frame_count);
-	m_init_action_list.resize(m_frame_count);
+
+	// TODO: This seems dangerous, check closely
+	if(m_frame_count == 0) m_frame_count++;
+
+	// Allocate 1 more then the expected slots
+	// for actions, to make handling malformed SWF easier.
+	m_playlist.resize(m_frame_count+1);
+	m_init_action_list.resize(m_frame_count+1);
 
 	IF_VERBOSE_PARSE(
 		m_frame_size.print();
-		log_parse("frame rate = %f, frames = " SIZET_FMT,
+		log_parse(_("frame rate = %f, frames = " SIZET_FMT),
 			m_frame_rate, m_frame_count);
 	);
 
+	setBytesLoaded(_str->get_position());
+	return true;
+}
+
+// Fire up the loading thread
+bool
+movie_def_impl::completeLoad()
+{
+
+	// should call this only once
+	assert( ! _loader.started() );
+
+	// The VM is needed by the parser
+	// to allocate swf_function objects !
+	assert ( VM::isInitialized() );
+
+	// should call readHeader before this
+	assert( _str.get() != NULL );
+
 #ifdef LOAD_MOVIES_IN_A_SEPARATE_THREAD
 
 	// Start the loading frame
 	if ( ! _loader.start() )
 	{
-		log_error("Could not start loading thread");
+		log_error(_("Could not start loading thread"));
+		return false;
 	}
 
 	// Wait until 'startup_frames' have been loaded
@@ -582,35 +549,41 @@ movie_def_impl::read(tu_file* in, const std::string& url)
 	read_all_swf();
 #endif
 
-// Can't delete here as we will keep reading from it while playing
-// FIXME: remove this at end of reading (or in destructor)
-#if 0
-	if (original_in)
-        {
-            // Done with the zlib_adapter.
-            delete in;
-        }
-#endif
-
 	return true;
 }
 
+// Read a .SWF movie.
+bool
+movie_def_impl::read(std::auto_ptr<tu_file> in, const std::string& url)
+{
+
+	if ( ! readHeader(in, url) ) return false;
+
+	return completeLoad();
+}
+
 
 // 1-based frame number
 bool
 movie_def_impl::ensure_frame_loaded(size_t framenum)
 {
-#ifdef LOAD_MOVIES_IN_A_SEPARATE_THREAD 
-        //log_msg("Waiting for frame %u to be loaded", framenum);
-	_loader.wait_for_frame(framenum);
-        //log_msg("Condition reached (m_loading_frame=%u)", m_loading_frame);
-#else
-	assert ( framenum <= m_loading_frame );
+#ifndef LOAD_MOVIES_IN_A_SEPARATE_THREAD
+	return ( framenum <= _frames_loaded );
 #endif
 
+	boost::mutex::scoped_lock lock(_frames_loaded_mutex);
 
-	// TODO: return false on timeout 
-	return true;
+	if ( framenum <= _frames_loaded ) return true;
+
+	_waiting_for_frame = framenum;
+        //log_msg(_("Waiting for frame %u to be loaded"), framenum);
+
+	// TODO: return false on timeout
+	_frame_reached_condition.wait(lock);
+
+        //log_msg(_("Condition reached (_frames_loaded=%u)"), _frames_loaded);
+
+	return ( framenum <= _frames_loaded );
 }
 
 
@@ -623,11 +596,10 @@ void movie_def_impl::get_owned_fonts(std::vector<font*>* fonts)
 
     std::vector<int>	font_ids;
 
-    for (hash<int, smart_ptr<font> >::iterator it = m_fonts.begin();
-         it != m_fonts.end();
-         ++it)
+    for (FontMap::iterator it = m_fonts.begin(), itEnd=m_fonts.end();
+         it != itEnd; ++it)
         {
-            font*	f = it->second.get_ptr();
+            font*	f = it->second.get();
             if (f->get_owning_movie() == this)
                 {
                     // Sort by character id, so the ordering is
@@ -688,9 +660,9 @@ movie_def_impl::output_cached_data(tu_file* out, const cache_options& options)
 		out->write_le16(it->first);
 		it->second->output_cached_data(out, options);
 	}
-			
+
 #if 0
-	for (hash<int, smart_ptr<character_def> >::iterator it = m_characters.begin();
+	for (hash<int, boost::intrusive_ptr<character_def> >::iterator it = m_characters.begin();
           it != m_characters.end();
           ++it)
         {
@@ -712,13 +684,13 @@ movie_def_impl::input_cached_data(tu_file* in)
     in->read_bytes(header, 4);
     if (header[0] != 'g' || header[1] != 's' || header[2] != 'c')
         {
-            log_error("cache file does not have the correct format; skipping\n");
+            log_error(_("cache file does not have the correct format; skipping"));
             return;
         }
     else if (header[3] != CACHE_FILE_VERSION)
         {
             log_error(
-                "cached data is version %d, but we require version %d; skipping\n",
+                _("cached data is version %d, but we require version %d; skipping"),
                 int(header[3]), CACHE_FILE_VERSION);
             return;
         }
@@ -733,19 +705,19 @@ movie_def_impl::input_cached_data(tu_file* in)
         {
             if (in->get_error() != TU_FILE_NO_ERROR)
                 {
-                    log_error("error reading cache file (characters); skipping\n");
+                    log_error(_("error reading cache file (characters); skipping"));
                     return;
                 }
             if (in->get_eof())
                 {
-                    log_error("unexpected eof reading cache file (characters); skipping\n");
+                    log_error(_("unexpected eof reading cache file (characters); skipping"));
                     return;
                 }
 
             int16_t	id = in->read_le16();
             if (id == (int16_t) -1) { break; }	// done
 
-            smart_ptr<character_def> ch = _dictionary.get_character(id);
+            boost::intrusive_ptr<character_def> ch = _dictionary.get_character(id);
             //m_characters.get(id, &ch);
             if (ch != NULL)
                 {
@@ -753,38 +725,27 @@ movie_def_impl::input_cached_data(tu_file* in)
                 }
             else
                 {
-                    log_error("sync error in cache file (reading characters)!  "
-                              "Skipping rest of cache data.\n");
+                    log_error(_("sync error in cache file (reading characters).  "
+                              "Skipping rest of cache data."));
                     return;
                 }
         }
 }
 
-movie_interface*
+sprite_instance*
 movie_def_impl::create_instance()
 {
+	return create_movie_instance();
+}
+
+movie_instance*
+movie_def_impl::create_movie_instance()
+{
+
 	// @@ Shouldn't we return a movie_instance instead ?
 	// @@ and leave movie_root creation to the caller ..
 
-    movie_root*	m = new movie_root(this);
-
-    movie_instance* root_movie = new movie_instance(this, m, NULL);
-
-    root_movie->set_name("_root");
-    m->set_root_movie(root_movie);
-
-    m->add_ref();
-
-	// I don't think we should be executing actions
-	// in first frame from *this* function, rather
-	// it should be the advance() function taking
-	// care of it... anyway, since we do, better
-	// to ensure the frame number 1 is loaded before
-	// messing with it.
-	ensure_frame_loaded(1);
-	root_movie->execute_frame_tags(0); // create _root dlist
-
-    return m;
+	return new movie_instance(this, NULL);
 }
 
 
@@ -798,30 +759,30 @@ CharacterDictionary::dump_chars() const
 	for ( const_iterator it=begin(), endIt=end();
 		it != endIt; ++it )
 	{
-		log_msg("Character %d @ %p", it->first, static_cast<void*>(it->second.get_ptr()));
+		log_msg(_("Character %d @ %p"), it->first, static_cast<void*>(it->second.get()));
 		//character_def* cdef = it->second;
 	}
 }
 
-smart_ptr<character_def>
+boost::intrusive_ptr<character_def>
 CharacterDictionary::get_character(int id)
 {
 	container::iterator it = _map.find(id);
 	if ( it == _map.end() )
 	{
 		IF_VERBOSE_PARSE(
-		log_parse("Could not find char %d, dump is:", id);
+		log_parse(_("Could not find char %d, dump is:"), id);
 		dump_chars();
 		);
-		return smart_ptr<character_def>();
+		return boost::intrusive_ptr<character_def>();
 	}
 	else return it->second;
 }
 
 void
-CharacterDictionary::add_character(int id, smart_ptr<character_def> c)
+CharacterDictionary::add_character(int id, boost::intrusive_ptr<character_def> c)
 {
-	//log_msg("CharacterDictionary: add char %d", id);
+	//log_msg(_("CharacterDictionary: add char %d"), id);
 	_map[id] = c;
 	//dump_chars();
 }
@@ -841,19 +802,19 @@ movie_def_impl::load_next_frame_chunk()
 	size_t nextframe = lastloaded+1;
 
 #if FRAMELOAD_CHUNK
-	nextframe += FRAMELOAD_CHUNK; // load in chunks of 10 frames 
+	nextframe += FRAMELOAD_CHUNK; // load in chunks of 10 frames
 	if ( nextframe > framecount ) nextframe = framecount;
 #endif
-	//log_msg("Framecount: %u, Lastloaded: %u", framecount, lastloaded);
+	//log_msg(_("Framecount: %u, Lastloaded: %u"), framecount, lastloaded);
 	if ( nextframe <= framecount )
 	{
 #ifdef DEBUG_FRAMES_LOAD // debugging
-		log_msg("Ensure load of frame %u/%u (last loaded is: %u)",
+		log_msg(_("Ensure load of frame %u/%u (last loaded is: %u)"),
 			nextframe, framecount, lastloaded);
 #endif
 		if ( ! ensure_frame_loaded(nextframe) )
 		{
-			log_error("Could not advance to frame " SIZET_FMT "!",
+			log_error(_("Could not advance to frame " SIZET_FMT),
 				nextframe);
 			// these kind of errors should be handled by callers
 			assert(0);
@@ -862,7 +823,7 @@ movie_def_impl::load_next_frame_chunk()
 #ifdef DEBUG_FRAMES_LOAD
 	else
 	{
-		log_msg("No more frames to load. Framecount: %u, Lastloaded: %u, next to load: %u", framecount, lastloaded, nextframe);
+		log_msg(_("No more frames to load. Framecount: %u, Lastloaded: %u, next to load: %u"), framecount, lastloaded, nextframe);
 	}
 #endif
 }
@@ -872,25 +833,54 @@ movie_def_impl::read_all_swf()
 {
 	assert(_str.get() != NULL);
 
+#ifdef LOAD_MOVIES_IN_A_SEPARATE_THREAD
+	assert( _loader.isSelfThread() );
+	assert( _loader.started() );
+#else
+	assert( ! _loader.started() );
+	assert( ! _loader.isSelfThread() );
+#endif
+
 	stream &str = *_str;
 
+	try {
+
 	//size_t it=0;
 	while ( (uint32_t) str.get_position() < _swf_end_pos )
 	{
-		// Get exclusive lock on loader, to avoid
-		// race conditions with wait_for_frame
-		_loader.lock();
-	
-		//log_msg("Loading thread iteration %u", it++);
+		if ( _loadingCanceled )
+		{
+			log_debug("Loading thread cancelation requested, returning from read_all_swf");
+			return;
+		}
+
+		//log_msg(_("Loading thread iteration %u"), it++);
 
 		SWF::tag_type tag_type = str.open_tag();
 
+parse_tag:
+
 		if (s_progress_function != NULL)
                 {
 			s_progress_function((uint32_t)str.get_position(),
 				_swf_end_pos);
                 }
 
+		if (tag_type == SWF::END)
+                {
+			if ((unsigned int) str.get_position() != _swf_end_pos)
+                        {
+		    		IF_VERBOSE_MALFORMED_SWF(
+				// Safety break, so we don't read past
+				// the end of the  movie.
+				log_swferror(_("Hit stream-end tag, "
+					"but not at the advertised SWF end; "
+					"stopping for safety."));
+		    		)
+		    		break;
+			}
+		}
+
 		SWF::TagLoadersTable::loader_function lf = NULL;
 		//log_parse("tag_type = %d\n", tag_type);
 		if (tag_type == SWF::SHOWFRAME)
@@ -901,15 +891,23 @@ movie_def_impl::read_all_swf()
 				log_parse("  show_frame\n");
 			);
 
-			++m_loading_frame;
+			incrementLoadedFrames();
+			if ( _frames_loaded == m_frame_count )
+			{
+				str.close_tag();
+				tag_type = str.open_tag();
+				if ( tag_type != SWF::END )
+				{
+					IF_VERBOSE_MALFORMED_SWF(
+					log_swferror(_("last expected SHOWFRAME "
+						"in SWF stream '%s' isn't "
+						"followed by an END (%d)."),
+						get_url().c_str(), tag_type);
+					);
+				}
+				goto parse_tag;
+			}
 
-			// signal load of frame
-			_loader.signal_frame_loaded(m_loading_frame);
-
-#ifdef DEBUG_FRAMES_LOAD
-			log_msg("Loaded frame %u/%u",
-				m_loading_frame, m_frame_count);
-#endif
 		}
 		else if (_tag_loaders.get(tag_type, &lf))
                 {
@@ -920,35 +918,100 @@ movie_def_impl::read_all_swf()
 		else
 		{
 			// no tag loader for this tag type.
-			log_error("*** no tag loader for type %d (movie)",
+			log_error(_("*** no tag loader for type %d (movie)"),
 				tag_type);
 			IF_VERBOSE_PARSE(
 				dump_tag_bytes(&str);
 			);
-		} 
+		}
 
 		str.close_tag();
 
-		if (tag_type == SWF::END)
-                {
-			if ((unsigned int) str.get_position() != _swf_end_pos)
-                        {
-				// Safety break, so we don't read past
-				// the end of the  movie.
-				log_warning("hit stream-end tag, "
-					"but not at the advertised SWF end; "
-					"stopping for safety.");
-				_loader.unlock();
-				break;
-			}
-		}
-		_loader.unlock();
+		setBytesLoaded(str.get_position());
 	}
 
+	} catch (const std::exception& e) {
+		// FIXME: we should be setting some variable
+		//        so that it is possible for clients
+		//        to check the parser status
+		//        Also, we should probably call _loader.unlock()
+		//        and make sure any wait_for_frame call is
+		//        released (condition set and false result)
+		log_error(_("Parsing exception: %s"), e.what());
+	}
+
+	if ( ! m_playlist[_frames_loaded].empty() || ! m_init_action_list[_frames_loaded].empty() )
+	{
+		IF_VERBOSE_MALFORMED_SWF(
+		log_swferror(_(SIZET_FMT " action blocks and " SIZET_FMT " init action blocks are NOT followed by"
+			" a SHOWFRAME tag"),
+			m_playlist[_frames_loaded].size(),
+			m_init_action_list[_frames_loaded].size());
+		);
+	}
+
+	if ( m_frame_count > _frames_loaded )
+	{
+		IF_VERBOSE_MALFORMED_SWF(
+		log_swferror(_(SIZET_FMT " frames advertised in header, but only " SIZET_FMT " SHOWFRAME tags "
+			"found in stream. Updating total frames count"), m_frame_count, _frames_loaded);
+		);
+		boost::mutex::scoped_lock lock(_frames_loaded_mutex);
+		m_frame_count = _frames_loaded;
+		// Notify any thread waiting on frame reached condition
+		_frame_reached_condition.notify_all();
+	}
+}
+
+size_t
+movie_def_impl::get_loading_frame() const
+{
+	boost::mutex::scoped_lock lock(_frames_loaded_mutex);
+	return _frames_loaded;
 }
 
 void
-movie_def_impl::export_resource(const tu_string& symbol, resource* res)
+movie_def_impl::incrementLoadedFrames()
+{
+	boost::mutex::scoped_lock lock(_frames_loaded_mutex);
+
+	++_frames_loaded;
+
+	// Close current frame definition in Timeline object
+	_timeline.closeFrame();
+
+	if ( _frames_loaded > m_frame_count )
+	{
+		IF_VERBOSE_MALFORMED_SWF(
+			log_swferror(_("number of SHOWFRAME tags "
+				"in SWF stream '%s' (" SIZET_FMT
+				") exceeds "
+				"the advertised number in header ("
+			        SIZET_FMT ")."),
+				get_url().c_str(), _frames_loaded,
+				m_frame_count);
+		);
+		m_playlist.resize(_frames_loaded+1);
+		m_init_action_list.resize(_frames_loaded+1);
+	}
+
+#ifdef DEBUG_FRAMES_LOAD
+	log_msg(_("Loaded frame %u/%u"),
+		_frames_loaded, m_frame_count);
+#endif
+
+	// signal load of frame if anyone requested it
+	// FIXME: _waiting_form_frame needs mutex ?
+	if (_waiting_for_frame && _frames_loaded >= _waiting_for_frame )
+	{
+		// or should we notify_one ?
+		// See: http://boost.org/doc/html/condition.html
+		_frame_reached_condition.notify_all();
+	}
+}
+
+void
+movie_def_impl::export_resource(const std::string& symbol, resource* res)
 {
 	// FIXME: m_exports access should be protected by a mutex
 
@@ -957,77 +1020,152 @@ movie_def_impl::export_resource(const tu_string& symbol, resource* res)
 }
 
 
-smart_ptr<resource>
-movie_def_impl::get_exported_resource(const tu_string& symbol)
+boost::intrusive_ptr<resource>
+movie_def_impl::get_exported_resource(const std::string& symbol)
 {
-	smart_ptr<resource> res;
-
-//#define DEBUG_EXPORTS
+	boost::intrusive_ptr<resource> res;
 
 #ifdef DEBUG_EXPORTS
-	log_msg("get_exported_resource called, frame count=%u", m_frame_count);
+	log_msg(_("get_exported_resource called, frame count=%u"), m_frame_count);
 #endif
 
-	// FIXME: a movie importing from itself will likely
-	//        end up in a dead lock
-
-	// this is a simple utility so we don't forget
-	// to release our locks...
-	struct scoped_loader_locker {
-		MovieLoader& _loader;
-		scoped_loader_locker(MovieLoader& loader)
-			:
-			_loader(loader)
-		{
-			_loader.lock();
-		}
-		~scoped_loader_locker()
-		{
-			_loader.unlock();
-		}
-	};
+	// Don't call get_exported_resource() from this movie loader
+	assert( ! _loader.isSelfThread() );
 
 	// Keep trying until either we found the export or
-	// the stream is over.
-	bool found=false;
+	// the stream is over, or there is NO frames progress
+	// after def_timeout microseconds.
+	//
+	// Note that the NO frame progress might be due
+	// to a circular import chain:
+	//
+	// 	A imports B imports A
+	//
+
+	// Sleep 1/2 of a second between checks
+	// NOTE: make sure the nap is enough time for
+	//       thread execution switch !!
+	const unsigned long naptime=500000;
+
+	// Timeout after two seconds of NO frames progress
+	const unsigned long def_timeout=2000000/naptime;
+
+	unsigned long timeout=def_timeout;
+	size_t loading_frame = (size_t)-1; // used to keep track of advancements
 	for (;;)
 	{
+		// FIXME: make m_exports access thread-safe
+        ExportMap::iterator it = m_exports.find(symbol);
+        if ( it != m_exports.end() ) return it->second;
+
+		size_t new_loading_frame = get_loading_frame();
+
+		if ( new_loading_frame != loading_frame )
 		{
-		// lock the loader
-		scoped_loader_locker locker(_loader);
-	
-		if ( m_exports.get(symbol, &res) )
+			log_msg(_("frame load advancement (from "
+				SIZET_FMT " to " SIZET_FMT ")"),
+				loading_frame, new_loading_frame);
+			loading_frame = new_loading_frame;
+			timeout = def_timeout;
+		}
+		else
 		{
-			found=true;
-			break;
+			if ( ! timeout-- )
+			{
+				log_error(_("No frame progress in movie %s "
+					"after %lu milliseconds "
+					"(%lu microseconds = %lu iterations), "
+					"giving up on "
+					"get_exported_resource(%s): "
+					"circular IMPORTS?"),
+					get_url().c_str(),
+					(def_timeout*naptime)/1000,
+					def_timeout*naptime,
+					def_timeout,
+					symbol.c_str());
+				return res;
+			}
+
+			log_error(_("no frame progress at iteration %lu"), timeout);
+
+			continue; // not worth checking
 		}
 
-		// be aware of not getting the lock twice
-		// (can happen if get_loading_frame() becomes
-		//  a locking function)
-		if ( get_loading_frame() >= m_frame_count ) break;
+		if ( loading_frame >= m_frame_count )
+		{
+			log_error(_("At end of stream, still no '%s' symbol found "
+				"in m_exports (" SIZET_FMT " entries in it, "
+				"follow)"), symbol.c_str(), m_exports.size());
+			return res;
+		}
 
 #ifdef DEBUG_EXPORTS
-		log_msg("We haven't finished loading (loading frame %u), "
+		log_msg(_("We haven't finished loading (loading frame %u), "
 			"and m_exports.get returned no entries, "
-			"sleeping a bit and trying again",
+			"sleeping a bit and trying again"),
 			get_loading_frame());
 #endif
-		}
 
-		usleep(100); // take a breath
-	}
-
-	if ( ! found )
-	{
-		log_msg("At end of stream, still no '%s' symbol found "
-			"in m_exports (%u entries in it)",
-			symbol.c_str(), m_exports.size());
+		usleep(naptime); // take a breath
 	}
 
 	return res;
 }
 
+void
+movie_def_impl::add_frame_name(const std::string& n)
+{
+	//log_msg(_("labelframe: frame %d, name %s"), _frames_loaded, name);
+	assert(_frames_loaded < m_frame_count);
+    m_named_frames[n] = _frames_loaded;
+}
+
+bool
+movie_def_impl::get_labeled_frame(const std::string& label, size_t& frame_number)
+{
+    NamedFrameMap::iterator it = m_named_frames.find(label);
+    if ( it == m_named_frames.end() ) return false;
+    frame_number = it->second;
+    return true;
+}
+
+#ifdef GNASH_USE_GC
+void
+movie_def_impl::markReachableResources() const
+{
+	for (FontMap::const_iterator i=m_fonts.begin(), e=m_fonts.end(); i!=e; ++i)
+	{
+		i->second->setReachable();
+	}
+
+	for (BitmapMap::const_iterator i=m_bitmap_characters.begin(), e=m_bitmap_characters.end(); i!=e; ++i)
+	{
+		i->second->setReachable();
+	}
+
+	for (BitmapVect::const_iterator i=m_bitmap_list.begin(), e=m_bitmap_list.end(); i!=e; ++i)
+	{
+		(*i)->setReachable();
+	}
+
+	for (SoundSampleMap::const_iterator i=m_sound_samples.begin(), e=m_sound_samples.end(); i!=e; ++i)
+	{
+		i->second->setReachable();
+	}
+
+	for (ExportMap::const_iterator i=m_exports.begin(), e=m_exports.end(); i!=e; ++i)
+	{
+		i->second->setReachable();
+	}
+
+	for (ImportVect::const_iterator i=m_import_source_movies.begin(), e=m_import_source_movies.end(); i!=e; ++i)
+	{
+		(*i)->setReachable();
+	}
+
+	_dictionary.markReachableResources();
+
+}
+#endif // GNASH_USE_GC
 
 } // namespace gnash
-
