@@ -18,7 +18,7 @@
 #include <cassert>
 
 namespace gnash {
-	static render_handler* s_render_handler;
+	static render_handler* s_render_handler = NULL;
 
 	void set_render_handler(render_handler* r)
 	{
@@ -48,7 +48,6 @@ namespace gnash {
 			bogus_bi() {}
 		};
 
-
 		bitmap_info*	create_bitmap_info_alpha(int w, int h, unsigned char* data)
 		{
 #ifdef DEBUG_RENDER_CALLS
@@ -87,11 +86,32 @@ namespace gnash {
 			if (s_render_handler) s_render_handler->delete_bitmap_info(bi);
 		}
 
+		void get_scale(point& scale) {
+			if (s_render_handler) {
+				s_render_handler->get_scale(scale);
+			} else {
+				scale.m_x = 1.0;
+				scale.m_y = 1.0;
+			}
+		}
+
+	 	// Returns the format the current renderer wants videoframes in.
+		int videoFrameFormat() {
+			if (s_render_handler) return s_render_handler->videoFrameFormat();
+			else return NONE;
+		}
+		
+		// Draws the video frames
+		void drawVideoFrame(image::image_base* frame, const matrix* mat, const rect* bounds){
+			if (s_render_handler) return s_render_handler->drawVideoFrame(frame, mat, bounds);
+		}
+
+
 		// Bracket the displaying of a frame from a movie.
 		// Fill the background color, and set up default
 		// transforms, etc.
 		void	begin_display(
-			rgba background_color,
+			const rgba& background_color,
 			int viewport_x0, int viewport_y0,
 			int viewport_width, int viewport_height,
 			float x0, float x1, float y0, float y1)
@@ -139,7 +159,7 @@ namespace gnash {
 		}
 
 
-		void	draw_line_strip(const int16_t coords[], int vertex_count, const rgba color)
+		void	draw_line_strip(const int16_t coords[], int vertex_count, const rgba& color)
 		{
 #ifdef DEBUG_RENDER_CALLS
 			GNASH_REPORT_FUNCTION;
@@ -148,8 +168,8 @@ namespace gnash {
     }
 
 
-    void  draw_poly(const point* corners, int corner_count, const rgba fill, 
-      const rgba outline) 
+    void  draw_poly(const point* corners, int corner_count, const rgba& fill, 
+      const rgba& outline) 
 		{
 #ifdef DEBUG_RENDER_CALLS
 			GNASH_REPORT_FUNCTION;
@@ -170,7 +190,7 @@ namespace gnash {
     
     void draw_glyph(shape_character_def *def,
       const matrix& mat,
-      rgba color,
+      const rgba& color,
       float pixel_scale) 
     {
 #ifdef DEBUG_RENDER_CALLS
@@ -187,6 +207,28 @@ namespace gnash {
         return true;
     }
 
+    bool bounds_in_clipping_area(const rect& bounds) {
+    	return bounds_in_clipping_area(bounds.getRange());
+      if (s_render_handler) 
+        return s_render_handler->bounds_in_clipping_area(bounds);
+      else
+        return true;
+    }
+    
+    bool bounds_in_clipping_area(const InvalidatedRanges& ranges) {
+      if (s_render_handler) 
+        return s_render_handler->bounds_in_clipping_area(ranges);
+      else
+        return true;
+		}
+    
+    bool bounds_in_clipping_area(const geometry::Range2d<float>& bounds) {
+      if (s_render_handler) 
+        return s_render_handler->bounds_in_clipping_area(bounds);
+      else
+        return true;
+		}
+
 
 
 
@@ -217,7 +259,7 @@ namespace gnash {
 		// Special function to draw a rectangular bitmap;
 		// intended for textured glyph rendering.  Ignores
 		// current transforms.
-		void	draw_bitmap(const matrix& m, const bitmap_info* bi, const rect& coords, const rect& uv_coords, rgba color)
+		void	draw_bitmap(const matrix& m, const bitmap_info* bi, const rect& coords, const rect& uv_coords, const rgba& color)
 		{
 #ifdef DEBUG_RENDER_CALLS
 			GNASH_REPORT_FUNCTION;
