@@ -3,7 +3,7 @@
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
@@ -19,7 +19,7 @@
 //
 //
 
-/* $Id: tag_loaders.h,v 1.18 2007/05/28 15:41:10 ann Exp $ */
+/* $Id: tag_loaders.h,v 1.20 2007/08/08 18:26:38 strk Exp $ */
 
 #ifndef GNASH_SWF_TAG_LOADERS_H
 #define GNASH_SWF_TAG_LOADERS_H
@@ -78,6 +78,10 @@ void	define_shape_morph_loader(stream*, tag_type, movie_definition*);
 ///
 void	define_font_loader(stream*, tag_type, movie_definition*);
 
+/// SWF Tags Reflex (777)
+//
+void	reflex_loader(stream*, tag_type, movie_definition*);
+
 /// SWF Tag DefineFontInfo (13 or 62) 
 //
 /// Load a DefineFontInfo or DefineFontInfo2 tag. 
