@@ -3,7 +3,7 @@
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
@@ -43,11 +43,6 @@ namespace gnash {
 
 namespace gnash {
   
-  struct variable {
-    std::string name;
-    as_value value;
-  };
-
 /// An interval timer.
 //
 /// This is constructed when _global.setInterval() is called.
@@ -90,7 +85,7 @@ public:
       ///	It is allowed to be NULL as long as fn_call is allowed
       ///	a NULL as 'this_ptr' (we might want to change this).
       ///
-      void setInterval(as_function& method, unsigned ms, boost::intrusive_ptr<as_object> this_ptr);
+      void setInterval(as_function& method, uint64_t ms, boost::intrusive_ptr<as_object> this_ptr);
 
       /// Setup the Timer, enabling it.
       //
@@ -110,7 +105,7 @@ public:
       /// @param args
       /// 	The list of arguments to pass to the function being invoked.
       ///
-      void setInterval(as_function& method, unsigned ms, boost::intrusive_ptr<as_object> this_ptr, 
+      void setInterval(as_function& method, uint64_t ms, boost::intrusive_ptr<as_object> this_ptr, 
 		      std::vector<as_value>& args);
 
       /// Clear the timer, ready for reuse
@@ -128,12 +123,35 @@ public:
       //
       bool expired();
 
+      /// Return true if interval has been cleared.
+      //
+      /// Note that the timer is constructed as cleared and you
+      /// need to call setInterval() to make it not-cleared.
+      bool cleared() const { return ! _start; }
+
       /// Execute associated function properly setting up context
       void operator() ();
       
       /// Arguments list type
       typedef std::vector<as_value> ArgsContainer;
-      
+
+      /// Return number of microseconds between expirations 
+      uint64_t getInterval() const { return _interval; }
+
+      /// Return number of milliseconds after VM start this timer was last reset
+      uint64_t getStart() const { return _start; }
+
+#ifdef GNASH_USE_GC
+	/// Mark all reachable resources (for GC)
+	//
+	/// Resources reachable from Timer are:
+	///
+	///	- Arguments list (_args)
+	///	- Associated function (_function)
+	///	- Target object (_object)
+	///
+	void markReachableResources() const;
+#endif // GNASH_USE_GC
 
 private:
 
@@ -143,10 +161,10 @@ private:
       ///
       void start();
 
-      /// Number of microseconds between expirations 
+      /// Number of milliseconds between expirations 
       uint64_t _interval;
 
-      /// Number of microseconds since epoch at Timer start
+      /// Number of microseconds since epoch at Timer start (?)
       uint64_t _start;
 
       /// The associated function, stored in an intrusive pointer
