@@ -54,6 +54,37 @@ namespace gnash {
 		{
 		}
 
+		/// \brief
+		/// Parse a 32-bit unsigned integer
+		/// as three packed R,G,B bytes.
+		//
+		/// Alpha will be untouched.
+		/// Blue is the least significant byte.
+		///
+		/// This function is meant to be used to
+		/// parse ActionScript colors in numeric format.
+		///
+		void parseRGB(uint32_t rgbCol)
+		{
+			m_r = (uint8_t)(rgbCol>>16);
+			m_g = (uint8_t)(rgbCol>>8);
+			m_b = (uint8_t)(rgbCol);
+		}
+
+		/// \brief
+		/// Return a 32-bit unsigned integer
+		/// as four packed R,G,B bytes.
+		//
+		/// Blue is the least significant byte.
+		///
+		/// This function is meant to be used to
+		/// output ActionScript colors in numeric format.
+		///
+		uint32_t toRGB() const
+		{
+			return (m_r<<16) + (m_g<<8) + m_b;
+		}
+
 		/// Initialize from intput stream.
 		//
 		///
@@ -65,9 +96,16 @@ namespace gnash {
 		///	a value <= 22 makes it read RGB
 		///	and value > 22 makes it read RGBA
 		///
+		/// Throw a ParserException if there's no enough bytes in the
+		/// currently opened tag for reading. See stream::ensureBytes()
+		///
 		void	read(stream* in, int tag_type);
 
 		/// Initialize from intput stream (reads RGBA)
+		//
+		/// Throw a ParserException if there's no enough bytes in the
+		/// currently opened tag for reading. See stream::ensureBytes()
+		///
 		void	read_rgba(stream* in);
 
 		/// Initialize from intput stream (reads RGB)
@@ -92,6 +130,16 @@ namespace gnash {
 
 		// neater string output (example: "0,0,0,255")
 		std::string toShortString() const;
+
+		bool operator== (const rgba& o) const
+		{
+			return m_r == o.m_r && m_g == o.m_g && m_b == o.m_b && m_a == o.m_a;
+		}
+
+		bool operator!= (const rgba& o) const
+		{
+			return ! ( *this == o );
+		}
 	};
 
 	std::ostream& operator<< (std::ostream& os, const rgba& r);
